@@ -9,14 +9,22 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 #if defined(MPT_ENABLE_FILEIO)
 
+#include "mpt/io_read/filecursor_memory.hpp"
+#include "mpt/io_read/filecursor_stdstream.hpp"
+
 #include "../common/mptString.h"
 #include "../common/mptPathString.h"
-#include "../common/mptIO.h"
+#include "../common/FileReaderFwd.h"
 
+#if defined(MPT_COMPILER_QUIRK_WINDOWS_FSTREAM_NO_WCHAR)
+#if MPT_GCC_AT_LEAST(9,1,0)
+#include <filesystem>
+#endif // MPT_GCC_AT_LEAST(9,1,0)
+#endif // MPT_COMPILER_QUIRK_WINDOWS_FSTREAM_NO_WCHAR
 #include <fstream>
 #include <ios>
 #include <ostream>
@@ -27,9 +35,11 @@
 #include <cstdio>
 #endif // !MPT_COMPILER_MSVC
 
-#if MPT_COMPILER_MSVC
-#include <stdio.h>
-#endif // !MPT_COMPILER_MSVC
+#ifdef MODPLUG_TRACKER
+#if MPT_OS_WINDOWS
+#include <windows.h>
+#endif // MPT_OS_WINDOWS
+#endif // MODPLUG_TRACKER
 
 #endif // MPT_ENABLE_FILEIO
 
@@ -56,41 +66,50 @@ bool SetFilesystemCompression(const mpt::PathString &filename);
 namespace mpt
 {
 
-#if MPT_COMPILER_GCC && MPT_OS_WINDOWS
-// GCC C++ library has no wchar_t overloads
-#define MPT_FSTREAM_DO_CONVERSIONS_ANSI
-#endif
-
 namespace detail
 {
 
 template<typename Tbase>
 inline void fstream_open(Tbase & base, const mpt::PathString & filename, std::ios_base::openmode mode)
 {
-#if defined(MPT_FSTREAM_DO_CONVERSIONS_ANSI)
-	base.open(mpt::ToCharset(mpt::CharsetLocale, filename.AsNative()).c_str(), mode);
-#else
-	base.open(filename.AsNativePrefixed().c_str(), mode);
-#endif
+	#if defined(MPT_COMPILER_QUIRK_WINDOWS_FSTREAM_NO_WCHAR)
+		#if MPT_GCC_AT_LEAST(9,1,0)
+			base.open(static_cast<std::filesystem::path>(filename.AsNative()), mode);
+		#else // !MPT_GCC_AT_LEAST(9,1,0)
+			// Warning: MinGW with GCC earlier than 9.1 detected. Standard library does neither provide std::fstream wchar_t overloads nor std::filesystem with wchar_t support. Unicode filename support is thus unavailable.
+			base.open(mpt::ToCharset(mpt::Charset::Locale, filename.AsNative()).c_str(), mode);
+		#endif // MPT_GCC_AT_LEAST(9,1,0)
+	#else // !MPT_COMPILER_QUIRK_WINDOWS_FSTREAM_NO_WCHAR
+		base.open(filename.AsNativePrefixed().c_str(), mode);
+	#endif // MPT_COMPILER_QUIRK_WINDOWS_FSTREAM_NO_WCHAR
 }
 
 } // namespace detail
 
-class SafeOutputFile;
+// We cannot rely on implicit conversion of mpt::PathString to std::filesystem::path when constructing std::fstream
+// because of broken overload implementation in GCC libstdc++ 8, 9, 10.
+// https://gcc.gnu.org/bugzilla/show_bug.cgi?id=95642
+// https://gcc.gnu.org/bugzilla/show_bug.cgi?id=90704
 
 class fstream
 	: public std::fstream
 {
 private:
 	typedef std::fstream Tbase;
-public:
-	friend SafeOutputFile;
 public:
 	fstream() {}
 	fstream(const mpt::PathString & filename, std::ios_base::openmode mode = std::ios_base::in | std::ios_base::out)
 	{
 		detail::fstream_open<Tbase>(*this, filename, mode);
 	}
+#if MPT_COMPILER_MSVC
+protected:
+	fstream(std::FILE * file)
+		: std::fstream(file)
+	{
+	}
+#endif // MPT_COMPILER_MSVC
+public:
 	void open(const mpt::PathString & filename, std::ios_base::openmode mode = std::ios_base::in | std::ios_base::out)
 	{
 		detail::fstream_open<Tbase>(*this, filename, mode);
@@ -108,14 +127,20 @@ class ifstream
 {
 private:
 	typedef std::ifstream Tbase;
-public:
-	friend SafeOutputFile;
 public:
 	ifstream() {}
 	ifstream(const mpt::PathString & filename, std::ios_base::openmode mode = std::ios_base::in)
 	{
 		detail::fstream_open<Tbase>(*this, filename, mode);
 	}
+#if MPT_COMPILER_MSVC
+protected:
+	ifstream(std::FILE * file)
+		: std::ifstream(file)
+	{
+	}
+#endif // MPT_COMPILER_MSVC
+public:
 	void open(const mpt::PathString & filename, std::ios_base::openmode mode = std::ios_base::in)
 	{
 		detail::fstream_open<Tbase>(*this, filename, mode);
@@ -133,8 +158,6 @@ class ofstream
 {
 private:
 	typedef std::ofstream Tbase;
-public:
-	friend SafeOutputFile;
 public:
 	ofstream() {}
 	ofstream(const mpt::PathString & filename, std::ios_base::openmode mode = std::ios_base::out)
@@ -143,11 +166,10 @@ public:
 	}
 #if MPT_COMPILER_MSVC
 protected:
-	ofstream(FILE * file)
+	ofstream(std::FILE * file)
 		: std::ofstream(file)
 	{
 	}
-
 #endif // MPT_COMPILER_MSVC
 public:
 	void open(const mpt::PathString & filename, std::ios_base::openmode mode = std::ios_base::out)
@@ -169,7 +191,7 @@ enum class FlushMode
 	Full   = 2,  // explicitly flush *all* layers, up to and including disk write caches
 };
 
-static inline FlushMode FlushModeFromBool(bool flush)
+inline FlushMode FlushModeFromBool(bool flush)
 {
 	return flush ? FlushMode::Full : FlushMode::None;
 }
@@ -181,12 +203,24 @@ class SafeOutputFile
 private:
 	FlushMode m_FlushMode;
 #if MPT_COMPILER_MSVC
-	FILE *m_f;
-#endif // MPT_COMPILER_MSVC
+	std::FILE *m_f = nullptr;
+#else // !MPT_COMPILER_MSVC
 	mpt::ofstream m_s;
+#endif // MPT_COMPILER_MSVC
 #if MPT_COMPILER_MSVC
+	class FILEostream
+		: public mpt::ofstream
+	{
+	public:
+		FILEostream(std::FILE * file)
+			: mpt::ofstream(file)
+		{
+			return;
+		}
+	};
+	FILEostream m_s;
 	static mpt::tstring convert_mode(std::ios_base::openmode mode, FlushMode flushMode);
-	FILE * internal_fopen(const mpt::PathString &filename, std::ios_base::openmode mode, FlushMode flushMode);
+	std::FILE * internal_fopen(const mpt::PathString &filename, std::ios_base::openmode mode, FlushMode flushMode);
 #endif // MPT_COMPILER_MSVC
 public:
 	SafeOutputFile() = delete;
@@ -198,6 +232,10 @@ public:
 		, m_s(filename, mode)
 #endif // MPT_COMPILER_MSVC
 	{
+		if(!stream().is_open())
+		{
+			stream().setstate(mpt::ofstream::failbit);
+		}
 	}
 	mpt::ofstream& stream()
 	{
@@ -245,10 +283,10 @@ public:
 		return;
 	}
 public:
-	LazyFileRef & operator = (const std::vector<mpt::byte> &data);
+	LazyFileRef & operator = (const std::vector<std::byte> &data);
 	LazyFileRef & operator = (const std::vector<char> &data);
 	LazyFileRef & operator = (const std::string &data);
-	operator std::vector<mpt::byte> () const;
+	operator std::vector<std::byte> () const;
 	operator std::vector<char> () const;
 	operator std::string () const;
 };
@@ -259,65 +297,102 @@ public:
 } // namespace mpt
 
 
-
-#ifdef MODPLUG_TRACKER
-#if MPT_OS_WINDOWS
-class CMappedFile
-{
-protected:
-	HANDLE m_hFile;
-	HANDLE m_hFMap;
-	void *m_pData;
-	mpt::PathString m_FileName;
-
-public:
-	CMappedFile() : m_hFile(nullptr), m_hFMap(nullptr), m_pData(nullptr) { }
-	~CMappedFile();
-
-public:
-	bool Open(const mpt::PathString &filename);
-	bool IsOpen() const { return m_hFile != NULL && m_hFile != INVALID_HANDLE_VALUE; }
-	const mpt::PathString * GetpFilename() const { return &m_FileName; }
-	void Close();
-	size_t GetLength();
-	const mpt::byte *Lock();
-};
-#endif // MPT_OS_WINDOWS
-#endif // MODPLUG_TRACKER
-
-
 class InputFile
 {
 private:
 	mpt::PathString m_Filename;
-	#ifdef MPT_FILEREADER_STD_ISTREAM
-		mpt::ifstream m_File;
-	#else
-		CMappedFile m_File;
-	#endif
+	mpt::ifstream m_File;
+	bool m_IsValid;
+	bool m_IsCached;
+	std::vector<std::byte> m_Cache;
 public:
-	InputFile();
-	InputFile(const mpt::PathString &filename);
+	InputFile(const mpt::PathString &filename, bool allowWholeFileCaching = false);
 	~InputFile();
-	bool Open(const mpt::PathString &filename);
 	bool IsValid() const;
-#if defined(MPT_FILEREADER_STD_ISTREAM)
-	typedef std::pair<std::istream*, const mpt::PathString*> ContentsRef;
-#else
-	struct Data
-	{
-		const mpt::byte *data;
-		std::size_t size;
-	};
-	typedef std::pair<InputFile::Data, const mpt::PathString*> ContentsRef;
-#endif
-	InputFile::ContentsRef Get();
+	bool IsCached() const;
+	mpt::PathString GetFilename() const;
+	std::istream& GetStream();
+	mpt::const_byte_span GetCache();
+private:
+	bool Open(const mpt::PathString &filename, bool allowWholeFileCaching = false);
 };
 
 
+template <typename Targ1>
+inline FileCursor make_FileCursor(Targ1 &&arg1)
+{
+	return mpt::IO::make_FileCursor<mpt::PathString>(std::forward<Targ1>(arg1));
+}
+
+template <typename Targ1, typename Targ2>
+inline FileCursor make_FileCursor(Targ1 &&arg1, Targ2 &&arg2)
+{
+	return mpt::IO::make_FileCursor<mpt::PathString>(std::forward<Targ1>(arg1), std::forward<Targ2>(arg2));
+}
+
+
+// templated in order to reduce header inter-dependencies
+class InputFile;
+template <typename TInputFile, std::enable_if_t<std::is_same<TInputFile, InputFile>::value, bool> = true>
+inline FileCursor make_FileCursor(TInputFile &file)
+{
+	if(!file.IsValid())
+	{
+		return FileCursor();
+	}
+	if(file.IsCached())
+	{
+		return mpt::IO::make_FileCursor<mpt::PathString>(file.GetCache(), std::make_shared<mpt::PathString>(file.GetFilename()));
+	} else
+	{
+		return mpt::IO::make_FileCursor<mpt::PathString>(file.GetStream(), std::make_shared<mpt::PathString>(file.GetFilename()));
+	}
+}
+
+
+template <typename Targ1>
+inline FileCursor GetFileReader(Targ1 &&arg1)
+{
+	return make_FileCursor(std::forward<Targ1>(arg1));
+}
+
+
+template <typename Targ1, typename Targ2>
+inline FileCursor GetFileReader(Targ1 &&arg1, Targ2 &&arg2)
+{
+	return make_FileCursor(std::forward<Targ1>(arg1), std::forward<Targ2>(arg2));
+}
+
+
+#if defined(MODPLUG_TRACKER) && MPT_OS_WINDOWS
+
+class OnDiskFileWrapper
+{
+
+private:
+
+	mpt::PathString m_Filename;
+	bool m_IsTempFile;
+
+public:
+
+	OnDiskFileWrapper(FileCursor& file, const mpt::PathString& fileNameExtension = P_("tmp"));
+
+	~OnDiskFileWrapper();
+
+public:
+
+	bool IsValid() const;
+
+	mpt::PathString GetFilename() const;
+
+}; // class OnDiskFileWrapper
+
+#endif // MODPLUG_TRACKER && MPT_OS_WINDOWS
+
+
 #endif // MPT_ENABLE_FILEIO
 
 
-
 OPENMPT_NAMESPACE_END
 
