@@ -23,6 +23,13 @@ OPENMPT_NAMESPACE_BEGIN
 
 
 
+#define MPT_MAKE_VERSION_NUMERIC_HELPER(prefix,v0,v1,v2,v3) Version( prefix ## v0 , prefix ## v1 , prefix ## v2 , prefix ## v3 )
+#define MPT_MAKE_VERSION_NUMERIC(v0,v1,v2,v3) MPT_MAKE_VERSION_NUMERIC_HELPER(0x, v0, v1, v2, v3)
+
+#define MPT_VERSION_CURRENT MPT_MAKE_VERSION_NUMERIC(VER_MAJORMAJOR,VER_MAJOR,VER_MINOR,VER_MINORMINOR)
+
+
+
 static_assert((MPT_VERSION_CURRENT.GetRawVersion() & 0xffffu) != 0x0000u, "Version numbers ending in .00.00 shall never exist again, as they make interpreting the version number ambiguous for file formats which can only store the two major parts of the version number (e.g. IT and S3M).");
 
 
@@ -58,11 +65,11 @@ mpt::ustring Version::ToUString() const
 	} else if((v & 0xFFFF) == 0)
 	{
 		// Only parts of the version number are known (e.g. when reading the version from the IT or S3M file header)
-		return mpt::format(U_("%1.%2"))(mpt::ufmt::HEX((v >> 24) & 0xFF), mpt::ufmt::HEX0<2>((v >> 16) & 0xFF));
+		return MPT_UFORMAT("{}.{}")(mpt::ufmt::HEX((v >> 24) & 0xFF), mpt::ufmt::HEX0<2>((v >> 16) & 0xFF));
 	} else
 	{
 		// Full version info available
-		return mpt::format(U_("%1.%2.%3.%4"))(mpt::ufmt::HEX((v >> 24) & 0xFF), mpt::ufmt::HEX0<2>((v >> 16) & 0xFF), mpt::ufmt::HEX0<2>((v >> 8) & 0xFF), mpt::ufmt::HEX0<2>((v) & 0xFF));
+		return MPT_UFORMAT("{}.{}.{}.{}")(mpt::ufmt::HEX((v >> 24) & 0xFF), mpt::ufmt::HEX0<2>((v >> 16) & 0xFF), mpt::ufmt::HEX0<2>((v >> 8) & 0xFF), mpt::ufmt::HEX0<2>((v) & 0xFF));
 	}
 }
 
@@ -80,10 +87,10 @@ bool Version::IsTestVersion() const noexcept
 {
 	return (
 			// Legacy
-			(*this > MAKE_VERSION_NUMERIC(1,17,02,54) && *this < MAKE_VERSION_NUMERIC(1,18,02,00) && *this != MAKE_VERSION_NUMERIC(1,18,00,00))
+			(*this > MPT_V("1.17.02.54") && *this < MPT_V("1.18.02.00") && *this != MPT_V("1.18.00.00"))
 		||
 			// Test builds have non-zero VER_MINORMINOR
-			(*this > MAKE_VERSION_NUMERIC(1,18,02,00) && ((m_Version & 0xFFFFFF00u) != m_Version))
+			(*this > MPT_V("1.18.02.00") && ((m_Version & 0xFFFFFF00u) != m_Version))
 		);
 }
 
@@ -94,7 +101,7 @@ namespace Source {
 static mpt::ustring GetUrl()
 {
 	#ifdef OPENMPT_VERSION_URL
-		return mpt::ToUnicode(mpt::CharsetASCII, OPENMPT_VERSION_URL);
+		return mpt::ToUnicode(mpt::Charset::ASCII, OPENMPT_VERSION_URL);
 	#else
 		return mpt::ustring();
 	#endif
@@ -132,15 +139,7 @@ static int GetRevision()
 		}
 		return ConvertStrTo<int>(svnversion);
 	#else
-		#if MPT_COMPILER_MSVC
-			#pragma message("SVN revision unknown. Please check your build system.")
-		#elif MPT_COMPILER_GCC || MPT_COMPILER_CLANG
-			#warning "SVN revision unknown. Please check your build system."
-		#else
-			// There is no portable way to display a warning.
-			// Try to provoke a warning with an unused variable.
-			int SVN_revision_unknown__Please_check_your_build_system;
-		#endif
+		MPT_WARNING_STATEMENT("SVN revision unknown. Please check your build system.");
 		return 0;
 	#endif
 }
@@ -209,7 +208,7 @@ static bool IsPackage()
 static mpt::ustring GetSourceDate()
 {
 	#if defined(OPENMPT_VERSION_DATE)
-		return mpt::ToUnicode(mpt::CharsetASCII, OPENMPT_VERSION_DATE);
+		return mpt::ToUnicode(mpt::Charset::ASCII, OPENMPT_VERSION_DATE);
 	#else
 		return mpt::ustring();
 	#endif
@@ -265,6 +264,43 @@ SourceInfo SourceInfo::Current()
 
 
 
+VersionWithRevision VersionWithRevision::Current()
+{
+	return {Version::Current(), static_cast<uint64>(SourceInfo::Current().Revision())};
+}
+
+VersionWithRevision VersionWithRevision::Parse(const mpt::ustring &s)
+{
+	Version version = Version::Parse(mpt::ustring());
+	uint64 revision = 0;
+	const auto tokens = mpt::String::Split<mpt::ustring>(s, U_("-"));
+	if(tokens.size() >= 1)
+	{
+		version = Version::Parse(tokens[0]);
+	}
+	if(tokens.size() >= 2)
+	{
+		revision = ConvertStrTo<uint64>(tokens[1].substr(1));
+	}
+	return {version, revision};
+}
+
+mpt::ustring VersionWithRevision::ToUString() const
+{
+	if(!HasRevision())
+	{
+		return mpt::ufmt::val(version);
+	}
+	if(!version.IsTestVersion())
+	{
+		return mpt::ufmt::val(version);
+	}
+	return MPT_UFORMAT("{}-r{}")(version, revision);
+}
+
+
+
+
 namespace Build {
 
 bool IsReleasedBuild()
@@ -286,9 +322,9 @@ mpt::ustring GetBuildDateString()
 	mpt::ustring result;
 	#ifdef MODPLUG_TRACKER
 		#if defined(OPENMPT_BUILD_DATE)
-			result = mpt::ToUnicode(mpt::CharsetASCII, OPENMPT_BUILD_DATE );
+			result = mpt::ToUnicode(mpt::Charset::ASCII, OPENMPT_BUILD_DATE );
 		#else
-			result = mpt::ToUnicode(mpt::CharsetASCII, __DATE__ " " __TIME__ );
+			result = mpt::ToUnicode(mpt::Charset::ASCII, __DATE__ " " __TIME__ );
 		#endif
 	#else // !MODPLUG_TRACKER
 		result = SourceInfo::Current().Date();
@@ -300,6 +336,9 @@ static mpt::ustring GetBuildFlagsString()
 {
 	mpt::ustring retval;
 	#ifdef MODPLUG_TRACKER
+		#if defined(MPT_BUILD_RETRO)
+			retval += UL_(" RETRO");
+		#endif // MPT_BUILD_RETRO
 		if(Version::Current().IsTestVersion())
 		{
 			retval += UL_(" TEST");
@@ -317,18 +356,6 @@ mpt::ustring GetBuildFeaturesString()
 	mpt::ustring retval;
 	#ifdef LIBOPENMPT_BUILD
 		retval = UL_("")
-		#if defined(MPT_CHARSET_WIN32)
-			UL_(" +WINAPI")
-		#endif
-		#if defined(MPT_CHARSET_ICONV)
-			UL_(" +ICONV")
-		#endif
-		#if defined(MPT_CHARSET_CODECVTUTF8)
-			UL_(" +CODECVTUTF8")
-		#endif
-		#if defined(MPT_CHARSET_INTERNAL)
-			UL_(" +INTERNALCHARSETS")
-		#endif
 		#if defined(MPT_WITH_ZLIB)
 			UL_(" +ZLIB")
 		#endif
@@ -364,46 +391,26 @@ mpt::ustring GetBuildFeaturesString()
 		#else
 			UL_(" -PLUGINS")
 		#endif
-		#if !defined(NO_DMO)
+		#if defined(MPT_WITH_DMO)
 			UL_(" +DMO")
 		#endif
 		;
 	#endif
 	#ifdef MODPLUG_TRACKER
-		MPT_CONSTANT_IF(mpt::arch_bits == 64)
-		{
-			if (true
-				&& (mpt::Windows::Version::GetMinimumKernelLevel() <= mpt::Windows::Version::WinXP64)
-				&& (mpt::Windows::Version::GetMinimumAPILevel() <= mpt::Windows::Version::WinXP64)
-			) {
-				retval += UL_(" WIN64OLD");
-			}
-		} else MPT_CONSTANT_IF(mpt::arch_bits == 32)
-		{
-			if (true
-				&& (mpt::Windows::Version::GetMinimumKernelLevel() <= mpt::Windows::Version::WinXP)
-				&& (mpt::Windows::Version::GetMinimumAPILevel() <= mpt::Windows::Version::WinXP)
-			) {
-				retval += UL_(" WIN32OLD");
-			}
-		}
 		retval += UL_("")
 		#if defined(UNICODE)
 			UL_(" UNICODE")
 		#else
 			UL_(" ANSI")
 		#endif
-		#ifdef NO_VST
+		#ifndef MPT_WITH_VST
 			UL_(" NO_VST")
 		#endif
-		#ifdef NO_DMO
+		#ifndef MPT_WITH_DMO
 			UL_(" NO_DMO")
 		#endif
 		#ifdef NO_PLUGINS
 			UL_(" NO_PLUGINS")
-		#endif
-		#ifndef MPT_WITH_ASIO
-			UL_(" NO_ASIO")
 		#endif
 			;
 	#endif
@@ -417,25 +424,25 @@ mpt::ustring GetBuildCompilerString()
 		retval += U_("Generic C++11 Compiler");
 	#elif MPT_COMPILER_MSVC
 		#if defined(_MSC_FULL_VER) && defined(_MSC_BUILD) && (_MSC_BUILD > 0)
-			retval += mpt::format(U_("Microsoft Compiler %1.%2.%3.%4"))
+			retval += MPT_UFORMAT("Microsoft Compiler {}.{}.{}.{}")
 				( _MSC_FULL_VER / 10000000
 				, mpt::ufmt::dec0<2>((_MSC_FULL_VER / 100000) % 100)
 				, mpt::ufmt::dec0<5>(_MSC_FULL_VER % 100000)
 				, mpt::ufmt::dec0<2>(_MSC_BUILD)
 				);
 		#elif defined(_MSC_FULL_VER)
-			retval += mpt::format(U_("Microsoft Compiler %1.%2.%3"))
+			retval += MPT_UFORMAT("Microsoft Compiler {}.{}.{}")
 				( _MSC_FULL_VER / 10000000
 				, mpt::ufmt::dec0<2>((_MSC_FULL_VER / 100000) % 100)
 				, mpt::ufmt::dec0<5>(_MSC_FULL_VER % 100000)
 				);
 		#else
-			retval += mpt::format(U_("Microsoft Compiler %1.%2"))(MPT_COMPILER_MSVC_VERSION / 100, MPT_COMPILER_MSVC_VERSION % 100);
+			retval += MPT_UFORMAT("Microsoft Compiler {}.{}")(MPT_COMPILER_MSVC_VERSION / 100, MPT_COMPILER_MSVC_VERSION % 100);
 		#endif
 	#elif MPT_COMPILER_GCC
-		retval += mpt::format(U_("GNU Compiler Collection %1.%2.%3"))(MPT_COMPILER_GCC_VERSION / 10000, (MPT_COMPILER_GCC_VERSION / 100) % 100, MPT_COMPILER_GCC_VERSION % 100);
+		retval += MPT_UFORMAT("GNU Compiler Collection {}.{}.{}")(MPT_COMPILER_GCC_VERSION / 10000, (MPT_COMPILER_GCC_VERSION / 100) % 100, MPT_COMPILER_GCC_VERSION % 100);
 	#elif MPT_COMPILER_CLANG
-		retval += mpt::format(U_("Clang %1.%2.%3"))(MPT_COMPILER_CLANG_VERSION / 10000, (MPT_COMPILER_CLANG_VERSION / 100) % 100, MPT_COMPILER_CLANG_VERSION % 100);
+		retval += MPT_UFORMAT("Clang {}.{}.{}")(MPT_COMPILER_CLANG_VERSION / 10000, (MPT_COMPILER_CLANG_VERSION / 100) % 100, MPT_COMPILER_CLANG_VERSION % 100);
 	#else
 		retval += U_("unknown");
 	#endif
@@ -479,24 +486,20 @@ mpt::ustring GetVersionString(FlagSet<Build::Strings> strings)
 			result.push_back(GetRevisionString());
 		}
 	}
-	if(strings[StringBitness])
-	{
-		result.push_back(mpt::format(U_(" %1 bit"))(mpt::arch_bits));
-	}
 	if(strings[StringSourceInfo])
 	{
 		const SourceInfo sourceInfo = SourceInfo::Current();
 		if(!sourceInfo.GetUrlWithRevision().empty())
 		{
-			result.push_back(mpt::format(U_(" %1"))(sourceInfo.GetUrlWithRevision()));
+			result.push_back(MPT_UFORMAT(" {}")(sourceInfo.GetUrlWithRevision()));
 		}
 		if(!sourceInfo.Date().empty())
 		{
-			result.push_back(mpt::format(U_(" (%1)"))(sourceInfo.Date()));
+			result.push_back(MPT_UFORMAT(" ({})")(sourceInfo.Date()));
 		}
 		if(!sourceInfo.GetStateString().empty())
 		{
-			result.push_back(mpt::format(U_(" %1"))(sourceInfo.GetStateString()));
+			result.push_back(MPT_UFORMAT(" {}")(sourceInfo.GetStateString()));
 		}
 	}
 	if(strings[StringBuildFlags])
@@ -510,7 +513,7 @@ mpt::ustring GetVersionString(FlagSet<Build::Strings> strings)
 	{
 		result.push_back(GetBuildFeaturesString());
 	}
-	return mpt::String::Trim(mpt::String::Combine<mpt::ustring>(result, U_("")));
+	return mpt::trim(mpt::String::Combine<mpt::ustring>(result, U_("")));
 }
 
 mpt::ustring GetVersionStringPure()
@@ -518,9 +521,6 @@ mpt::ustring GetVersionStringPure()
 	FlagSet<Build::Strings> strings;
 	strings |= Build::StringVersion;
 	strings |= Build::StringRevision;
-	#ifdef MODPLUG_TRACKER
-		strings |= Build::StringBitness;
-	#endif
 	return GetVersionString(strings);
 }
 
@@ -538,9 +538,6 @@ mpt::ustring GetVersionStringExtended()
 	FlagSet<Build::Strings> strings;
 	strings |= Build::StringVersion;
 	strings |= Build::StringRevision;
-	#ifdef MODPLUG_TRACKER
-		strings |= Build::StringBitness;
-	#endif
 	#ifndef MODPLUG_TRACKER
 		strings |= Build::StringSourceInfo;
 	#endif
@@ -588,19 +585,19 @@ mpt::ustring GetURL(Build::Url key)
 
 mpt::ustring GetFullCreditsString()
 {
-	return mpt::ToUnicode(mpt::CharsetUTF8,
+	return mpt::ToUnicode(mpt::Charset::UTF8,
 #ifdef MODPLUG_TRACKER
-		"OpenMPT / ModPlug Tracker\n"
+		"OpenMPT / Open ModPlug Tracker\n"
 #else
-		"libopenmpt (based on OpenMPT / ModPlug Tracker)\n"
+		"libopenmpt (based on OpenMPT / Open ModPlug Tracker)\n"
 #endif
 		"\n"
-		"Copyright \xC2\xA9 2004-2021 Contributors\n"
+		"Copyright \xC2\xA9 2004-2022 OpenMPT Project Developers and Contributors\n"
 		"Copyright \xC2\xA9 1997-2003 Olivier Lapicque\n"
 		"\n"
-		"Contributors:\n"
-		"Johannes Schultz (2008-2021)\n"
-		"J\xC3\xB6rn Heusipp (2012-2021)\n"
+		"Developers:\n"
+		"Johannes Schultz (2008-2022)\n"
+		"J\xC3\xB6rn Heusipp (2012-2022)\n"
 		"Ahti Lepp\xC3\xA4nen (2005-2011)\n"
 		"Robin Fernandes (2004-2007)\n"
 		"Sergiy Pylypenko (2007)\n"
@@ -608,8 +605,9 @@ mpt::ustring GetFullCreditsString()
 		"Trevor Nunes (2004)\n"
 		"Olivier Lapicque (1997-2003)\n"
 		"\n"
-		"Additional patch submitters:\n"
+		"Additional contributors:\n"
 		"coda (https://coda.s3m.us/)\n"
+		"Jo\xC3\xA3o Baptista de Paula e Silva (https://joaobapt.com/)\n"
 		"kode54 (https://kode54.net/)\n"
 		"Revenant (https://revenant1.net/)\n"
 		"xaimus (http://xaimus.com/)\n"
@@ -630,7 +628,7 @@ mpt::ustring GetFullCreditsString()
 		"https://www.surina.net/soundtouch/\n"
 		"\n"
 #endif
-#ifndef NO_VST
+#ifdef MPT_WITH_VST
 		"Hermann Seib for his example VST Host implementation\n"
 		"http://www.hermannseib.com/english/vsthost.htm\n"
 		"\n"
@@ -657,6 +655,9 @@ mpt::ustring GetFullCreditsString()
 		"Shayde / Reality Productions for Opal OPL3 emulator\n"
 		"https://www.3eality.com/\n"
 		"\n"
+		"Ryuhei Mori for TinyFFT\n"
+		"https://github.com/ryuhei-mori/tinyfft\n"
+		"\n"
 #ifdef MPT_WITH_ZLIB
 		"Jean-loup Gailly and Mark Adler for zlib\n"
 		"https://zlib.net/\n"
@@ -677,13 +678,18 @@ mpt::ustring GetFullCreditsString()
 		"https://rarlab.com/\n"
 		"\n"
 #endif
+#ifdef MPT_WITH_ANCIENT
+		"Teemu Suutari for ancient\n"
+		"https://github.com/temisu/ancient\n"
+		"\n"
+#endif
 #ifdef MPT_WITH_PORTAUDIO
 		"PortAudio contributors\n"
 		"http://www.portaudio.com/\n"
 		"\n"
 #endif
 #ifdef MPT_WITH_RTAUDIO
-		"Gary P. Scavone, McGill University\n"
+		"Gary P. Scavone, McGill University for RtAudio\n"
 		"https://www.music.mcgill.ca/~gary/rtaudio/\n"
 		"\n"
 #endif
@@ -772,7 +778,7 @@ mpt::ustring GetFullCreditsString()
 		"Harbinger, jmkz, KrazyKatz, LPChip, Nofold, Rakib, Sam Zen\n"
 		"Skaven, Skilletaudio, Snu, Squirrel Havoc, Teimoso, Waxhead\n"
 		"\n"
-#ifndef NO_VST
+#ifdef MPT_WITH_VST
 		"VST PlugIn Technology by Steinberg Media Technologies GmbH\n"
 		"\n"
 #endif
@@ -786,7 +792,7 @@ mpt::ustring GetFullCreditsString()
 mpt::ustring GetLicenseString()
 {
 	return MPT_UTF8(
-		"Copyright (c) 2004-2021, OpenMPT contributors" "\n"
+		"Copyright (c) 2004-2022, OpenMPT Project Developers and Contributors" "\n"
 		"Copyright (c) 1997-2003, Olivier Lapicque" "\n"
 		"All rights reserved." "\n"
 		"" "\n"
@@ -801,16 +807,16 @@ mpt::ustring GetLicenseString()
 		"      names of its contributors may be used to endorse or promote products" "\n"
 		"      derived from this software without specific prior written permission." "\n"
 		"" "\n"
-		"THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY" "\n"
-		"EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED" "\n"
-		"WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE" "\n"
-		"DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY" "\n"
-		"DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES" "\n"
-		"(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;" "\n"
-		"LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND" "\n"
-		"ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT" "\n"
-		"(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS" "\n"
-		"SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE." "\n"
+		"THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"" "\n"
+		"AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE" "\n"
+		"IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE" "\n"
+		"DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE" "\n"
+		"FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL" "\n"
+		"DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR" "\n"
+		"SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER" "\n"
+		"CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY," "\n"
+		"OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE" "\n"
+		"OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE." "\n"
 		);
 }
 
