@@ -8,7 +8,7 @@
  */
 
 static const char * const license =
-"Copyright (c) 2004-2021, OpenMPT contributors" "\n"
+"Copyright (c) 2004-2022, OpenMPT Project Developers and Contributors" "\n"
 "Copyright (c) 1997-2003, Olivier Lapicque" "\n"
 "All rights reserved." "\n"
 "" "\n"
@@ -23,16 +23,16 @@ static const char * const license =
 "      names of its contributors may be used to endorse or promote products" "\n"
 "      derived from this software without specific prior written permission." "\n"
 "" "\n"
-"THIS SOFTWARE IS PROVIDED BY THE CONTRIBUTORS ``AS IS'' AND ANY" "\n"
-"EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED" "\n"
-"WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE" "\n"
-"DISCLAIMED. IN NO EVENT SHALL THE CONTRIBUTORS BE LIABLE FOR ANY" "\n"
-"DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES" "\n"
-"(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;" "\n"
-"LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND" "\n"
-"ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT" "\n"
-"(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS" "\n"
-"SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE." "\n"
+"THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\"" "\n"
+"AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE" "\n"
+"IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE" "\n"
+"DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE" "\n"
+"FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL" "\n"
+"DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR" "\n"
+"SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER" "\n"
+"CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY," "\n"
+"OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE" "\n"
+"OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE." "\n"
 ;
 
 #include "openmpt123_config.hpp"
@@ -46,6 +46,7 @@ static const char * const license =
 #include <limits>
 #include <locale>
 #include <map>
+#include <memory>
 #include <random>
 #include <set>
 #include <sstream>
@@ -75,6 +76,9 @@ static const char * const license =
 #include <fcntl.h>
 #include <io.h>
 #include <stdio.h>
+#if defined(__MINGW32__) && !defined(__MINGW64__)
+#include <string.h>
+#endif
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <windows.h>
@@ -460,7 +464,7 @@ static std::string seconds_to_string( double time ) {
 
 static void show_info( std::ostream & log, bool verbose ) {
 	log << "openmpt123" << " v" << OPENMPT123_VERSION_STRING << ", libopenmpt " << openmpt::string::get( "library_version" ) << " (" << "OpenMPT " << openmpt::string::get( "core_version" ) << ")" << std::endl;
-	log << "Copyright (c) 2013-2021 OpenMPT developers <https://lib.openmpt.org/>" << std::endl;
+	log << "Copyright (c) 2013-2022 OpenMPT Project Developers and Contributors <https://lib.openmpt.org/>" << std::endl;
 	if ( !verbose ) {
 		log << std::endl;
 		return;
@@ -504,7 +508,7 @@ static void show_info( std::ostream & log, bool verbose ) {
 	SDL_version sdlver;
 	std::memset( &sdlver, 0, sizeof( SDL_version ) );
 	SDL_GetVersion( &sdlver );
-	log << static_cast<int>( sdlver.major ) << "." << static_cast<int>( sdlver.minor ) << "." << static_cast<int>( sdlver.patch ) << "." << SDL_GetRevisionNumber();
+	log << static_cast<int>( sdlver.major ) << "." << static_cast<int>( sdlver.minor ) << "." << static_cast<int>( sdlver.patch );
 	const char * revision = SDL_GetRevision();
 	if ( revision ) {
 		log << " (" << revision << ")";
@@ -537,7 +541,7 @@ static void show_info( std::ostream & log, bool verbose ) {
 static void show_man_version( textout & log ) {
 	log << "openmpt123" << " v" << OPENMPT123_VERSION_STRING << std::endl;
 	log << std::endl;
-	log << "Copyright (c) 2013-2021 OpenMPT developers <https://lib.openmpt.org/>" << std::endl;
+	log << "Copyright (c) 2013-2022 OpenMPT Project Developers and Contributors <https://lib.openmpt.org/>" << std::endl;
 }
 
 static void show_short_version( textout & log ) {
@@ -921,9 +925,9 @@ static std::string channel_to_string( int channels, int channel, const meter_cha
 			<< " : "
 			;
 		res2 
-			<< std::string(val,'>') << std::string(48-val,' ')
+			<< std::string(val,'>') << std::string(std::size_t{48}-val,' ')
 			<< ( ( meter.clip != 0.0f ) ? "#" : ":" )
-			<< std::string(headroom,'>') << std::string(12-headroom,' ')
+			<< std::string(headroom,'>') << std::string(std::size_t{12}-headroom,' ')
 			;
 		std::string tmp = res2.str();
 		if ( 0 <= hold_pos && hold_pos <= 60 ) {
@@ -1029,7 +1033,7 @@ void render_loop( commandlineflags & flags, Tmod & mod, double & duration, texto
 		bufsize = 1024;
 	}
 
-	std::int64_t last_redraw_frame = 0 - flags.ui_redraw_interval;
+	std::int64_t last_redraw_frame = std::int64_t{0} - flags.ui_redraw_interval;
 	std::int64_t rendered_frames = 0;
 
 	std::vector<Tsample> left( bufsize );
@@ -1445,7 +1449,7 @@ static void probe_mod_file( commandlineflags & flags, const std::string & filena
 		set_field( fields, "Size" ).ostream() << bytes_to_string( filesize );
 	}
 	
-	int probe_result = openmpt::probe_file_header( openmpt::probe_file_header_flags_default, data_stream );
+	int probe_result = openmpt::probe_file_header( openmpt::probe_file_header_flags_default2, data_stream );
 	std::string probe_result_string;
 	switch ( probe_result ) {
 		case openmpt::probe_file_header_result_success:
@@ -1598,7 +1602,7 @@ static void probe_file( commandlineflags & flags, const std::string & filename,
 				// Only MSVC has std::ifstream::ifstream(std::wstring).
 				// Fake it for other compilers using _wfopen().
 				std::string data;
-				FILE * f = _wfopen( utf8_to_wstring( filename ).c_str(), L"rb" );
+				FILE * f = _wfopen( mpt::transcode<std::wstring>( mpt::common_encoding::utf8, filename ).c_str(), L"rb" );
 				if ( f ) {
 					while ( !feof( f ) ) {
 						static const std::size_t BUFFER_SIZE = 4096;
@@ -1612,7 +1616,7 @@ static void probe_file( commandlineflags & flags, const std::string & filename,
 				file_stream.str( data );
 				filesize = data.length();
 			#elif defined(_MSC_VER) && defined(UNICODE)
-				file_stream.open( utf8_to_wstring( filename ), std::ios::binary );
+				file_stream.open( mpt::transcode<std::wstring>( mpt::common_encoding::utf8, filename ), std::ios::binary );
 				file_stream.seekg( 0, std::ios::end );
 				filesize = file_stream.tellg();
 				file_stream.seekg( 0, std::ios::beg );
@@ -1674,7 +1678,7 @@ static void render_file( commandlineflags & flags, const std::string & filename,
 				// Only MSVC has std::ifstream::ifstream(std::wstring).
 				// Fake it for other compilers using _wfopen().
 				std::string data;
-				FILE * f = _wfopen( utf8_to_wstring( filename ).c_str(), L"rb" );
+				FILE * f = _wfopen( mpt::transcode<std::wstring>( mpt::common_encoding::utf8, filename ).c_str(), L"rb" );
 				if ( f ) {
 					while ( !feof( f ) ) {
 						static const std::size_t BUFFER_SIZE = 4096;
@@ -1688,7 +1692,7 @@ static void render_file( commandlineflags & flags, const std::string & filename,
 				file_stream.str( data );
 				filesize = data.length();
 			#elif defined(_MSC_VER) && defined(UNICODE)
-				file_stream.open( utf8_to_wstring( filename ), std::ios::binary );
+				file_stream.open( mpt::transcode<std::wstring>( mpt::common_encoding::utf8, filename ), std::ios::binary );
 				file_stream.seekg( 0, std::ios::end );
 				filesize = file_stream.tellg();
 				file_stream.seekg( 0, std::ios::beg );
@@ -1841,7 +1845,7 @@ static bool parse_playlist( commandlineflags & flags, std::string filename, std:
 			// Only MSVC has std::ifstream::ifstream(std::wstring).
 			// Fake it for other compilers using _wfopen().
 			std::string data;
-			FILE * f = _wfopen( utf8_to_wstring( filename ).c_str(), L"rb" );
+			FILE * f = _wfopen( mpt::transcode<std::wstring>( mpt::common_encoding::utf8, filename ).c_str(), L"rb" );
 			if ( f ) {
 				while ( !feof( f ) ) {
 					static const std::size_t BUFFER_SIZE = 4096;
@@ -1854,7 +1858,7 @@ static bool parse_playlist( commandlineflags & flags, std::string filename, std:
 			}
 			file_stream.str( data );
 		#elif defined(_MSC_VER) && defined(UNICODE)
-			file_stream.open( utf8_to_wstring( filename ), std::ios::binary );
+			file_stream.open( mpt::transcode<std::wstring>( mpt::common_encoding::utf8, filename ), std::ios::binary );
 		#else
 			file_stream.open( filename, std::ios::binary );
 		#endif
@@ -1903,7 +1907,7 @@ static bool parse_playlist( commandlineflags & flags, std::string filename, std:
 					newfile = line;
 				} else {
 #if defined(WIN32)
-					newfile = wstring_to_utf8( locale_to_wstring( line ) );
+					newfile = mpt::transcode<std::string>( mpt::common_encoding::utf8, mpt::logical_encoding::locale, line );
 #else
 					newfile = line;
 #endif
@@ -1913,7 +1917,7 @@ static bool parse_playlist( commandlineflags & flags, std::string filename, std:
 					newfile = line;
 				} else {
 #if defined(WIN32)
-					newfile = wstring_to_utf8( locale_to_wstring( line ) );
+					newfile = mpt::transcode<std::string>( mpt::common_encoding::utf8, mpt::logical_encoding::locale, line );
 #else
 					newfile = line;
 #endif
@@ -2276,7 +2280,7 @@ static int main( int argc, char * argv [] ) {
 	std::vector<std::string> args;
 	#if defined(WIN32) && defined(UNICODE)
 		for ( int arg = 0; arg < wargc; ++arg ) {
-			args.push_back( wstring_to_utf8( wargv[arg] ) );
+			args.push_back( mpt::transcode<std::string>( mpt::common_encoding::utf8, wargv[arg] ) );
 		}
 	#else
 		args = std::vector<std::string>( argv, argv + argc );
@@ -2526,6 +2530,7 @@ static int main( int argc, char * argv [] ) {
 #if defined(WIN32) && defined(UNICODE)
 #if defined(__GNUC__)
 // mingw64 does only default to special C linkage for "main", but not for "wmain".
+extern "C" int wmain( int wargc, wchar_t * wargv [] );
 extern "C"
 #endif
 int wmain( int wargc, wchar_t * wargv [] ) {
