@@ -12,6 +12,13 @@
 
 #include "openmpt123_config.hpp"
 
+#include "mpt/base/compiletime_warning.hpp"
+#include "mpt/base/floatingpoint.hpp"
+#include "mpt/base/preprocessor.hpp"
+#include "mpt/string_transcode/transcode.hpp"
+
+#include <string>
+
 namespace openmpt123 {
 
 struct exception : public openmpt::exception {
@@ -31,84 +38,11 @@ struct args_error_exception {
 struct show_help_keyboard_exception { };
 
 #if defined(WIN32)
-
-std::string wstring_to_utf8( const std::wstring & unicode_string ) {
-	int required_size = WideCharToMultiByte( CP_UTF8, 0, unicode_string.c_str(), -1, NULL, 0, NULL, NULL );
-	if ( required_size <= 0 ) {
-		return std::string();
-	}
-	std::vector<char> utf8_buf( required_size );
-	WideCharToMultiByte( CP_UTF8, 0, unicode_string.c_str(), -1, &utf8_buf[0], required_size, NULL, NULL );
-	return &utf8_buf[0];
-}
-
-std::wstring utf8_to_wstring( const std::string & utf8_string ) {
-	int required_size = MultiByteToWideChar( CP_UTF8, 0, utf8_string.c_str(), -1, NULL, 0 );
-	if ( required_size <= 0 ) {
-		return std::wstring();
-	}
-	std::vector<wchar_t> unicode_buf( required_size );
-	MultiByteToWideChar( CP_UTF8, 0, utf8_string.data(), -1, &unicode_buf[0], required_size );
-	return &unicode_buf[0];
-}
-
-std::string wstring_to_locale( const std::wstring & unicode_string ) {
-	int required_size = WideCharToMultiByte( CP_ACP, 0, unicode_string.c_str(), -1, NULL, 0, NULL, NULL );
-	if ( required_size <= 0 ) {
-		return std::string();
-	}
-	std::vector<char> locale_buf( required_size );
-	WideCharToMultiByte( CP_ACP, 0, unicode_string.c_str(), -1, &locale_buf[0], required_size, NULL, NULL );
-	return &locale_buf[0];
-}
-
-std::wstring locale_to_wstring( const std::string & locale_string ) {
-	int required_size = MultiByteToWideChar( CP_ACP, 0, locale_string.c_str(), -1, NULL, 0 );
-	if ( required_size <= 0 ) {
-		return std::wstring();
-	}
-	std::vector<wchar_t> unicode_buf( required_size );
-	MultiByteToWideChar( CP_ACP, 0, locale_string.data(), -1, &unicode_buf[0], required_size );
-	return &unicode_buf[0];
-}
-
-#endif // WIN32
-
-#if defined(MPT_NEEDS_THREADS)
-
-#if defined(WIN32)
-
-class mutex {
-private:
-	CRITICAL_SECTION impl;
-public:
-	mutex() { InitializeCriticalSection(&impl); }
-	~mutex() { DeleteCriticalSection(&impl); }
-	void lock() { EnterCriticalSection(&impl); }
-	void unlock() { LeaveCriticalSection(&impl); }
-};
-
-#else
-
-class mutex {
-private:
-	pthread_mutex_t impl;
-public:
-	mutex() {
-		pthread_mutexattr_t attr;
-		pthread_mutexattr_init( &attr );
-		pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_NORMAL );
-		pthread_mutex_init( &impl, &attr );
-		pthread_mutexattr_destroy( &attr );
-	}
-	~mutex() { pthread_mutex_destroy( &impl ); }
-	void lock() { pthread_mutex_lock( &impl ); }
-	void unlock() { pthread_mutex_unlock( &impl ); }
-};
-
+bool IsConsole( DWORD stdHandle );
 #endif
+bool IsTerminal( int fd );
+
 
-#endif
 
 struct field {
 	std::string key;
@@ -128,11 +62,16 @@ public:
 	virtual ~textout() {
 		return;
 	}
-public:
-	virtual void write( const std::string & text ) = 0;
-	virtual void writeout() {
-		write( str() );
+protected:
+	std::string pop() {
+		std::string text = str();
 		str(std::string());
+		return text;
+	}
+public:
+	virtual void writeout() = 0;
+	virtual void cursor_up( std::size_t lines ) {
+		static_cast<void>( lines );
 	}
 };
 
@@ -145,55 +84,123 @@ public:
 		return;
 	}
 public:
-	void write( const std::string & /* text */ ) override {
-		return;
+	void writeout() override {
+		static_cast<void>( pop() );
 	}
 };
 
 class textout_ostream : public textout {
 private:
 	std::ostream & s;
+#if defined(__DJGPP__)
+	mpt::common_encoding codepage;
+#endif
 public:
 	textout_ostream( std::ostream & s_ )
 		: s(s_)
+#if defined(__DJGPP__)
+		, codepage(mpt::common_encoding::cp437)
+#endif
 	{
+		#if defined(__DJGPP__)
+			codepage = mpt::djgpp_get_locale_encoding();
+		#endif
 		return;
 	}
 	virtual ~textout_ostream() {
-		writeout();
+		writeout_impl();
+	}
+private:
+	void writeout_impl() {
+		std::string text = pop();
+		if ( text.length() > 0 ) {
+			#if defined(__DJGPP__)
+				s << mpt::transcode<std::string>( codepage, mpt::common_encoding::utf8, text );
+			#elif defined(__EMSCRIPTEN__)
+				s << text;
+			#else
+				s << mpt::transcode<std::string>( mpt::logical_encoding::locale, mpt::common_encoding::utf8, text );
+		#endif
+			s.flush();
+		}	
 	}
 public:
-	void write( const std::string & text ) override {
-		s << text;
-	}
 	void writeout() override {
-		textout::writeout();
+		writeout_impl();
+	}
+	void cursor_up( std::size_t lines ) override {
 		s.flush();
+		for ( std::size_t line = 0; line < lines; ++line ) {
+			*this << "\x1b[1A";
+		}
 	}
 };
 
 #if defined(WIN32)
 
-class textout_console : public textout {
+class textout_ostream_console : public textout {
 private:
+#if defined(UNICODE)
+	std::wostream & s;
+#else
+	std::ostream & s;
+#endif
 	HANDLE handle;
+	bool console;
 public:
-	textout_console( HANDLE handle_ )
-		: handle(handle_)
+#if defined(UNICODE)
+	textout_ostream_console( std::wostream & s_, DWORD stdHandle_ )
+#else
+	textout_ostream_console( std::ostream & s_, DWORD stdHandle_ )
+#endif
+		: s(s_)
+		, handle(GetStdHandle( stdHandle_ ))
+		, console(IsConsole( stdHandle_ ))
 	{
 		return;
 	}
-	virtual ~textout_console() {
-		writeout();
+	virtual ~textout_ostream_console() {
+		writeout_impl();
+	}
+private:
+	void writeout_impl() {
+		std::string text = pop();
+		if ( text.length() > 0 ) {
+			if ( console ) {
+				#if defined(UNICODE)
+					std::wstring wtext = mpt::transcode<std::wstring>( mpt::common_encoding::utf8, text );
+					WriteConsole( handle, wtext.data(), static_cast<DWORD>( wtext.size() ), NULL, NULL );
+				#else
+					std::string ltext = mpt::transcode<std::string>( mpt::logical_encoding::locale, mpt::common_encoding::utf8, text );
+					WriteConsole( handle, ltext.data(), static_cast<DWORD>( ltext.size() ), NULL, NULL );
+				#endif
+			} else {
+				#if defined(UNICODE)
+					s << mpt::transcode<std::wstring>( mpt::common_encoding::utf8, text );
+				#else
+					s << mpt::transcode<std::string>( mpt::logical_encoding::locale, mpt::common_encoding::utf8, text );
+				#endif
+				s.flush();
+			}
+		}
 	}
 public:
-	void write( const std::string & text ) override {
-		#if defined(UNICODE)
-			std::wstring wtext = utf8_to_wstring( text );
-			WriteConsole( handle, wtext.data(), static_cast<DWORD>( wtext.size() ), NULL, NULL );
-		#else
-			WriteConsole( handle, text.data(), static_cast<DWORD>( text.size() ), NULL, NULL );
-		#endif
+	void writeout() override {
+		writeout_impl();
+	}
+	void cursor_up( std::size_t lines ) override {
+		if ( console ) {
+			s.flush();
+			CONSOLE_SCREEN_BUFFER_INFO csbi;
+			ZeroMemory( &csbi, sizeof( CONSOLE_SCREEN_BUFFER_INFO ) );
+			COORD coord_cursor = COORD();
+			if ( GetConsoleScreenBufferInfo( handle, &csbi ) != FALSE ) {
+				coord_cursor = csbi.dwCursorPosition;
+				coord_cursor.X = 1;
+				coord_cursor.Y -= static_cast<SHORT>( lines );
+				SetConsoleCursorPosition( handle, coord_cursor );
+			}
+		}
 	}
 };
 
@@ -232,25 +239,23 @@ static inline std::string get_extension( std::string filename ) {
 	return "";
 }
 
-bool IsTerminal( int fd );
-
-enum Mode {
-	ModeNone,
-	ModeProbe,
-	ModeInfo,
-	ModeUI,
-	ModeBatch,
-	ModeRender
+enum class Mode {
+	None,
+	Probe,
+	Info,
+	UI,
+	Batch,
+	Render
 };
 
 static inline std::string mode_to_string( Mode mode ) {
 	switch ( mode ) {
-		case ModeNone:   return "none"; break;
-		case ModeProbe:  return "probe"; break;
-		case ModeInfo:   return "info"; break;
-		case ModeUI:     return "ui"; break;
-		case ModeBatch:  return "batch"; break;
-		case ModeRender: return "render"; break;
+		case Mode::None:   return "none"; break;
+		case Mode::Probe:  return "probe"; break;
+		case Mode::Info:   return "info"; break;
+		case Mode::UI:     return "ui"; break;
+		case Mode::Batch:  return "batch"; break;
+		case Mode::Render: return "render"; break;
 	}
 	return "";
 }
@@ -322,7 +327,7 @@ struct commandlineflags {
 		}
 	}
 	commandlineflags() {
-		mode = ModeUI;
+		mode = Mode::UI;
 		ui_redraw_interval = default_high;
 		driver = "";
 		device = "";
@@ -335,7 +340,7 @@ struct commandlineflags {
 #else
 		samplerate = 48000;
 		channels = 2;
-		use_float = true;
+		use_float = mpt::float_traits<float>::is_hard && mpt::float_traits<float>::is_ieee754_binary;
 #endif
 		gain = 0;
 		separation = 100;
@@ -358,11 +363,17 @@ struct commandlineflags {
 		terminal_height = 23;
 #endif
 #if defined(WIN32)
-		CONSOLE_SCREEN_BUFFER_INFO csbi;
-		ZeroMemory( &csbi, sizeof( CONSOLE_SCREEN_BUFFER_INFO ) );
-		GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &csbi );
-		terminal_width = csbi.dwSize.X - 1;
-		terminal_height = 23; //csbi.dwSize.Y - 1;
+		terminal_width = 72;
+		terminal_height = 23;
+		HANDLE hStdOutput = GetStdHandle( STD_OUTPUT_HANDLE );
+		if ( ( hStdOutput != NULL ) && ( hStdOutput != INVALID_HANDLE_VALUE ) ) {
+			CONSOLE_SCREEN_BUFFER_INFO csbi;
+			ZeroMemory( &csbi, sizeof( CONSOLE_SCREEN_BUFFER_INFO ) );
+			if ( GetConsoleScreenBufferInfo( hStdOutput, &csbi ) != FALSE ) {
+				terminal_width = std::min( static_cast<int>( 1 + csbi.srWindow.Right - csbi.srWindow.Left ), static_cast<int>( csbi.dwSize.X ) );
+				terminal_height = std::min( static_cast<int>( 1 + csbi.srWindow.Bottom - csbi.srWindow.Top ), static_cast<int>( csbi.dwSize.Y ) );
+			}
+		}
 #else // WIN32
 		if ( isatty( STDERR_FILENO ) ) {
 			if ( std::getenv( "COLUMNS" ) ) {
@@ -435,45 +446,45 @@ struct commandlineflags {
 			}
 		}
 		show_ui = canUI;
-		if ( mode == ModeNone ) {
+		if ( mode == Mode::None ) {
 			if ( canUI ) {
-				mode = ModeUI;
+				mode = Mode::UI;
 			} else {
-				mode = ModeBatch;
+				mode = Mode::Batch;
 			}
 		}
-		if ( mode == ModeUI && !canUI ) {
+		if ( mode == Mode::UI && !canUI ) {
 			throw args_error_exception();
 		}
 		if ( show_progress && !canProgress ) {
 			throw args_error_exception();
 		}
 		switch ( mode ) {
-			case ModeNone:
+			case Mode::None:
 				throw args_error_exception();
 			break;
-			case ModeProbe:
+			case Mode::Probe:
 				show_ui = false;
 				show_progress = false;
 				show_meters = false;
 				show_channel_meters = false;
 				show_pattern = false;
 			break;
-			case ModeInfo:
+			case Mode::Info:
 				show_ui = false;
 				show_progress = false;
 				show_meters = false;
 				show_channel_meters = false;
 				show_pattern = false;
 			break;
-			case ModeUI:
+			case Mode::UI:
 			break;
-			case ModeBatch:
+			case Mode::Batch:
 				show_meters = false;
 				show_channel_meters = false;
 				show_pattern = false;
 			break;
-			case ModeRender:
+			case Mode::Render:
 				show_meters = false;
 				show_channel_meters = false;
 				show_pattern = false;
@@ -499,13 +510,13 @@ struct commandlineflags {
 		if ( output_extension == "auto" ) {
 			output_extension = "";
 		}
-		if ( mode != ModeRender && !output_extension.empty() ) {
+		if ( mode != Mode::Render && !output_extension.empty() ) {
 			throw args_error_exception();
 		}
-		if ( mode == ModeRender && !output_filename.empty() ) {
+		if ( mode == Mode::Render && !output_filename.empty() ) {
 			throw args_error_exception();
 		}
-		if ( mode != ModeRender && !output_filename.empty() ) {
+		if ( mode != Mode::Render && !output_filename.empty() ) {
 			output_extension = get_extension( output_filename );
 		}
 		if ( output_extension.empty() ) {
@@ -555,77 +566,6 @@ public:
 	}
 };
 
-class write_buffers_blocking_wrapper : public write_buffers_interface {
-protected:
-	std::size_t channels;
-	std::size_t sampleQueueMaxFrames;
-	std::deque<float> sampleQueue;
-protected:
-	virtual ~write_buffers_blocking_wrapper() {
-		return;
-	}
-protected:
-	write_buffers_blocking_wrapper( const commandlineflags & flags )
-		: channels(flags.channels)
-		, sampleQueueMaxFrames(0)
-	{
-		return;
-	}
-	void set_queue_size_frames( std::size_t frames ) {
-		sampleQueueMaxFrames = frames;
-	}
-	template < typename Tsample >
-	Tsample pop_queue() {
-		float val = 0.0f;
-		if ( !sampleQueue.empty() ) {
-			val = sampleQueue.front();
-			sampleQueue.pop_front();
-		}
-		return convert_sample_to<Tsample>( val );
-	}
-	template < typename Tsample >
-	void fill_buffer( Tsample * buf, std::size_t framesToRender ) {
-		for ( std::size_t frame = 0; frame < framesToRender; ++frame ) {
-			for ( std::size_t channel = 0; channel < channels; ++channel ) {
-				*buf = pop_queue<Tsample>();
-				buf++;
-			}
-		}
-	}
-private:
-	void wait_for_queue_space() {
-		while ( sampleQueue.size() >= sampleQueueMaxFrames * channels ) {
-			unlock();
-			sleep( 1 );
-			lock();
-		}
-	}
-public:
-	void write( const std::vector<float*> buffers, std::size_t frames ) override {
-		lock();
-		for ( std::size_t frame = 0; frame < frames; ++frame ) {
-			for ( std::size_t channel = 0; channel < channels; ++channel ) {
-				wait_for_queue_space();
-				sampleQueue.push_back( buffers[channel][frame] );
-			}
-		}
-		unlock();
-	}
-	void write( const std::vector<std::int16_t*> buffers, std::size_t frames ) override {
-		lock();
-		for ( std::size_t frame = 0; frame < frames; ++frame ) {
-			for ( std::size_t channel = 0; channel < channels; ++channel ) {
-				wait_for_queue_space();
-				sampleQueue.push_back( buffers[channel][frame] * (1.0f/32768.0f) );
-			}
-		}
-		unlock();
-	}
-	virtual void lock() = 0;
-	virtual void unlock() = 0;
-	bool sleep( int ms ) override = 0;
-};
-
 class write_buffers_polling_wrapper : public write_buffers_interface {
 protected:
 	std::size_t channels;
