@@ -10,10 +10,7 @@
 #ifndef OPENMPT123_CONFIG_HPP
 #define OPENMPT123_CONFIG_HPP
 
-#if defined(HAVE_CONFIG_H)
-// wrapper for autoconf macros
-#include "config.h"
-#endif // HAVE_CONFIG_H
+#define MPT_INLINE_NS mpt_openmpt123
 
 #if defined(_WIN32)
 #ifndef WIN32
@@ -28,24 +25,22 @@
 #ifndef NOMINMAX
 #define NOMINMAX
 #endif
+#ifdef _UNICODE
 #ifndef UNICODE
 #define UNICODE
 #endif
+#endif
+#ifdef UNICODE
 #ifndef _UNICODE
 #define _UNICODE
 #endif
+#endif
 #endif // WIN32
 
 #if defined(WIN32)
 #define MPT_WITH_MMIO
 #endif // WIN32
 
-#if defined(_MSC_VER)
-
-#pragma warning( disable : 4996 ) // 'foo': The POSIX name for this item is deprecated. Instead, use the ISO C++ conformant name: _foo. See online help for details.
-
-#endif // _MSC_VER
-
 #if defined(MPT_BUILD_MSVC)
 
 #define MPT_WITH_FLAC
@@ -57,16 +52,6 @@
 
 #endif // MPT_BUILD_MSVC
 
-#if defined(MPT_WITH_SDL) && defined(MPT_WITH_SDL2)
-#error "MPT_WITH_SDL2 and MPT_WITH_SDL are mutually exclusive."
-#endif
-
-#if defined(MPT_WITH_SDL)
-#ifndef MPT_NEEDS_THREADS
-#define MPT_NEEDS_THREADS
-#endif
-#endif
-
 #define OPENMPT123_VERSION_STRING OPENMPT_API_VERSION_STRING
 
 #endif // OPENMPT123_CONFIG_HPP
