@@ -14,11 +14,9 @@
 #ifndef NO_REVERB
 #include "Reverb.h"
 #include "../soundlib/MixerLoops.h"
+#include "mpt/base/numbers.hpp"
 
-#ifdef ENABLE_MMX
-#include <mmintrin.h>
-#endif
-#ifdef ENABLE_SSE2
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
 #include <emmintrin.h>
 #endif
 
@@ -31,40 +29,22 @@ OPENMPT_NAMESPACE_BEGIN
 #ifndef NO_REVERB
 
 
-#ifdef ENABLE_MMX
-// Load two 32-bit values
-static MPT_FORCEINLINE __m64 Load64MMX(const int32 *x) { return _mm_set_pi32(x[1], x[0]); }
-// Load four 16-bit values
-static MPT_FORCEINLINE __m64 Load64MMX(const LR16 (&x)[2]) { return Load64MMX(&x->lr); }
-// Store 64-bit value from register (MSVC does not have_mm_cvtsi64_si64x) - macro to avoid emms warnings
-#define Store64MMX(dst, src) \
-	MPT_DO \
-	{ \
-		STATIC_ASSERT(sizeof((dst)[0]) == 4); \
-		(dst)[0] = _mm_cvtsi64_si32(src); \
-		(dst)[1] = _mm_cvtsi64_si32(_mm_unpackhi_pi32(src, src)); \
-	} MPT_WHILE_0
-#endif
-#ifdef ENABLE_SSE2
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
 // Load two 32-bit values
 static MPT_FORCEINLINE __m128i Load64SSE(const int32 *x) { return _mm_loadl_epi64(reinterpret_cast<const __m128i *>(x)); }
 // Load four 16-bit values
-static MPT_FORCEINLINE __m128i Load64SSE(const LR16 (&x)[2]) { return _mm_loadl_epi64(reinterpret_cast<const __m128i *>(&x)); }
+static MPT_FORCEINLINE __m128i Load64SSE(const LR16 (&x)[2]) { return _mm_loadl_epi64(&reinterpret_cast<const __m128i &>(x)); }
 // Store two 32-bit or four 16-bit values from register
 static MPT_FORCEINLINE void Store64SSE(int32 *dst, __m128i src) { return _mm_storel_epi64(reinterpret_cast<__m128i *>(dst), src); }
-static MPT_FORCEINLINE void Store64SSE(LR16 *dst, __m128i src) { return _mm_storel_epi64(reinterpret_cast<__m128i *>(dst), src); }
+static MPT_FORCEINLINE void Store64SSE(LR16 (&dst)[2], __m128i src) { return _mm_storel_epi64(&reinterpret_cast<__m128i &>(dst), src); }
 #endif
 
 
 CReverb::CReverb()
 {
-	// Shared reverb state
-	InitMixBuffer(MixReverbBuffer, static_cast<uint32>(mpt::size(MixReverbBuffer)));
-
 	// Reverb mix buffers
 	MemsetZero(g_RefDelay);
 	MemsetZero(g_LateReverb);
-
 }
 
 
@@ -74,7 +54,7 @@ static int32 OnePoleLowPassCoef(int32 scale, float g, float F_c, float F_s)
 
 	g *= g;
 	double scale_over_1mg = scale / (1.0 - g);
-	double cosw = std::cos(2.0 * M_PI * F_c / F_s);
+	double cosw = std::cos((2.0 * mpt::numbers::pi) * F_c / F_s);
 	return mpt::saturate_round<int32>((1.0 - (std::sqrt((g + g) * (1.0 - cosw) - g * g * (1.0 - cosw * cosw)) + g * cosw)) * scale_over_1mg);
 }
 
@@ -92,64 +72,56 @@ static int32 mBToLinear(int32 scale, int32 value_mB)
 	return mpt::saturate_round<int32>(mBToLinear(value_mB) * scale);
 }
 
-
-struct SNDMIX_REVERB_PROPERTIES
+static constexpr std::pair<SNDMIX_REVERB_PROPERTIES, const char *> ReverbPresets[NUM_REVERBTYPES] =
 {
-	int32 lRoom;                   // [-10000, 0]      default: -10000 mB
-	int32 lRoomHF;                 // [-10000, 0]      default: 0 mB
-	float flDecayTime;             // [0.1, 20.0]      default: 1.0 s
-	float flDecayHFRatio;          // [0.1, 2.0]       default: 0.5
-	int32 lReflections;            // [-10000, 1000]   default: -10000 mB
-	float flReflectionsDelay;      // [0.0, 0.3]       default: 0.02 s
-	int32 lReverb;                 // [-10000, 2000]   default: -10000 mB
-	float flReverbDelay;           // [0.0, 0.1]       default: 0.04 s
-	float flDiffusion;             // [0.0, 100.0]     default: 100.0 %
-	float flDensity;               // [0.0, 100.0]     default: 100.0 %
+	// Examples simulating General MIDI 2'musical' reverb presets
+	// Name  (Decay time)  Description
+	// Plate       (1.3s)  A plate reverb simulation.
+	{{ -1000, -200, 1.30f,0.90f,     0,0.002f,     0,0.010f,100.0f, 75.0f }, "GM Plate"},
+	// Small Room  (1.1s)  A small size room with a length of 5m or so.
+	{{ -1000, -600, 1.10f,0.83f,  -400,0.005f,   500,0.010f,100.0f,100.0f }, "GM Small Room"},
+	// Medium Room (1.3s)  A medium size room with a length of 10m or so.
+	{{ -1000, -600, 1.30f,0.83f, -1000,0.010f,  -200,0.020f,100.0f,100.0f }, "GM Medium Room"},
+	// Large Room  (1.5s)  A large size room suitable for live performances.
+	{{ -1000, -600, 1.50f,0.83f, -1600,0.020f, -1000,0.040f,100.0f,100.0f }, "GM Large Room"},
+	// Medium Hall (1.8s)  A medium size concert hall.
+	{{ -1000, -600, 1.80f,0.70f, -1300,0.015f,  -800,0.030f,100.0f,100.0f }, "GM Medium Hall"},
+	// Large Hall  (1.8s)  A large size concert hall suitable for a full orchestra.
+	{{ -1000, -600, 1.80f,0.70f, -2000,0.030f, -1400,0.060f,100.0f,100.0f }, "GM Large Hall"},
+
+	{{ -1000, -100, 1.49f,0.83f, -2602,0.007f,   200,0.011f,100.0f,100.0f }, "Generic"},
+	{{ -1000,-6000, 0.17f,0.10f, -1204,0.001f,   207,0.002f,100.0f,100.0f }, "Padded Cell"},
+	{{ -1000, -454, 0.40f,0.83f, -1646,0.002f,    53,0.003f,100.0f,100.0f }, "Room"},
+	{{ -1000,-1200, 1.49f,0.54f,  -370,0.007f,  1030,0.011f,100.0f, 60.0f }, "Bathroom"},
+	{{ -1000,-6000, 0.50f,0.10f, -1376,0.003f, -1104,0.004f,100.0f,100.0f }, "Living Room"},
+	{{ -1000, -300, 2.31f,0.64f,  -711,0.012f,    83,0.017f,100.0f,100.0f }, "Stone Room"},
+	{{ -1000, -476, 4.32f,0.59f,  -789,0.020f,  -289,0.030f,100.0f,100.0f }, "Auditorium"},
+	{{ -1000, -500, 3.92f,0.70f, -1230,0.020f,    -2,0.029f,100.0f,100.0f }, "Concert Hall"},
+	{{ -1000,    0, 2.91f,1.30f,  -602,0.015f,  -302,0.022f,100.0f,100.0f }, "Cave"},
+	{{ -1000, -698, 7.24f,0.33f, -1166,0.020f,    16,0.030f,100.0f,100.0f }, "Arena"},
+	{{ -1000,-1000,10.05f,0.23f,  -602,0.020f,   198,0.030f,100.0f,100.0f }, "Hangar"},
+	{{ -1000,-4000, 0.30f,0.10f, -1831,0.002f, -1630,0.030f,100.0f,100.0f }, "Carpeted Hallway"},
+	{{ -1000, -300, 1.49f,0.59f, -1219,0.007f,   441,0.011f,100.0f,100.0f }, "Hallway"},
+	{{ -1000, -237, 2.70f,0.79f, -1214,0.013f,   395,0.020f,100.0f,100.0f }, "Stone Corridor"},
+	{{ -1000, -270, 1.49f,0.86f, -1204,0.007f,    -4,0.011f,100.0f,100.0f }, "Alley"},
+	{{ -1000,-3300, 1.49f,0.54f, -2560,0.162f,  -613,0.088f, 79.0f,100.0f }, "Forest"},
+	{{ -1000, -800, 1.49f,0.67f, -2273,0.007f, -2217,0.011f, 50.0f,100.0f }, "City"},
+	{{ -1000,-2500, 1.49f,0.21f, -2780,0.300f, -2014,0.100f, 27.0f,100.0f }, "Mountains"},
+	{{ -1000,-1000, 1.49f,0.83f,-10000,0.061f,   500,0.025f,100.0f,100.0f }, "Quarry"},
+	{{ -1000,-2000, 1.49f,0.50f, -2466,0.179f, -2514,0.100f, 21.0f,100.0f }, "Plain"},
+	{{ -1000,    0, 1.65f,1.50f, -1363,0.008f, -1153,0.012f,100.0f,100.0f }, "Parking Lot"},
+	{{ -1000,-1000, 2.81f,0.14f,   429,0.014f,   648,0.021f, 80.0f, 60.0f }, "Sewer Pipe"},
+	{{ -1000,-4000, 1.49f,0.10f,  -449,0.007f,  1700,0.011f,100.0f,100.0f }, "Underwater"},
 };
 
-struct SNDMIX_RVBPRESET
+mpt::ustring GetReverbPresetName(uint32 preset)
 {
-	SNDMIX_REVERB_PROPERTIES Preset;
-	const MPT_UCHAR_TYPE *name;
-};
+	return (preset < NUM_REVERBTYPES) ? mpt::ToUnicode(mpt::Charset::ASCII, ReverbPresets[preset].second) : mpt::ustring{};
+}
 
-
-static const SNDMIX_RVBPRESET gRvbPresets[NUM_REVERBTYPES] =
+const SNDMIX_REVERB_PROPERTIES *GetReverbPreset(uint32 preset)
 {
-	{{ SNDMIX_REVERB_PRESET_PLATE },           UL_("GM Plate")},
-	{{ SNDMIX_REVERB_PRESET_SMALLROOM },       UL_("GM Small Room")},
-	{{ SNDMIX_REVERB_PRESET_MEDIUMROOM },      UL_("GM Medium Room")},
-	{{ SNDMIX_REVERB_PRESET_LARGEROOM },       UL_("GM Large Room")},
-	{{ SNDMIX_REVERB_PRESET_MEDIUMHALL },      UL_("GM Medium Hall")},
-	{{ SNDMIX_REVERB_PRESET_LARGEHALL },       UL_("GM Large Hall")},
-	{{ SNDMIX_REVERB_PRESET_GENERIC },         UL_("Generic")},
-	{{ SNDMIX_REVERB_PRESET_PADDEDCELL },      UL_("Padded Cell")},
-	{{ SNDMIX_REVERB_PRESET_ROOM },            UL_("Room")},
-	{{ SNDMIX_REVERB_PRESET_BATHROOM },        UL_("Bathroom")},
-	{{ SNDMIX_REVERB_PRESET_LIVINGROOM },      UL_("Living Room")},
-	{{ SNDMIX_REVERB_PRESET_STONEROOM },       UL_("Stone Room")},
-	{{ SNDMIX_REVERB_PRESET_AUDITORIUM },      UL_("Auditorium")},
-	{{ SNDMIX_REVERB_PRESET_CONCERTHALL },     UL_("Concert Hall")},
-	{{ SNDMIX_REVERB_PRESET_CAVE },            UL_("Cave")},
-	{{ SNDMIX_REVERB_PRESET_ARENA },           UL_("Arena")},
-	{{ SNDMIX_REVERB_PRESET_HANGAR },          UL_("Hangar")},
-	{{ SNDMIX_REVERB_PRESET_CARPETEDHALLWAY }, UL_("Carpeted Hallway")},
-	{{ SNDMIX_REVERB_PRESET_HALLWAY },         UL_("Hallway")},
-	{{ SNDMIX_REVERB_PRESET_STONECORRIDOR },   UL_("Stone Corridor")},
-	{{ SNDMIX_REVERB_PRESET_ALLEY },           UL_("Alley")},
-	{{ SNDMIX_REVERB_PRESET_FOREST },          UL_("Forest")},
-	{{ SNDMIX_REVERB_PRESET_CITY },            UL_("City")},
-	{{ SNDMIX_REVERB_PRESET_MOUNTAINS },       UL_("Mountains")},
-	{{ SNDMIX_REVERB_PRESET_QUARRY },          UL_("Quarry")},
-	{{ SNDMIX_REVERB_PRESET_PLAIN },           UL_("Plain")},
-	{{ SNDMIX_REVERB_PRESET_PARKINGLOT },      UL_("Parking Lot")},
-	{{ SNDMIX_REVERB_PRESET_SEWERPIPE },       UL_("Sewer Pipe")},
-	{{ SNDMIX_REVERB_PRESET_UNDERWATER },      UL_("Underwater")},
-};
-
-mpt::ustring GetReverbPresetName(uint32 nPreset)
-{
-	return (nPreset < NUM_REVERBTYPES) ? mpt::ustring(gRvbPresets[nPreset].name) : mpt::ustring();
+	return (preset < NUM_REVERBTYPES) ? &ReverbPresets[preset].first : nullptr;
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -271,9 +243,9 @@ static void I3dl2_to_Generic(
 }
 
 
-void CReverb::Shutdown()
+void CReverb::Shutdown(MixSampleInt &gnRvbROfsVol, MixSampleInt &gnRvbLOfsVol)
 {
-	gnReverbSend = 0;
+	gnReverbSend = false;
 
 	gnRvbLOfsVol = 0;
 	gnRvbROfsVol = 0;
@@ -298,10 +270,10 @@ void CReverb::Shutdown()
 }
 
 
-void CReverb::Initialize(bool bReset, uint32 MixingFreq)
+void CReverb::Initialize(bool bReset, MixSampleInt &gnRvbROfsVol, MixSampleInt &gnRvbLOfsVol, uint32 MixingFreq)
 {
 	if (m_Settings.m_nReverbType >= NUM_REVERBTYPES) m_Settings.m_nReverbType = 0;
-	const SNDMIX_REVERB_PROPERTIES *rvbPreset = &gRvbPresets[m_Settings.m_nReverbType].Preset;
+	const SNDMIX_REVERB_PROPERTIES *rvbPreset = &ReverbPresets[m_Settings.m_nReverbType].first;
 
 	if ((rvbPreset != m_currentPreset) || (bReset))
 	{
@@ -396,7 +368,7 @@ void CReverb::Initialize(bool bReset, uint32 MixingFreq)
 	if (bReset)
 	{
 		gnReverbSamples = 0;
-		Shutdown();
+		Shutdown(gnRvbROfsVol, gnRvbLOfsVol);
 	}
 	// Wait at least 5 seconds before shutting down the reverb
 	if (gnReverbDecaySamples < MixingFreq*5)
@@ -406,26 +378,25 @@ void CReverb::Initialize(bool bReset, uint32 MixingFreq)
 }
 
 
-mixsample_t *CReverb::GetReverbSendBuffer(uint32 nSamples)
+void CReverb::TouchReverbSendBuffer(MixSampleInt *MixReverbBuffer, MixSampleInt &gnRvbROfsVol, MixSampleInt &gnRvbLOfsVol, uint32 nSamples)
 {
 	if(!gnReverbSend)
 	{ // and we did not clear the buffer yet, do it now because we will get new data
 		StereoFill(MixReverbBuffer, nSamples, gnRvbROfsVol, gnRvbLOfsVol);
 	}
-	gnReverbSend = 1; // we will have to process reverb
-	return MixReverbBuffer;
+	gnReverbSend = true; // we will have to process reverb
 }
 
 
 // Reverb
-void CReverb::Process(mixsample_t *MixSoundBuffer, uint32 nSamples)
+void CReverb::Process(MixSampleInt *MixSoundBuffer, MixSampleInt *MixReverbBuffer, MixSampleInt &gnRvbROfsVol, MixSampleInt &gnRvbLOfsVol, uint32 nSamples)
 {
 	if((!gnReverbSend) && (!gnReverbSamples))
 	{ // no data is sent to reverb and reverb decayed completely
 		return;
 	}
 	if(!gnReverbSend)
-	{ // no input data in MixReverbBuffer, so the buffer got not cleared in GetReverbSendBuffer(), do it now for decay
+	{ // no input data in MixReverbBuffer, so the buffer got not cleared in TouchReverbSendBuffer(), do it now for decay
 		StereoFill(MixReverbBuffer, nSamples, gnRvbROfsVol, gnRvbLOfsVol);
 	}
 
@@ -458,7 +429,7 @@ void CReverb::Process(mixsample_t *MixSoundBuffer, uint32 nSamples)
 	if (nIn > 0) ProcessPreDelay(&g_RefDelay, MixReverbBuffer, nIn);
 	// Process Reverb Reflections and Late Reverberation
 	int32 *pRvbOut = MixReverbBuffer;
-	uint32 nRvbSamples = nOut, nCount = 0;
+	uint32 nRvbSamples = nOut;
 	while (nRvbSamples > 0)
 	{
 		uint32 nPosRef = g_RefDelay.nRefOutPos & SNDMIX_REVERB_DELAY_MASK;
@@ -476,7 +447,6 @@ void CReverb::Process(mixsample_t *MixSoundBuffer, uint32 nSamples)
 		// Update delay positions
 		g_RefDelay.nRefOutPos = (g_RefDelay.nRefOutPos + n) & SNDMIX_REVERB_DELAY_MASK;
 		g_RefDelay.nDelayPos = (g_RefDelay.nDelayPos + n) & SNDMIX_REFLECTIONS_DELAY_MASK;
-		nCount += n*2;
 		pRvbOut += n*2;
 		nRvbSamples -= n;
 	}
@@ -489,10 +459,10 @@ void CReverb::Process(mixsample_t *MixSoundBuffer, uint32 nSamples)
 	else if(gnReverbSamples > nSamples) gnReverbSamples -= nSamples; // decay
 	else // decayed
 	{
-		Shutdown();
+		Shutdown(gnRvbROfsVol, gnRvbLOfsVol);
 		gnReverbSamples = 0;
 	}
-	gnReverbSend = 0; // no input data in MixReverbBuffer
+	gnReverbSend = false; // no input data in MixReverbBuffer
 }
 
 
@@ -618,28 +588,27 @@ void CReverb::ReverbProcessPostFiltering2x(const int32 * MPT_RESTRICT pRvb, int3
 // Stereo Add + DC removal
 void CReverb::ReverbProcessPostFiltering1x(const int32 * MPT_RESTRICT pRvb, int32 * MPT_RESTRICT pDry, uint32 nSamples)
 {
-#ifdef ENABLE_MMX
-	if(GetProcSupport() & PROCSUPPORT_MMX)
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
+	if(CPU::HasFeatureSet(CPU::feature::sse2))
 	{
-		__m64 nDCRRvb_Y1 = Load64MMX(gnDCRRvb_Y1);
-		__m64 nDCRRvb_X1 = Load64MMX(gnDCRRvb_X1);
-		__m64 in = _mm_set1_pi32(0);
+		__m128i nDCRRvb_Y1 = Load64SSE(gnDCRRvb_Y1);
+		__m128i nDCRRvb_X1 = Load64SSE(gnDCRRvb_X1);
+		__m128i in = _mm_set1_epi32(0);
 		while(nSamples--)
 		{
-			in = Load64MMX(pRvb);
+			in = Load64SSE(pRvb);
 			pRvb += 2;
 			// x(n-1) - x(n)
-			__m64 diff = _mm_sub_pi32(nDCRRvb_X1, in);
-			nDCRRvb_X1 = _mm_add_pi32(nDCRRvb_Y1, _mm_sub_pi32(_mm_srai_pi32(diff, DCR_AMOUNT + 1), diff));
-			__m64 out = _mm_add_pi32(Load64MMX(pDry), nDCRRvb_X1);
-			nDCRRvb_Y1 = _mm_sub_pi32(nDCRRvb_X1, _mm_srai_pi32(nDCRRvb_X1, DCR_AMOUNT));
+			__m128i diff = _mm_sub_epi32(nDCRRvb_X1, in);
+			nDCRRvb_X1 = _mm_add_epi32(nDCRRvb_Y1, _mm_sub_epi32(_mm_srai_epi32(diff, DCR_AMOUNT + 1), diff));
+			__m128i out = _mm_add_epi32(Load64SSE(pDry), nDCRRvb_X1);
+			nDCRRvb_Y1 = _mm_sub_epi32(nDCRRvb_X1, _mm_srai_epi32(nDCRRvb_X1, DCR_AMOUNT));
 			nDCRRvb_X1 = in;
-			Store64MMX(pDry, out);
+			Store64SSE(pDry, out);
 			pDry += 2;
 		}
-		Store64MMX(gnDCRRvb_X1, in);
-		Store64MMX(gnDCRRvb_Y1, nDCRRvb_Y1);
-		_mm_empty();
+		Store64SSE(gnDCRRvb_X1, in);
+		Store64SSE(gnDCRRvb_Y1, nDCRRvb_Y1);
 		return;
 	}
 #endif
@@ -681,24 +650,23 @@ void CReverb::ReverbProcessPostFiltering1x(const int32 * MPT_RESTRICT pRvb, int3
 
 void CReverb::ReverbDCRemoval(int32 * MPT_RESTRICT pBuffer, uint32 nSamples)
 {
-#ifdef ENABLE_MMX
-	if(GetProcSupport() & PROCSUPPORT_MMX)
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
+	if(CPU::HasFeatureSet(CPU::feature::sse2))
 	{
-		__m64 nDCRRvb_Y1 = Load64MMX(gnDCRRvb_Y1);
-		__m64 nDCRRvb_X1 = Load64MMX(gnDCRRvb_X1);
+		__m128i nDCRRvb_Y1 = Load64SSE(gnDCRRvb_Y1);
+		__m128i nDCRRvb_X1 = Load64SSE(gnDCRRvb_X1);
 		while(nSamples--)
 		{
-			__m64 in = Load64MMX(pBuffer);
-			__m64 diff = _mm_sub_pi32(nDCRRvb_X1, in);
-			__m64 out = _mm_add_pi32(nDCRRvb_Y1, _mm_sub_pi32(_mm_srai_pi32(diff, DCR_AMOUNT + 1), diff));
-			Store64MMX(pBuffer, out);
+			__m128i in = Load64SSE(pBuffer);
+			__m128i diff = _mm_sub_epi32(nDCRRvb_X1, in);
+			__m128i out = _mm_add_epi32(nDCRRvb_Y1, _mm_sub_epi32(_mm_srai_epi32(diff, DCR_AMOUNT + 1), diff));
+			Store64SSE(pBuffer, out);
 			pBuffer += 2;
-			nDCRRvb_Y1 = _mm_sub_pi32(out, _mm_srai_pi32(out, DCR_AMOUNT));
+			nDCRRvb_Y1 = _mm_sub_epi32(out, _mm_srai_epi32(out, DCR_AMOUNT));
 			nDCRRvb_X1 = in;
 		}
-		Store64MMX(gnDCRRvb_X1, nDCRRvb_X1);
-		Store64MMX(gnDCRRvb_Y1, nDCRRvb_Y1);
-		_mm_empty();
+		Store64SSE(gnDCRRvb_X1, nDCRRvb_X1);
+		Store64SSE(gnDCRRvb_Y1, nDCRRvb_Y1);
 		return;
 	}
 #endif
@@ -741,14 +709,14 @@ void CReverb::ReverbDCRemoval(int32 * MPT_RESTRICT pBuffer, uint32 nSamples)
 //
 
 // Save some typing
-static MPT_FORCEINLINE int32 Clamp16(int32 x) { return Clamp(x, int16_min, int16_max); }
+static MPT_FORCEINLINE int32 Clamp16(int32 x) { return Clamp(x, std::numeric_limits<int16>::min(), std::numeric_limits<int16>::max()); }
 
 void CReverb::ProcessPreDelay(SWRvbRefDelay * MPT_RESTRICT pPreDelay, const int32 * MPT_RESTRICT pIn, uint32 nSamples)
 {
 	uint32 preDifPos = pPreDelay->nPreDifPos;
 	uint32 delayPos = pPreDelay->nDelayPos - 1;
-#ifdef ENABLE_SSE2
-	if(GetProcSupport() & PROCSUPPORT_SSE2)
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
+	if(CPU::HasFeatureSet(CPU::feature::sse2))
 	{
 		__m128i coeffs = _mm_cvtsi32_si128(pPreDelay->nCoeffs.lr);
 		__m128i history = _mm_cvtsi32_si128(pPreDelay->History.lr);
@@ -773,34 +741,6 @@ void CReverb::ProcessPreDelay(SWRvbRefDelay * MPT_RESTRICT pPreDelay, const int3
 		pPreDelay->History.lr = _mm_cvtsi128_si32(history);
 		return;
 	}
-#endif
-#ifdef ENABLE_MMX
-	if(GetProcSupport() & PROCSUPPORT_MMX)
-	{
-		__m64 coeffs = _mm_cvtsi32_si64(pPreDelay->nCoeffs.lr);
-		__m64 history = _mm_cvtsi32_si64(pPreDelay->History.lr);
-		__m64 preDifCoeffs = _mm_cvtsi32_si64(pPreDelay->nPreDifCoeffs.lr);
-		while(nSamples--)
-		{
-			__m64 in32 = Load64MMX(pIn);				// 16-bit unsaturated reverb input [  r  |  l  ]
-			__m64 inSat = _mm_packs_pi32(in32, in32);	// [ r | l | r | l ] (16-bit saturated)
-			pIn += 2;
-			// Low-pass
-			__m64 lp = _mm_mulhi_pi16(_mm_subs_pi16(history, inSat), coeffs);
-			__m64 preDif = _mm_cvtsi32_si64(pPreDelay->PreDifBuffer[preDifPos].lr);
-			history = _mm_adds_pi16(_mm_adds_pi16(lp, lp), inSat);
-			// Pre-Diffusion
-			preDifPos = (preDifPos + 1) & SNDMIX_PREDIFFUSION_DELAY_MASK;
-			delayPos = (delayPos + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			__m64 preDif2 = _mm_subs_pi16(history, _mm_mulhi_pi16(preDif, preDifCoeffs));
-			pPreDelay->PreDifBuffer[preDifPos].lr = _mm_cvtsi64_si32(preDif2);
-			pPreDelay->RefDelayBuffer[delayPos].lr = _mm_cvtsi64_si32(_mm_adds_pi16(_mm_mulhi_pi16(preDifCoeffs, preDif2), preDif));
-		}
-		pPreDelay->nPreDifPos = preDifPos;
-		pPreDelay->History.lr = _mm_cvtsi64_si32(history);
-		_mm_empty();
-		return;
-	}
 #endif
 	const int32 coeffsL = pPreDelay->nCoeffs.c.l, coeffsR = pPreDelay->nCoeffs.c.r;
 	const int32 preDifCoeffsL = pPreDelay->nPreDifCoeffs.c.l, preDifCoeffsR = pPreDelay->nPreDifCoeffs.c.r;
@@ -847,8 +787,8 @@ void CReverb::ProcessPreDelay(SWRvbRefDelay * MPT_RESTRICT pPreDelay, const int3
 
 void CReverb::ProcessReflections(SWRvbRefDelay * MPT_RESTRICT pPreDelay, LR16 * MPT_RESTRICT pRefOut, int32 * MPT_RESTRICT pOut, uint32 nSamples)
 {
-#ifdef ENABLE_SSE2
-	if(GetProcSupport() & PROCSUPPORT_SSE2)
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
+	if(CPU::HasFeatureSet(CPU::feature::sse2))
 	{
 		union
 		{
@@ -895,75 +835,6 @@ void CReverb::ProcessReflections(SWRvbRefDelay * MPT_RESTRICT pPreDelay, LR16 *
 		}
 		return;
 	}
-#endif
-#ifdef ENABLE_MMX
-	if(GetProcSupport() & PROCSUPPORT_MMX)
-	{
-		// First stage
-		uint32 numSamples = nSamples;
-		const LR16 *refDelayBuffer = pPreDelay->RefDelayBuffer;
-		int pos1 = pPreDelay->nDelayPos - pPreDelay->Reflections[0].Delay - 1;
-		int pos2 = pPreDelay->nDelayPos - pPreDelay->Reflections[1].Delay - 1;
-		int pos3 = pPreDelay->nDelayPos - pPreDelay->Reflections[2].Delay - 1;
-		int pos4 = pPreDelay->nDelayPos - pPreDelay->Reflections[3].Delay - 1;
-		__m64 gain1 = Load64MMX(pPreDelay->Reflections[0].Gains);
-		__m64 gain2 = Load64MMX(pPreDelay->Reflections[1].Gains);
-		__m64 gain3 = Load64MMX(pPreDelay->Reflections[2].Gains);
-		__m64 gain4 = Load64MMX(pPreDelay->Reflections[3].Gains);
-		while(numSamples--)
-		{
-			pos1 = (pos1 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			pos2 = (pos2 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			pos3 = (pos3 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			pos4 = (pos4 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			__m64 ref1 = _mm_cvtsi32_si64(refDelayBuffer[pos1].lr);	// [0 | 0 | r | l ]
-			__m64 ref2 = _mm_cvtsi32_si64(refDelayBuffer[pos2].lr);
-			__m64 ref3 = _mm_cvtsi32_si64(refDelayBuffer[pos3].lr);
-			__m64 ref4 = _mm_cvtsi32_si64(refDelayBuffer[pos4].lr);
-			__m64 refOut = _mm_srai_pi32(_mm_add_pi32(
-				_mm_add_pi32(_mm_madd_pi16(_mm_unpacklo_pi32(ref1, ref1), gain1), _mm_madd_pi16(_mm_unpacklo_pi32(ref2, ref2), gain2)),
-				_mm_add_pi32(_mm_madd_pi16(_mm_unpacklo_pi32(ref3, ref3), gain3), _mm_madd_pi16(_mm_unpacklo_pi32(ref4, ref4), gain4))),
-				15);
-			pRefOut->lr = _mm_cvtsi64_si32(_mm_packs_pi32(refOut, refOut));
-			pRefOut++;
-		}
-
-		// Second stage
-		numSamples = nSamples;
-		pRefOut -= nSamples;
-
-		__m64 refGain = _mm_unpacklo_pi16(_mm_cvtsi32_si64(pPreDelay->ReflectionsGain.lr), _mm_cvtsi32_si64(0));	// [0 | g_r | 0 | g_l]
-		refGain = _mm_srai_pi32(refGain, 3);	// For 28-bit final output: 16+15-3 = 28
-		int pos5 = pPreDelay->nDelayPos - pPreDelay->Reflections[4].Delay - 1;
-		int pos6 = pPreDelay->nDelayPos - pPreDelay->Reflections[5].Delay - 1;
-		int pos7 = pPreDelay->nDelayPos - pPreDelay->Reflections[6].Delay - 1;
-		__m64 gain5 = Load64MMX(pPreDelay->Reflections[4].Gains);
-		__m64 gain6 = Load64MMX(pPreDelay->Reflections[5].Gains);
-		__m64 gain7 = Load64MMX(pPreDelay->Reflections[6].Gains);
-		while(numSamples--)
-		{
-			pos5 = (pos5 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			pos6 = (pos6 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			pos7 = (pos7 + 1) & SNDMIX_REFLECTIONS_DELAY_MASK;
-			__m64 ref5 = _mm_cvtsi32_si64(refDelayBuffer[pos5].lr);	// [0 | 0 | r | l ]
-			__m64 ref6 = _mm_cvtsi32_si64(refDelayBuffer[pos6].lr);
-			__m64 ref7 = _mm_cvtsi32_si64(refDelayBuffer[pos7].lr);
-			__m64 refPrev = _mm_cvtsi32_si64(pRefOut->lr);	// output of previous reflections
-			__m64 refOut = _mm_srai_pi32(_mm_add_pi32(
-				_mm_add_pi32(_mm_madd_pi16(_mm_unpacklo_pi32(ref5, ref5), gain5), _mm_madd_pi16(_mm_unpacklo_pi32(ref7, ref7), gain7)),
-				_mm_madd_pi16(_mm_unpacklo_pi32(ref6, ref6), gain6)),
-				15);
-			refOut = _mm_adds_pi16(_mm_packs_pi32(refOut, refOut), refPrev);
-			pRefOut->lr = _mm_cvtsi64_si32(refOut);	// late reverb stereo input
-			pRefOut++;
-			__m64 out = _mm_madd_pi16(_mm_unpacklo_pi16(refOut, refOut), refGain);	// Apply reflections gain
-			// At this, point, this is the only output of the reverb
-			Store64MMX(pOut, out);
-			pOut += 2;
-		}
-		_mm_empty();
-		return;
-	}
 #endif
 	int pos[7];
 	for(int i = 0; i < 7; i++)
@@ -1011,8 +882,8 @@ void CReverb::ProcessLateReverb(SWLateReverb * MPT_RESTRICT pReverb, LR16 * MPT_
 	// Calculate delay line offset from current delay position
 	#define DELAY_OFFSET(x) ((delayPos - (x)) & RVBDLY_MASK)
 
-#ifdef ENABLE_SSE2
-	if(GetProcSupport() & PROCSUPPORT_SSE2)
+#if defined(MPT_ENABLE_ARCH_INTRINSICS_SSE2)
+	if(CPU::HasFeatureSet(CPU::feature::sse2))
 	{
 		int delayPos = pReverb->nDelayPos & RVBDLY_MASK;
 		__m128i rvbOutGains = Load64SSE(pReverb->RvbOutGains);
@@ -1077,73 +948,6 @@ void CReverb::ProcessLateReverb(SWLateReverb * MPT_RESTRICT pReverb, LR16 * MPT_
 		pReverb->nDelayPos = delayPos;
 		return;
 	}
-#endif
-#ifdef ENABLE_MMX
-	if(GetProcSupport() & PROCSUPPORT_MMX)
-	{
-		int delayPos = pReverb->nDelayPos & RVBDLY_MASK;
-		__m64 rvbOutGains = Load64MMX(pReverb->RvbOutGains);
-		__m64 difCoeffs = Load64MMX(pReverb->nDifCoeffs);
-		__m64 decayLP = Load64MMX(pReverb->nDecayLP);
-		__m64 lpHistory = Load64MMX(pReverb->LPHistory);
-		while(nSamples--)
-		{
-			__m64 refIn = _mm_cvtsi32_si64(pRefOut->lr);	// 16-bit stereo input
-			pRefOut++;
-
-			__m64 delay2 = _mm_unpacklo_pi32(
-				_mm_cvtsi32_si64(pReverb->Delay2[DELAY_OFFSET(RVBDLY2L_LEN)].lr),
-				_mm_cvtsi32_si64(pReverb->Delay2[DELAY_OFFSET(RVBDLY2R_LEN)].lr));
-
-			// Unsigned to avoid sign extension
-			uint16 diff1L = pReverb->Diffusion1[DELAY_OFFSET(RVBDIF1L_LEN)].c.l;
-			uint16 diff1R = pReverb->Diffusion1[DELAY_OFFSET(RVBDIF1R_LEN)].c.r;
-			int32 diffusion1 = diff1L | (diff1R << 16);	// diffusion1 history
-
-			uint16 diff2L = pReverb->Diffusion2[DELAY_OFFSET(RVBDIF2L_LEN)].c.l;
-			uint16 diff2R = pReverb->Diffusion2[DELAY_OFFSET(RVBDIF2R_LEN)].c.r;
-			int32 diffusion2 = diff2L | (diff2R << 16);	// diffusion2 history
-
-			__m64 lpDecay = _mm_mulhi_pi16(_mm_subs_pi16(lpHistory, delay2), decayLP);
-			lpHistory = _mm_adds_pi16(_mm_adds_pi16(lpDecay, lpDecay), delay2);	// Low-passed decay
-
-			// Apply decay gain
-			__m64 histDecay = _mm_srai_pi32(_mm_madd_pi16(Load64MMX(pReverb->nDecayDC), lpHistory), 15);
-			__m64 histDecayIn = _mm_adds_pi16(_mm_packs_pi32(histDecay, histDecay), _mm_srai_pi16(_mm_unpacklo_pi32(refIn, refIn), 2));
-			__m64 histDecayInDiff = _mm_subs_pi16(histDecayIn, _mm_mulhi_pi16(_mm_cvtsi32_si64(diffusion1), difCoeffs));
-			pReverb->Diffusion1[delayPos].lr = _mm_cvtsi64_si32(histDecayInDiff);
-
-			__m64 delay1Out = _mm_adds_pi16(_mm_mulhi_pi16(difCoeffs, histDecayInDiff), _mm_cvtsi32_si64(diffusion1));
-			// Insert the diffusion output in the reverb delay line
-			pReverb->Delay1[delayPos].lr = _mm_cvtsi64_si32(delay1Out);
-			__m64 histDecayInDelay = _mm_adds_pi16(histDecayIn, _mm_unpacklo_pi32(delay1Out, delay1Out));
-
-			// Input to second diffuser
-			__m64 delay1 = _mm_unpacklo_pi32(
-				_mm_cvtsi32_si64(pReverb->Delay1[DELAY_OFFSET(RVBDLY1L_LEN)].lr),
-				_mm_cvtsi32_si64(pReverb->Delay1[DELAY_OFFSET(RVBDLY1R_LEN)].lr));
-
-			__m64 delay1Gains = _mm_srai_pi32(_mm_madd_pi16(delay1, Load64MMX(pReverb->Dif2InGains)), 15);
-			__m64 delay1GainsSat = _mm_packs_pi32(delay1Gains, delay1Gains);
-			__m64 histDelay1 = _mm_subs_pi16(_mm_adds_pi16(histDecayInDelay, delay1), delay1GainsSat);	// accumulate with reverb output
-			__m64 diff2out = _mm_subs_pi16(delay1GainsSat, _mm_mulhi_pi16(_mm_cvtsi32_si64(diffusion2), difCoeffs));
-			__m64 diff2outCoeffs = _mm_mulhi_pi16(difCoeffs, diff2out);
-			pReverb->Diffusion2[delayPos].lr = _mm_cvtsi64_si32(diff2out);
-
-			__m64 mixOut = Load64MMX(pMixOut);
-			__m64 delay2out = _mm_adds_pi16(diff2outCoeffs, _mm_cvtsi32_si64(diffusion2));
-			pReverb->Delay2[delayPos].lr = _mm_cvtsi64_si32(delay2out);
-			delayPos = (delayPos + 1) & RVBDLY_MASK;
-			// Accumulate with reverb output
-			__m64 out = _mm_add_pi32(_mm_madd_pi16(_mm_adds_pi16(histDelay1, delay2out), rvbOutGains), mixOut);
-			Store64MMX(pMixOut, out);
-			pMixOut += 2;
-		}
-		Store64MMX(&pReverb->LPHistory[0].lr, lpHistory);
-		pReverb->nDelayPos = delayPos;
-		_mm_empty();
-		return;
-	}
 #endif
 	int delayPos = pReverb->nDelayPos & RVBDLY_MASK;
 	while(nSamples--)
