@@ -21,7 +21,12 @@
 OPENMPT_NAMESPACE_BEGIN
 
 
-//#define MMCMP_LOG
+#if !defined(MPT_WITH_ANCIENT)
+
+
+#ifdef MPT_ALL_LOGGING
+#define MMCMP_LOG
+#endif
 
 
 struct XPKFILEHEADER
@@ -56,7 +61,7 @@ struct XPK_BufferBounds
 
 static int32 bfextu(std::size_t p, int32 bo, int32 bc, XPK_BufferBounds &bufs)
 {
-	int32 r;
+	uint32 r;
 
 	p += bo / 8;
 	r = bufs.SrcRead(p); p++;
@@ -73,7 +78,7 @@ static int32 bfextu(std::size_t p, int32 bo, int32 bc, XPK_BufferBounds &bufs)
 
 static int32 bfexts(std::size_t p, int32 bo, int32 bc, XPK_BufferBounds &bufs)
 {
-	int32 r;
+	uint32 r;
 
 	p += bo / 8;
 	r = bufs.SrcRead(p); p++;
@@ -82,19 +87,17 @@ static int32 bfexts(std::size_t p, int32 bo, int32 bc, XPK_BufferBounds &bufs)
 	r <<= 8;
 	r |= bufs.SrcRead(p);
 	r <<= (bo % 8) + 8;
-	r >>= 32 - bc;
-
-	return r;
+	return mpt::rshift_signed(static_cast<int32>(r), 32 - bc);
 }
 
 
 static uint8 XPK_ReadTable(int32 index)
 {
-	static const uint8 xpk_table[] = {
+	static constexpr uint8 xpk_table[] = {
 		2,3,4,5,6,7,8,0,3,2,4,5,6,7,8,0,4,3,5,2,6,7,8,0,5,4,6,2,3,7,8,0,6,5,7,2,3,4,8,0,7,6,8,2,3,4,5,0,8,7,6,2,3,4,5,0
 	};
 	if(index < 0) throw XPK_error();
-	if(static_cast<std::size_t>(index) >= mpt::size(xpk_table)) throw XPK_error();
+	if(static_cast<std::size_t>(index) >= std::size(xpk_table)) throw XPK_error();
 	return xpk_table[index];
 }
 
@@ -138,9 +141,9 @@ static bool XPK_DoUnpack(const uint8 *src_, uint32 srcLen, std::vector<char> &un
 
 		if (type != 1)
 		{
-		#ifdef MMCMP_LOG
-			Log("Invalid XPK type! (%d bytes left)\n", len);
-		#endif
+			#ifdef MMCMP_LOG
+				MPT_LOG_GLOBAL(LogDebug, "XPK", MPT_UFORMAT("Invalid XPK type! ({} bytes left)")(len));
+			#endif
 			break;
 		}
 		LimitMax(cup1, len);
@@ -327,7 +330,7 @@ static bool ValidateHeader(const XPKFILEHEADER &header)
 	{
 		return false;
 	}
-	MPT_STATIC_ASSERT(sizeof(XPKFILEHEADER) >= 8);
+	static_assert(sizeof(XPKFILEHEADER) >= 8);
 	if(header.SrcLen < (sizeof(XPKFILEHEADER) - 8))
 	{
 		return false;
@@ -393,19 +396,19 @@ bool UnpackXPK(std::vector<ContainerItem> &containerItems, FileReader &file, Con
 	}
 
 	containerItems.emplace_back();
-	containerItems.back().data_cache = mpt::make_unique<std::vector<char> >();
+	containerItems.back().data_cache = std::make_unique<std::vector<char> >();
 	std::vector<char> & unpackedData = *(containerItems.back().data_cache);
 
-#ifdef MMCMP_LOG
-	Log("XPK detected (SrcLen=%d DstLen=%d) filesize=%d\n", header.SrcLen, header.DstLen, file.GetLength());
-#endif
+	#ifdef MMCMP_LOG
+		MPT_LOG_GLOBAL(LogDebug, "XPK", MPT_UFORMAT("XPK detected (SrcLen={} DstLen={}) filesize={}")(static_cast<uint32>(header.SrcLen), static_cast<uint32>(header.DstLen), file.GetLength()));
+	#endif
 	bool result = false;
 	try
 	{
-		result = XPK_DoUnpack(file.GetRawData<uint8>(), header.SrcLen - (sizeof(XPKFILEHEADER) - 8), unpackedData, header.DstLen);
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+		result = XPK_DoUnpack(file.GetRawData<uint8>().data(), header.SrcLen - (sizeof(XPKFILEHEADER) - 8), unpackedData, header.DstLen);
+	} catch(mpt::out_of_memory e)
 	{
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 		return false;
 	} catch(const XPK_error &)
 	{
@@ -420,4 +423,7 @@ bool UnpackXPK(std::vector<ContainerItem> &containerItems, FileReader &file, Con
 }
 
 
+#endif // !MPT_WITH_ANCIENT
+
+
 OPENMPT_NAMESPACE_END
