@@ -11,7 +11,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 OPENMPT_NAMESPACE_BEGIN
 class CSoundFile;
@@ -23,31 +23,32 @@ OPENMPT_NAMESPACE_BEGIN
 #ifdef MODPLUG_TRACKER
 
 
-#define DLSMAXREGIONS		128
-
 struct DLSREGION
 {
 	uint32 ulLoopStart;
 	uint32 ulLoopEnd;
 	uint16 nWaveLink;
 	uint16 uPercEnv;
-	uint16 usVolume;	// 0..256
-	uint16 fuOptions;	// flags + key group
-	int16  sFineTune;	// +128 = +1 semitone
+	uint16 usVolume;     // 0..256
+	uint16 fuOptions;    // flags + key group
+	int16 sFineTune;     // +128 = +1 semitone
+	int16 panning = -1;  // -1= unset (DLS), otherwise 0...256
 	uint8  uKeyMin;
 	uint8  uKeyMax;
 	uint8  uUnityNote;
 	uint8  tuning = 100;
+
+	constexpr bool IsDummy() const noexcept { return uKeyMin == 0xFF || nWaveLink == Util::MaxValueOfType(nWaveLink); }
 };
 
 struct DLSENVELOPE
 {
 	// Volume Envelope
-	uint16 wVolAttack;		// Attack Time: 0-1000, 1 = 20ms (1/50s) -> [0-20s]
-	uint16 wVolDecay;		// Decay Time: 0-1000, 1 = 20ms (1/50s) -> [0-20s]
-	uint16 wVolRelease;		// Release Time: 0-1000, 1 = 20ms (1/50s) -> [0-20s]
-	uint8 nVolSustainLevel;	// Sustain Level: 0-128, 128=100%	
-	uint8 nDefPan;			// Default Pan
+	uint16 wVolAttack;       // Attack Time: 0-1000, 1 = 20ms (1/50s) -> [0-20s]
+	uint16 wVolDecay;        // Decay Time: 0-1000, 1 = 20ms (1/50s) -> [0-20s]
+	uint16 wVolRelease;      // Release Time: 0-1000, 1 = 20ms (1/50s) -> [0-20s]
+	uint8 nVolSustainLevel;  // Sustain Level: 0-128, 128=100%
+	uint8 nDefPan;           // Default Pan
 };
 
 // Special Bank bits
@@ -55,12 +56,12 @@ struct DLSENVELOPE
 
 struct DLSINSTRUMENT
 {
-	uint32 ulBank, ulInstrument;
-	uint32 nRegions, nMelodicEnv;
-	DLSREGION Regions[DLSMAXREGIONS];
+	uint32 ulBank = 0, ulInstrument = 0;
+	uint32 nMelodicEnv = 0;
+	std::vector<DLSREGION> Regions;
 	char szName[32];
 	// SF2 stuff (DO NOT USE! -> used internally by the SF2 loader)
-	uint16 wPresetBagNdx, wPresetBagNum;
+	uint16 wPresetBagNdx = 0, wPresetBagNum = 0;
 };
 
 struct DLSSAMPLEEX
@@ -72,6 +73,7 @@ struct DLSSAMPLEEX
 	uint32 dwSampleRate;
 	uint8  byOriginalPitch;
 	int8   chPitchCorrection;
+	bool   compressed = false;
 };
 
 
@@ -101,6 +103,7 @@ protected:
 	uint32 m_nType;
 	// DLS Information
 	uint32 m_nMaxWaveLink;
+	uint32 m_sf2version = 0;
 	std::vector<size_t> m_WaveForms;
 	std::vector<DLSINSTRUMENT> m_Instruments;
 	std::vector<DLSSAMPLEEX> m_SamplesEx;
@@ -108,6 +111,9 @@ protected:
 
 public:
 	CDLSBank();
+
+	bool operator==(const CDLSBank &other) const noexcept { return !mpt::PathString::CompareNoCase(m_szFileName, other.m_szFileName); }
+
 	static bool IsDLSBank(const mpt::PathString &filename);
 	static uint32 MakeMelodicCode(uint32 bank, uint32 instr) { return ((bank << 16) | (instr));}
 	static uint32 MakeDrumCode(uint32 rgn, uint32 instr) { return (0x80000000 | (rgn << 16) | (instr));}
@@ -130,7 +136,7 @@ public:
 	bool ExtractSample(CSoundFile &sndFile, SAMPLEINDEX nSample, uint32 nIns, uint32 nRgn, int transpose = 0) const;
 	bool ExtractInstrument(CSoundFile &sndFile, INSTRUMENTINDEX nInstr, uint32 nIns, uint32 nDrumRgn) const;
 	const char *GetRegionName(uint32 nIns, uint32 nRgn) const;
-	uint8 GetPanning(uint32 ins, uint32 region) const;
+	uint16 GetPanning(uint32 ins, uint32 region) const;
 
 // Internal Loader Functions
 protected:
