@@ -11,7 +11,6 @@
 
 #include "stdafx.h"
 #include "Loaders.h"
-#include "ChunkReader.h"
 
 #ifdef LIBOPENMPT_BUILD
 #define MPT_PSM_USE_REAL_SUBSONGS
@@ -27,9 +26,9 @@ OPENMPT_NAMESPACE_BEGIN
 // PSM File Header
 struct PSMFileHeader
 {
-	char     formatID[4];	// "PSM " (new format)
-	uint32le fileSize;		// Filesize - 12
-	char     fileInfoID[4];	// "FILE"
+	char     formatID[4];    // "PSM " (new format)
+	uint32le fileSize;       // Filesize - 12
+	char     fileInfoID[4];  // "FILE"
 };
 
 MPT_BINARY_STRUCT(PSMFileHeader, 12)
@@ -71,9 +70,9 @@ MPT_BINARY_STRUCT(PSMChunk, 8)
 // Song Information
 struct PSMSongHeader
 {
-	char  songType[9];		// Mostly "MAINSONG " (But not in Extreme Pinball!)
-	uint8 compression;		// 1 - uncompressed
-	uint8 numChannels;		// Number of channels
+	char  songType[9];  // Mostly "MAINSONG " (But not in Extreme Pinball!)
+	uint8 compression;  // 1 - uncompressed
+	uint8 numChannels;  // Number of channels
 
 };
 
@@ -83,26 +82,26 @@ MPT_BINARY_STRUCT(PSMSongHeader, 11)
 struct PSMSampleHeader
 {
 	uint8le  flags;
-	char     fileName[8];		// Filename of the original module (without extension)
-	char     sampleID[4];		// INS0...INS9 (only last digit of sample ID, i.e. sample 1 and sample 11 are equal)
+	char     fileName[8];    // Filename of the original module (without extension)
+	char     sampleID[4];    // Identifier like "INS0" (only last digit of sample ID, i.e. sample 1 and sample 11 are equal) or "I0  "
 	char     sampleName[33];
-	uint8le  unknown1[6];		// 00 00 00 00 00 FF
+	uint8le  unknown1[6];    // 00 00 00 00 00 FF
 	uint16le sampleNumber;
 	uint32le sampleLength;
 	uint32le loopStart;
-	uint32le loopEnd;			// FF FF FF FF = end of sample
+	uint32le loopEnd;        // FF FF FF FF = end of sample
 	uint8le  unknown3;
-	uint8le  finetune;		// unused? always 0
+	uint8le  finetune;       // unused? always 0
 	uint8le  defaultVolume;
 	uint32le unknown4;
-	uint32le c5Freq;			// MASI ignores the high 16 bits
-	char     padding[19];		// 00 ... 00
+	uint32le c5Freq;         // MASI ignores the high 16 bits
+	char     padding[19];
 
 	// Convert header data to OpenMPT's internal format
 	void ConvertToMPT(ModSample &mptSmp) const
 	{
 		mptSmp.Initialize();
-		mpt::String::Read<mpt::String::maybeNullTerminated>(mptSmp.filename, fileName);
+		mptSmp.filename = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileName);
 
 		mptSmp.nC5Speed = c5Freq;
 		mptSmp.nLength = sampleLength;
@@ -126,26 +125,26 @@ MPT_BINARY_STRUCT(PSMSampleHeader, 96)
 struct PSMSinariaSampleHeader
 {
 	uint8le  flags;
-	char     fileName[8];		// Filename of the original module (without extension)
-	char     sampleID[8];		// INS0...INS99999
+	char     fileName[8];  // Filename of the original module (without extension)
+	char     sampleID[8];  // INS0...INS99999
 	char     sampleName[33];
-	uint8le  unknown1[6];		// 00 00 00 00 00 FF
+	uint8le  unknown1[6];  // 00 00 00 00 00 FF
 	uint16le sampleNumber;
 	uint32le sampleLength;
 	uint32le loopStart;
 	uint32le loopEnd;
 	uint16le unknown3;
-	uint8le  finetune;		// Possibly finetune like in PSM16, but sounds even worse than just ignoring it
+	uint8le  finetune;     // Appears to be unused
 	uint8le  defaultVolume;
 	uint32le unknown4;
 	uint16le c5Freq;
-	char     padding[16];		// 00 ... 00
+	char     padding[16];
 
 	// Convert header data to OpenMPT's internal format
 	void ConvertToMPT(ModSample &mptSmp) const
 	{
 		mptSmp.Initialize();
-		mpt::String::Read<mpt::String::maybeNullTerminated>(mptSmp.filename, fileName);
+		mptSmp.filename = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, fileName);
 
 		mptSmp.nC5Speed = c5Freq;
 		mptSmp.nLength = sampleLength;
@@ -167,15 +166,13 @@ struct PSMSubSong // For internal use (pattern conversion)
 	std::vector<bool> channelSurround;
 	ORDERINDEX startOrder = ORDERINDEX_INVALID, endOrder = ORDERINDEX_INVALID, restartPos = 0;
 	uint8 defaultTempo = 125, defaultSpeed = 6;
-	char songName[10];
+	char songName[10] = {};
 
 	PSMSubSong()
-	{
-		channelPanning.assign(MAX_BASECHANNELS, 128);
-		channelVolume.assign(MAX_BASECHANNELS, 64);
-		channelSurround.assign(MAX_BASECHANNELS, false);
-		MemsetZero(songName);
-	}
+	    : channelPanning(MAX_BASECHANNELS, 128)
+	    , channelVolume(MAX_BASECHANNELS, 64)
+	    , channelSurround(MAX_BASECHANNELS, false)
+	{ }
 };
 
 
@@ -245,7 +242,7 @@ CSoundFile::ProbeResult CSoundFile::ProbeFileHeaderPSM(MemoryFileReader file, co
 	{
 		return ProbeFailure;
 	}
-	if((chunkHeader.id & 0x7f7f7f7fu) != chunkHeader.id) // ASCII?
+	if((chunkHeader.id & 0x7F7F7F7Fu) != chunkHeader.id) // ASCII?
 	{
 		return ProbeFailure;
 	}
@@ -265,7 +262,7 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 
 #ifdef MPT_PSM_DECRYPT
 	// CONVERT.EXE /K - I don't think any game ever used this.
-	std::vector<mpt::byte> decrypted;
+	std::vector<std::byte> decrypted;
 	if(!memcmp(fileHeader.formatID, "QUP$", 4)
 		&& !memcmp(fileHeader.fileInfoID, "OSWQ", 4))
 	{
@@ -331,21 +328,21 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 		m_nChannels = Clamp(static_cast<CHANNELINDEX>(songHeader.numChannels), m_nChannels, MAX_BASECHANNELS);
 
 		PSMSubSong subsong;
-		mpt::String::Read<mpt::String::nullTerminated>(subsong.songName, songHeader.songType);
+		mpt::String::WriteAutoBuf(subsong.songName) = mpt::String::ReadBuf(mpt::String::nullTerminated, songHeader.songType);
 
 #ifdef MPT_PSM_USE_REAL_SUBSONGS
 		if(!Order().empty())
 		{
 			// Add a new sequence for this subsong
-			if(Order.AddSequence(false) == SEQUENCEINDEX_INVALID)
+			if(Order.AddSequence() == SEQUENCEINDEX_INVALID)
 				break;
 		}
-		Order().SetName(subsong.songName);
+		Order().SetName(mpt::ToUnicode(mpt::Charset::CP437, subsong.songName));
 #endif // MPT_PSM_USE_REAL_SUBSONGS
 
 		// Read "Sub chunks"
 		auto subChunks = chunk.ReadChunks<PSMChunk>(1);
-		for(const auto &subChunkIter : subChunks)
+		for(const auto &subChunkIter : subChunks.chunks)
 		{
 			FileReader subChunk(subChunkIter.GetData());
 			PSMChunk subChunkHead = subChunkIter.GetHeader();
@@ -454,9 +451,7 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 
 						case 0x0D: // Channel panning table - can be set using CONVERT.EXE /E
 							{
-								uint8 chn = subChunk.ReadUint8();
-								uint8 pan = subChunk.ReadUint8();
-								uint8 type = subChunk.ReadUint8();
+								const auto [chn, pan, type] = subChunk.ReadArray<uint8, 3>();
 								if(chn < subsong.channelPanning.size())
 								{
 									switch(type)
@@ -489,8 +484,7 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 
 						case 0x0E: // Channel volume table (0...255) - can be set using CONVERT.EXE /E, is 255 in all "official" PSMs except for some OMF 2097 tracks
 							{
-								uint8 chn = subChunk.ReadUint8();
-								uint8 vol = subChunk.ReadUint8();
+								const auto [chn, vol] = subChunk.ReadArray<uint8, 2>();
 								if(chn < subsong.channelVolume.size())
 								{
 									subsong.channelVolume[chn] = (vol / 4u) + 1;
@@ -516,8 +510,7 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 					if(!subChunk.CanRead(2))
 						break;
 
-					uint8 type = subChunk.ReadUint8();
-					uint8 pan = subChunk.ReadUint8();
+					const auto [type, pan] = subChunk.ReadArray<uint8, 2>();
 					switch(type)
 					{
 					case 0: // use panning
@@ -583,34 +576,28 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 				// Original header
 				PSMSampleHeader sampleHeader;
 				if(!chunk.ReadStruct(sampleHeader))
-				{
 					continue;
-				}
 
 				smp = static_cast<SAMPLEINDEX>(sampleHeader.sampleNumber + 1);
 				if(smp > 0 && smp < MAX_SAMPLES)
 				{
 					m_nSamples = std::max(m_nSamples, smp);
-					mpt::String::Read<mpt::String::nullTerminated>(m_szNames[smp], sampleHeader.sampleName);
-
 					sampleHeader.ConvertToMPT(Samples[smp]);
+					m_szNames[smp] = mpt::String::ReadBuf(mpt::String::nullTerminated, sampleHeader.sampleName);
 				}
 			} else
 			{
 				// Sinaria uses a slightly different sample header
 				PSMSinariaSampleHeader sampleHeader;
 				if(!chunk.ReadStruct(sampleHeader))
-				{
 					continue;
-				}
 
 				smp = static_cast<SAMPLEINDEX>(sampleHeader.sampleNumber + 1);
 				if(smp > 0 && smp < MAX_SAMPLES)
 				{
 					m_nSamples = std::max(m_nSamples, smp);
-					mpt::String::Read<mpt::String::nullTerminated>(m_szNames[smp], sampleHeader.sampleName);
-
 					sampleHeader.ConvertToMPT(Samples[smp]);
+					m_szNames[smp] = mpt::String::ReadBuf(mpt::String::nullTerminated, sampleHeader.sampleName);
 				}
 			}
 			if(smp > 0 && smp < MAX_SAMPLES)
@@ -638,7 +625,7 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 
 	m_modFormat.formatName = sinariaFormat ? U_("Epic MegaGames MASI (New Version / Sinaria)") : U_("Epic MegaGames MASI (New Version)");
 	m_modFormat.type = U_("psm");
-	m_modFormat.charset = mpt::CharsetCP437;
+	m_modFormat.charset = mpt::Charset::CP437;
 
 	if(!(loadFlags & loadPatternData) || m_nChannels == 0)
 	{
@@ -687,10 +674,9 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 
 			while(rowChunk.CanRead(3))
 			{
-				uint8 flags = rowChunk.ReadUint8();
-				uint8 channel = rowChunk.ReadUint8();
+				const auto [flags, channel] = rowChunk.ReadArray<uint8, 2>();
 				// Point to the correct channel
-				ModCommand &m = rowBase[std::min<CHANNELINDEX>(m_nChannels - 1, channel)];
+				ModCommand &m = rowBase[std::min(static_cast<CHANNELINDEX>(m_nChannels - 1), static_cast<CHANNELINDEX>(channel))];
 
 				if(flags & noteFlag)
 				{
@@ -720,17 +706,17 @@ bool CSoundFile::ReadPSM(FileReader &file, ModLoadingFlags loadFlags)
 					// Volume present
 					uint8 vol = rowChunk.ReadUint8();
 					m.volcmd = VOLCMD_VOLUME;
-					m.vol = (MIN(vol, 127) + 1) / 2;
+					m.vol = (std::min(vol, uint8(127)) + 1) / 2;
 				}
 
 				if(flags & effectFlag)
 				{
 					// Effect present - convert
-					m.command = rowChunk.ReadUint8();
-					m.param = rowChunk.ReadUint8();
+					const auto [command, param] = rowChunk.ReadArray<uint8, 2>();
+					m.param = param;
 
 					// This list is annoyingly similar to PSM16, but not quite identical.
-					switch(m.command)
+					switch(command)
 					{
 					// Volslides
 					case 0x01: // fine volslide up
@@ -1014,7 +1000,7 @@ struct PSM16SampleHeader
 	void ConvertToMPT(ModSample &mptSmp) const
 	{
 		mptSmp.Initialize();
-		mpt::String::Read<mpt::String::nullTerminated>(mptSmp.filename, filename);
+		mptSmp.filename = mpt::String::ReadBuf(mpt::String::nullTerminated, filename);
 
 		mptSmp.nLength = length;
 		mptSmp.nLoopStart = loopStart;
@@ -1024,7 +1010,7 @@ struct PSM16SampleHeader
 		mptSmp.nC5Speed = c2freq;
 		mptSmp.Transpose(((finetune ^ 0x08) - 0x78) / (12.0 * 16.0));
 
-		mptSmp.nVolume = std::min<uint8>(volume, 64u) * 4u;
+		mptSmp.nVolume = std::min(volume.get(), uint8(64)) * 4u;
 
 		mptSmp.uFlags.reset();
 		if(flags & PSM16SampleHeader::smp16Bit)
@@ -1132,7 +1118,7 @@ bool CSoundFile::ReadPSM16(FileReader &file, ModLoadingFlags loadFlags)
 	
 	m_modFormat.formatName = U_("Epic MegaGames MASI (Old Version)");
 	m_modFormat.type = U_("psm");
-	m_modFormat.charset = mpt::CharsetCP437;
+	m_modFormat.charset = mpt::Charset::CP437;
 
 	m_nChannels = Clamp(CHANNELINDEX(fileHeader.numChannelsPlay), CHANNELINDEX(fileHeader.numChannelsReal), MAX_BASECHANNELS);
 	m_nSamplePreAmp = fileHeader.masterVolume;
@@ -1144,7 +1130,7 @@ bool CSoundFile::ReadPSM16(FileReader &file, ModLoadingFlags loadFlags)
 	m_nDefaultSpeed = fileHeader.songSpeed;
 	m_nDefaultTempo.Set(fileHeader.songTempo);
 
-	mpt::String::Read<mpt::String::spacePadded>(m_songName, fileHeader.songName);
+	m_songName = mpt::String::ReadBuf(mpt::String::spacePadded, fileHeader.songName);
 
 	// Read orders
 	if(fileHeader.orderOffset > 4 && file.Seek(fileHeader.orderOffset - 4) && file.ReadMagic("PORD"))
@@ -1176,13 +1162,13 @@ bool CSoundFile::ReadPSM16(FileReader &file, ModLoadingFlags loadFlags)
 				break;
 			}
 
-			SAMPLEINDEX smp = sampleHeader.sampleNumber;
-			if(smp > 0 && smp < MAX_SAMPLES)
+			const SAMPLEINDEX smp = sampleHeader.sampleNumber;
+			if(smp > 0 && smp < MAX_SAMPLES && !Samples[smp].HasSampleData())
 			{
 				m_nSamples = std::max(m_nSamples, smp);
 
 				sampleHeader.ConvertToMPT(Samples[smp]);
-				mpt::String::Read<mpt::String::nullTerminated>(m_szNames[smp], sampleHeader.name);
+				m_szNames[smp] = mpt::String::ReadBuf(mpt::String::nullTerminated, sampleHeader.name);
 
 				if(loadFlags & loadSampleData)
 				{
@@ -1241,13 +1227,14 @@ bool CSoundFile::ReadPSM16(FileReader &file, ModLoadingFlags loadFlags)
 					continue;
 				}
 
-				ModCommand &m = *Patterns[pat].GetpModCommand(curRow, std::min<CHANNELINDEX>(chnFlag & channelMask, m_nChannels - 1));
+				ModCommand &m = *Patterns[pat].GetpModCommand(curRow, std::min(static_cast<CHANNELINDEX>(chnFlag & channelMask), static_cast<CHANNELINDEX>(m_nChannels - 1)));
 
 				if(chnFlag & noteFlag)
 				{
 					// note + instr present
-					m.note = patternChunk.ReadUint8() + 36;
-					m.instr = patternChunk.ReadUint8();
+					const auto [note, instr] = patternChunk.ReadArray<uint8, 2>();
+					m.note = note + 36;
+					m.instr = instr;
 				}
 				if(chnFlag & volFlag)
 				{
@@ -1258,10 +1245,10 @@ bool CSoundFile::ReadPSM16(FileReader &file, ModLoadingFlags loadFlags)
 				if(chnFlag & effectFlag)
 				{
 					// effect present - convert
-					m.command = patternChunk.ReadUint8();
-					m.param = patternChunk.ReadUint8();
+					const auto [command, param] = patternChunk.ReadArray<uint8, 2>();
+					m.param = param;
 
-					switch(m.command)
+					switch(command)
 					{
 					// Volslides
 					case 0x01: // fine volslide up
