@@ -13,6 +13,9 @@
 #include "S3MTools.h"
 #include "ITTools.h"
 #ifndef MODPLUG_NO_FILESAVE
+#include "mpt/io/base.hpp"
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_stdstream.hpp"
 #include "../common/mptFileIO.h"
 #ifdef MODPLUG_TRACKER
 #include "../mptrack/TrackerSettings.h"
@@ -28,37 +31,43 @@ void CSoundFile::S3MConvert(ModCommand &m, bool fromIT)
 {
 	switch(m.command | 0x40)
 	{
-	case 'A':	m.command = CMD_SPEED; break;
-	case 'B':	m.command = CMD_POSITIONJUMP; break;
-	case 'C':	m.command = CMD_PATTERNBREAK; if (!fromIT) m.param = (m.param >> 4) * 10 + (m.param & 0x0F); break;
-	case 'D':	m.command = CMD_VOLUMESLIDE; break;
-	case 'E':	m.command = CMD_PORTAMENTODOWN; break;
-	case 'F':	m.command = CMD_PORTAMENTOUP; break;
-	case 'G':	m.command = CMD_TONEPORTAMENTO; break;
-	case 'H':	m.command = CMD_VIBRATO; break;
-	case 'I':	m.command = CMD_TREMOR; break;
-	case 'J':	m.command = CMD_ARPEGGIO; break;
-	case 'K':	m.command = CMD_VIBRATOVOL; break;
-	case 'L':	m.command = CMD_TONEPORTAVOL; break;
-	case 'M':	m.command = CMD_CHANNELVOLUME; break;
-	case 'N':	m.command = CMD_CHANNELVOLSLIDE; break;
-	case 'O':	m.command = CMD_OFFSET; break;
-	case 'P':	m.command = CMD_PANNINGSLIDE; break;
-	case 'Q':	m.command = CMD_RETRIG; break;
-	case 'R':	m.command = CMD_TREMOLO; break;
-	case 'S':	m.command = CMD_S3MCMDEX; break;
-	case 'T':	m.command = CMD_TEMPO; break;
-	case 'U':	m.command = CMD_FINEVIBRATO; break;
-	case 'V':	m.command = CMD_GLOBALVOLUME; break;
-	case 'W':	m.command = CMD_GLOBALVOLSLIDE; break;
-	case 'X':	m.command = CMD_PANNING8; break;
-	case 'Y':	m.command = CMD_PANBRELLO; break;
-	case 'Z':	m.command = CMD_MIDI; break;
-	case '\\':	m.command = static_cast<ModCommand::COMMAND>(fromIT ? CMD_SMOOTHMIDI : CMD_MIDI); break;
-	// Chars under 0x40 don't save properly, so map : to ] and # to [.
-	case ']':	m.command = CMD_DELAYCUT; break;
-	case '[':	m.command = CMD_XPARAM; break;
-	default:	m.command = CMD_NONE;
+	case '@': m.command = (m.param ? CMD_DUMMY : CMD_NONE); break;
+	case 'A': m.command = CMD_SPEED; break;
+	case 'B': m.command = CMD_POSITIONJUMP; break;
+	case 'C': m.command = CMD_PATTERNBREAK; if (!fromIT) m.param = (m.param >> 4) * 10 + (m.param & 0x0F); break;
+	case 'D': m.command = CMD_VOLUMESLIDE; break;
+	case 'E': m.command = CMD_PORTAMENTODOWN; break;
+	case 'F': m.command = CMD_PORTAMENTOUP; break;
+	case 'G': m.command = CMD_TONEPORTAMENTO; break;
+	case 'H': m.command = CMD_VIBRATO; break;
+	case 'I': m.command = CMD_TREMOR; break;
+	case 'J': m.command = CMD_ARPEGGIO; break;
+	case 'K': m.command = CMD_VIBRATOVOL; break;
+	case 'L': m.command = CMD_TONEPORTAVOL; break;
+	case 'M': m.command = CMD_CHANNELVOLUME; break;
+	case 'N': m.command = CMD_CHANNELVOLSLIDE; break;
+	case 'O': m.command = CMD_OFFSET; break;
+	case 'P': m.command = CMD_PANNINGSLIDE; break;
+	case 'Q': m.command = CMD_RETRIG; break;
+	case 'R': m.command = CMD_TREMOLO; break;
+	case 'S': m.command = CMD_S3MCMDEX; break;
+	case 'T': m.command = CMD_TEMPO; break;
+	case 'U': m.command = CMD_FINEVIBRATO; break;
+	case 'V': m.command = CMD_GLOBALVOLUME; break;
+	case 'W': m.command = CMD_GLOBALVOLSLIDE; break;
+	case 'X': m.command = CMD_PANNING8; break;
+	case 'Y': m.command = CMD_PANBRELLO; break;
+	case 'Z': m.command = CMD_MIDI; break;
+	case '\\': m.command = fromIT ? CMD_SMOOTHMIDI : CMD_MIDI; break;
+	// Chars under 0x40 don't save properly, so the following commands don't map to their pattern editor representations
+	case ']': m.command = fromIT ? CMD_DELAYCUT : CMD_NONE; break;
+	case '[': m.command = fromIT ? CMD_XPARAM : CMD_NONE; break;
+	case '^': m.command = fromIT ? CMD_FINETUNE : CMD_NONE; break;
+	case '_': m.command = fromIT ? CMD_FINETUNE_SMOOTH : CMD_NONE; break;
+	// BeRoTracker extensions
+	case '1' + 0x41: m.command = fromIT ? CMD_KEYOFF : CMD_NONE; break;
+	case '2' + 0x41: m.command = fromIT ? CMD_SETENVPOSITION : CMD_NONE; break;
+	default: m.command = CMD_NONE;
 	}
 }
 
@@ -66,31 +75,34 @@ void CSoundFile::S3MConvert(ModCommand &m, bool fromIT)
 
 void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool compatibilityExport) const
 {
+	const bool extendedIT = !compatibilityExport && toIT;
 	switch(command)
 	{
-	case CMD_SPEED:				command = 'A'; break;
-	case CMD_POSITIONJUMP:		command = 'B'; break;
-	case CMD_PATTERNBREAK:		command = 'C'; if(!toIT) param = ((param / 10) << 4) + (param % 10); break;
-	case CMD_VOLUMESLIDE:		command = 'D'; break;
-	case CMD_PORTAMENTODOWN:	command = 'E'; if (param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
-	case CMD_PORTAMENTOUP:		command = 'F'; if (param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
-	case CMD_TONEPORTAMENTO:	command = 'G'; break;
-	case CMD_VIBRATO:			command = 'H'; break;
-	case CMD_TREMOR:			command = 'I'; break;
-	case CMD_ARPEGGIO:			command = 'J'; break;
-	case CMD_VIBRATOVOL:		command = 'K'; break;
-	case CMD_TONEPORTAVOL:		command = 'L'; break;
-	case CMD_CHANNELVOLUME:		command = 'M'; break;
-	case CMD_CHANNELVOLSLIDE:	command = 'N'; break;
-	case CMD_OFFSET:			command = 'O'; break;
-	case CMD_PANNINGSLIDE:		command = 'P'; break;
-	case CMD_RETRIG:			command = 'Q'; break;
-	case CMD_TREMOLO:			command = 'R'; break;
-	case CMD_S3MCMDEX:			command = 'S'; break;
-	case CMD_TEMPO:				command = 'T'; break;
-	case CMD_FINEVIBRATO:		command = 'U'; break;
-	case CMD_GLOBALVOLUME:		command = 'V'; break;
-	case CMD_GLOBALVOLSLIDE:	command = 'W'; break;
+	case CMD_DUMMY:           command = (param ? '@' : 0); break;
+	case CMD_SPEED:           command = 'A'; break;
+	case CMD_POSITIONJUMP:    command = 'B'; break;
+	case CMD_PATTERNBREAK:    command = 'C'; if(!toIT) param = ((param / 10) << 4) + (param % 10); break;
+	case CMD_VOLUMESLIDE:     command = 'D'; break;
+	case CMD_PORTAMENTODOWN:  command = 'E'; if (param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
+	case CMD_PORTAMENTOUP:    command = 'F'; if (param >= 0xE0 && (GetType() & (MOD_TYPE_MOD | MOD_TYPE_XM))) param = 0xDF; break;
+	case CMD_TONEPORTAMENTO:  command = 'G'; break;
+	case CMD_VIBRATO:         command = 'H'; break;
+	case CMD_TREMOR:          command = 'I'; break;
+	case CMD_ARPEGGIO:        command = 'J'; break;
+	case CMD_VIBRATOVOL:      command = 'K'; break;
+	case CMD_TONEPORTAVOL:    command = 'L'; break;
+	case CMD_CHANNELVOLUME:   command = 'M'; break;
+	case CMD_CHANNELVOLSLIDE: command = 'N'; break;
+	case CMD_OFFSETPERCENTAGE:
+	case CMD_OFFSET:          command = 'O'; break;
+	case CMD_PANNINGSLIDE:    command = 'P'; break;
+	case CMD_RETRIG:          command = 'Q'; break;
+	case CMD_TREMOLO:         command = 'R'; break;
+	case CMD_S3MCMDEX:        command = 'S'; break;
+	case CMD_TEMPO:           command = 'T'; break;
+	case CMD_FINEVIBRATO:     command = 'U'; break;
+	case CMD_GLOBALVOLUME:    command = 'V'; break;
+	case CMD_GLOBALVOLSLIDE:  command = 'W'; break;
 	case CMD_PANNING8:
 		command = 'X';
 		if(toIT && !(GetType() & (MOD_TYPE_IT | MOD_TYPE_MPT | MOD_TYPE_XM | MOD_TYPE_MOD)))
@@ -104,21 +116,21 @@ void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool co
 			param >>= 1;
 		}
 		break;
-	case CMD_PANBRELLO:			command = 'Y'; break;
-	case CMD_MIDI:				command = 'Z'; break;
+	case CMD_PANBRELLO: command = 'Y'; break;
+	case CMD_MIDI:      command = 'Z'; break;
 	case CMD_SMOOTHMIDI:
-		if(compatibilityExport || !toIT)
-			command = 'Z';
-		else
+		if(extendedIT)
 			command = '\\';
+		else
+			command = 'Z';
 		break;
 	case CMD_XFINEPORTAUPDOWN:
 		switch(param & 0xF0)
 		{
-		case 0x10:	command = 'F'; param = (param & 0x0F) | 0xE0; break;
-		case 0x20:	command = 'E'; param = (param & 0x0F) | 0xE0; break;
-		case 0x90:	command = 'S'; break;
-		default:	command = 0;
+		case 0x10: command = 'F'; param = (param & 0x0F) | 0xE0; break;
+		case 0x20: command = 'E'; param = (param & 0x0F) | 0xE0; break;
+		case 0x90: command = 'S'; break;
+		default:   command = 0;
 		}
 		break;
 	case CMD_MODCMDEX:
@@ -134,16 +146,16 @@ void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool co
 		return;
 	// Chars under 0x40 don't save properly, so map : to ] and # to [.
 	case CMD_DELAYCUT:
-		if(compatibilityExport || !toIT)
-			command = 0;
-		else
-			command = ']';
+		command = extendedIT ? ']' : 0;
 		break;
 	case CMD_XPARAM:
-		if(compatibilityExport || !toIT)
-			command = 0;
-		else
-			command = '[';
+		command = extendedIT ? '[' : 0;
+		break;
+	case CMD_FINETUNE:
+		command = extendedIT ? '^' : 0;
+		break;
+	case CMD_FINETUNE_SMOOTH:
+		command = extendedIT ? '_' : 0;
 		break;
 	default:
 		command = 0;
@@ -158,21 +170,6 @@ void CSoundFile::S3MSaveConvert(uint8 &command, uint8 &param, bool toIT, bool co
 
 #endif // MODPLUG_NO_FILESAVE
 
-// Pattern decoding flags
-enum S3MPattern
-{
-	s3mEndOfRow			= 0x00,
-	s3mChannelMask		= 0x1F,
-	s3mNotePresent		= 0x20,
-	s3mVolumePresent	= 0x40,
-	s3mEffectPresent	= 0x80,
-	s3mAnyPresent		= 0xE0,
-
-	s3mNoteOff			= 0xFE,
-	s3mNoteNone			= 0xFF,
-};
-
-
 static bool ValidateHeader(const S3MFileHeader &fileHeader)
 {
 	if(std::memcmp(fileHeader.magic, "SCRM", 4)
@@ -242,14 +239,27 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	bool nonCompatTracker = false;
 	bool isST3 = false;
 	bool isSchism = false;
+	const int32 schismDateVersion = SchismTrackerEpoch + ((fileHeader.cwtv == 0x4FFF) ? fileHeader.reserved2 : (fileHeader.cwtv - 0x4050));
 	switch(fileHeader.cwtv & S3MFileHeader::trackerMask)
 	{
+	case S3MFileHeader::trkAkord & S3MFileHeader::trackerMask:
+		if(fileHeader.cwtv == S3MFileHeader::trkAkord)
+			madeWithTracker = U_("Akord");
+		break;
 	case S3MFileHeader::trkScreamTracker:
 		if(fileHeader.cwtv == S3MFileHeader::trkST3_20 && fileHeader.special == 0 && (fileHeader.ordNum & 0x0F) == 0 && fileHeader.ultraClicks == 0 && (fileHeader.flags & ~0x50) == 0)
 		{
-			// MPT 1.16 and older versions of OpenMPT - Simply keep default (filter) MIDI macros
-			m_dwLastSavedWithVersion = MAKE_VERSION_NUMERIC(1, 16, 00, 00);
-			madeWithTracker = U_("ModPlug Tracker / OpenMPT");
+			// MPT and OpenMPT before 1.17.03.02 - Simply keep default (filter) MIDI macros
+			if((fileHeader.masterVolume & 0x80) != 0)
+			{
+				m_dwLastSavedWithVersion = MPT_V("1.16.00.00");
+				madeWithTracker = U_("ModPlug Tracker / OpenMPT 1.17");
+			} else
+			{
+				// MPT 1.0 alpha5 doesn't set the stereo flag, but MPT 1.0 beta1 does.
+				m_dwLastSavedWithVersion = MPT_V("1.00.00.00");
+				madeWithTracker = U_("ModPlug Tracker 1.0 alpha");
+			}
 			keepMidiMacros = true;
 			nonCompatTracker = true;
 			m_playBehaviour.set(kST3LimitPeriod);
@@ -258,6 +268,10 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			madeWithTracker = U_("Velvet Studio");
 		} else
 		{
+			// ST3.20 should only ever write ultra-click values 16, 24 and 32 (corresponding to 8, 12 and 16 in the GUI), ST3.01/3.03 should only write 0,
+			// though several ST3.01/3.03 files with ultra-click values of 16 have been found as well.
+			// However, we won't fingerprint these values here as it's unlikely that there is any other tracker out there disguising as ST3 and using a strange ultra-click value.
+			// Also, re-saving a file with a strange ultra-click value in ST3 doesn't fix this value unless the user manually changes it, or if it's below 16.
 			madeWithTracker = U_("Scream Tracker");
 			formatTrackerStr = true;
 			isST3 = true;
@@ -275,11 +289,11 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			formatTrackerStr = true;
 		} else
 		{
-			madeWithTracker = mpt::format(U_("Impulse Tracker 2.14p%1"))(fileHeader.cwtv - S3MFileHeader::trkIT2_14);
+			madeWithTracker = MPT_UFORMAT("Impulse Tracker 2.14p{}")(fileHeader.cwtv - S3MFileHeader::trkIT2_14);
 		}
 		if(fileHeader.cwtv >= S3MFileHeader::trkIT2_07 && fileHeader.reserved3 != 0)
 		{
-			// Starting from  version 2.07, IT stores the total edit time of a module in the "reserved" field
+			// Starting from version 2.07, IT stores the total edit time of a module in the "reserved" field
 			uint32 editTime = DecodeITEditTimer(fileHeader.cwtv, fileHeader.reserved3);
 
 			FileHistory hist;
@@ -287,6 +301,10 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			m_FileHistory.push_back(hist);
 		}
 		nonCompatTracker = true;
+		m_playBehaviour.set(kPeriodsAreHertz);
+		m_playBehaviour.set(kITRetrigger);
+		m_playBehaviour.set(kITShortSampleRetrig);
+		m_playBehaviour.set(kST3SampleSwap);  // Not exactly like ST3, but close enough
 		m_nMinPeriod = 1;
 		break;
 	case S3MFileHeader::trkSchismTracker:
@@ -299,13 +317,21 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 			madeWithTracker = GetSchismTrackerVersion(fileHeader.cwtv, fileHeader.reserved2);
 			m_nMinPeriod = 1;
 			isSchism = true;
+			if(schismDateVersion >= SchismVersionFromDate<2021, 05, 02>::date)
+				m_playBehaviour.set(kPeriodsAreHertz);
+			if(schismDateVersion >= SchismVersionFromDate<2016, 05, 13>::date)
+				m_playBehaviour.set(kITShortSampleRetrig);
 		}
 		nonCompatTracker = true;
 		break;
 	case S3MFileHeader::trkOpenMPT:
-		madeWithTracker = U_("OpenMPT");
-		formatTrackerStr = true;
-		m_dwLastSavedWithVersion = Version((fileHeader.cwtv & S3MFileHeader::versionMask) << 16);
+		{
+			uint32 mptVersion = (fileHeader.cwtv & S3MFileHeader::versionMask) << 16;
+			if(mptVersion >= 0x01'29'00'00)
+				mptVersion |= fileHeader.reserved2;
+			m_dwLastSavedWithVersion = Version(mptVersion);
+			madeWithTracker = U_("OpenMPT ") + mpt::ufmt::val(m_dwLastSavedWithVersion);
+		}
 		break; 
 	case S3MFileHeader::trkBeRoTracker:
 		madeWithTracker = U_("BeRoTracker");
@@ -321,13 +347,13 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	}
 	if(formatTrackerStr)
 	{
-		madeWithTracker = mpt::format(U_("%1 %2.%3"))(madeWithTracker, (fileHeader.cwtv & 0xF00) >> 8, mpt::ufmt::hex0<2>(fileHeader.cwtv & 0xFF));
+		madeWithTracker = MPT_UFORMAT("{} {}.{}")(madeWithTracker, (fileHeader.cwtv & 0xF00) >> 8, mpt::ufmt::hex0<2>(fileHeader.cwtv & 0xFF));
 	}
 
-	m_modFormat.formatName = U_("ScreamTracker 3");
+	m_modFormat.formatName = U_("Scream Tracker 3");
 	m_modFormat.type = U_("s3m");
 	m_modFormat.madeWithTracker = std::move(madeWithTracker);
-	m_modFormat.charset = m_dwLastSavedWithVersion ? mpt::CharsetWindows1252 : mpt::CharsetCP437;
+	m_modFormat.charset = m_dwLastSavedWithVersion ? mpt::Charset::Windows1252 : mpt::Charset::CP437;
 
 	if(nonCompatTracker)
 	{
@@ -337,6 +363,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		m_playBehaviour.reset(kST3VibratoMemory);
 		m_playBehaviour.reset(KST3PortaAfterArpeggio);
 		m_playBehaviour.reset(kST3OffsetWithoutInstrument);
+		m_playBehaviour.reset(kApplyUpperPeriodLimit);
 	}
 
 	if((fileHeader.cwtv & S3MFileHeader::trackerMask) > S3MFileHeader::trkScreamTracker)
@@ -355,7 +382,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		m_MidiCfg.ClearZxxMacros();
 	}
 
-	mpt::String::Read<mpt::String::nullTerminated>(m_songName, fileHeader.name);
+	m_songName = mpt::String::ReadBuf(mpt::String::nullTerminated, fileHeader.name);
 
 	if(fileHeader.flags & S3MFileHeader::amigaLimits) m_SongFlags.set(SONG_AMIGALIMITS);
 	if(fileHeader.flags & S3MFileHeader::st2Vibrato) m_SongFlags.set(SONG_S3MOLDVIBRATO);
@@ -382,7 +409,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	// Global Volume
-	m_nDefaultGlobalVolume = std::min<uint32>(fileHeader.globalVol, 64) * 4u;
+	m_nDefaultGlobalVolume = std::min(fileHeader.globalVol.get(), uint8(64)) * 4u;
 	// The following check is probably not very reliable, but it fixes a few tunes, e.g.
 	// DARKNESS.S3M by Purple Motion (ST 3.00) and "Image of Variance" by C.C.Catch (ST 3.01):
 	if(m_nDefaultGlobalVolume == 0 && fileHeader.cwtv < S3MFileHeader::trkST3_20)
@@ -396,12 +423,18 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	// However, this version check is missing in ST3, so any mono file with a master volume of 18 will be converted to a stereo file with master volume 32.
 	else if(fileHeader.masterVolume == 2 || fileHeader.masterVolume == (2 | 0x10))
 		m_nSamplePreAmp = 0x20;
+	else if(!(fileHeader.masterVolume & 0x7F))
+		m_nSamplePreAmp = 48;
 	else
 		m_nSamplePreAmp = std::max(fileHeader.masterVolume & 0x7F, 0x10);  // Bit 7 = Stereo (we always use stereo)
 
+	const bool isStereo = (fileHeader.masterVolume & 0x80) != 0 || m_dwLastSavedWithVersion;
+	if(!isStereo)
+		m_nSamplePreAmp = Util::muldivr_unsigned(m_nSamplePreAmp, 8, 11);
+
 	// Approximately as loud as in DOSBox and a real SoundBlaster 16
 	m_nVSTiVolume = 36;
-	if(isSchism && fileHeader.cwtv < SchismVersionFromDate<2018, 11, 12>::Version(S3MFileHeader::trkSchismTracker))
+	if(isSchism && schismDateVersion < SchismVersionFromDate<2018, 11, 12>::date)
 		m_nVSTiVolume = 64;
 
 	// Channel setup
@@ -415,7 +448,8 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		if(fileHeader.channels[i] != 0xFF)
 		{
 			m_nChannels = i + 1;
-			ChnSettings[i].nPan = (ctype & 8) ? 0xCC : 0x33;	// 200 : 56
+			if(isStereo)
+				ChnSettings[i].nPan = (ctype & 8) ? 0xCC : 0x33;	// 200 : 56
 		}
 		if(fileHeader.channels[i] & 0x80)
 		{
@@ -458,7 +492,9 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 	}
 
 	// Reading sample headers
-	m_nSamples = std::min<SAMPLEINDEX>(fileHeader.smpNum, MAX_SAMPLES - 1);
+	m_nSamples = std::min(static_cast<SAMPLEINDEX>(fileHeader.smpNum), static_cast<SAMPLEINDEX>(MAX_SAMPLES - 1));
+	bool anySamples = false;
+	uint16 gusAddresses = 0;
 	for(SAMPLEINDEX smp = 0; smp < m_nSamples; smp++)
 	{
 		S3MSampleHeader sampleHeader;
@@ -469,18 +505,42 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		}
 
 		sampleHeader.ConvertToMPT(Samples[smp + 1], isST3);
-		mpt::String::Read<mpt::String::nullTerminated>(m_szNames[smp + 1], sampleHeader.name);
+		m_szNames[smp + 1] = mpt::String::ReadBuf(mpt::String::nullTerminated, sampleHeader.name);
 
 		if(sampleHeader.sampleType < S3MSampleHeader::typeAdMel)
 		{
-			const uint32 sampleOffset = (sampleHeader.dataPointer[1] << 4) | (sampleHeader.dataPointer[2] << 12) | (sampleHeader.dataPointer[0] << 20);
+			const uint32 sampleOffset = sampleHeader.GetSampleOffset();
 			if((loadFlags & loadSampleData) && sampleHeader.length != 0 && file.Seek(sampleOffset))
 			{
 				sampleHeader.GetSampleFormat((fileHeader.formatVersion == S3MFileHeader::oldVersion)).ReadSample(Samples[smp + 1], file);
+				anySamples = true;
 			}
+			gusAddresses |= sampleHeader.gusAddress;
 		}
 	}
 
+	if(isST3 && anySamples && !gusAddresses && fileHeader.cwtv != S3MFileHeader::trkST3_00)
+	{
+		// All Scream Tracker versions except for some probably early revisions of Scream Tracker 3.00 write GUS addresses. GUS support might not have existed at that point (1992).
+		// Hence if a file claims to be written with ST3 (but not ST3.00), but has no GUS addresses, we deduce that it must be written by some other software (e.g. some PSM -> S3M conversions)
+		isST3 = false;
+		m_modFormat.madeWithTracker = U_("Unknown");
+	} else if(isST3)
+	{
+		// Saving an S3M file in ST3 with the Gravis Ultrasound driver loaded will write a unique GUS memory address for each non-empty sample slot (and 0 for unused slots).
+		// Re-saving that file in ST3 with the SoundBlaster driver loaded will reset the GUS address for all samples to 0 (unused) or 1 (used).
+		// The first used sample will also have an address of 1 with the GUS driver.
+		// So this is a safe way of telling if the file was last saved with the GUS driver loaded or not if there's more than one sample.
+		const bool useGUS = gusAddresses > 1;
+		m_playBehaviour.set(kST3PortaSampleChange, useGUS);
+		m_playBehaviour.set(kST3SampleSwap, !useGUS);
+		m_playBehaviour.set(kITShortSampleRetrig, !useGUS);  // Only half the truth but close enough for now
+		m_modFormat.madeWithTracker += useGUS ? UL_(" (GUS)") : UL_(" (SB)");
+		// ST3's GUS driver doesn't use this value. Ignoring it fixes the balance between FM and PCM samples (e.g. in Rotagilla by Manwe)
+		if(useGUS)
+			m_nSamplePreAmp = 48;
+	}
+
 	// Try to find out if Zxx commands are supposed to be panning commands (PixPlay).
 	// Actually I am only aware of one module that uses this panning style, namely "Crawling Despair" by $volkraq
 	// and I have no idea what PixPlay is, so this code is solely based on the sample text of that module.
@@ -496,7 +556,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 		return true;
 	}
 	// Order list cannot contain pattern indices > 255, so do not even try to load higher patterns
-	const PATTERNINDEX readPatterns = std::min<PATTERNINDEX>(fileHeader.patNum, uint8_max);
+	const PATTERNINDEX readPatterns = std::min(static_cast<PATTERNINDEX>(fileHeader.patNum), static_cast<PATTERNINDEX>(uint8_max));
 	Patterns.ResizeArray(readPatterns);
 	for(PATTERNINDEX pat = 0; pat < readPatterns; pat++)
 	{
@@ -508,7 +568,9 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 
 		// Skip pattern length indication.
 		// Some modules, for example http://aminet.net/mods/8voic/s3m_hunt.lha seem to have a wrong pattern length -
-		// If you strictly adhere the pattern length, you won't read the patterns correctly in that module.
+		// If you strictly adhere the pattern length, you won't read some patterns (e.g. 17) correctly in that module.
+		// It's most likely a broken copy because there are other versions of the track which don't have this issue.
+		// Still, we don't really need this information, so we just ignore it.
 		file.Skip(2);
 
 		// Read pattern data
@@ -535,23 +597,13 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 
 			if(info & s3mNotePresent)
 			{
-				uint8 note = file.ReadUint8(), instr = file.ReadUint8();
-
+				const auto [note, instr] = file.ReadArray<uint8, 2>();
 				if(note < 0xF0)
-				{
-					// Note
-					note = (note & 0x0F) + 12 * (note >> 4) + 12 + NOTE_MIN;
-				} else if(note == s3mNoteOff)
-				{
-					// ^^
-					note = NOTE_NOTECUT;
-				} else if(note == s3mNoteNone)
-				{
-					// ..
-					note = NOTE_NONE;
-				}
-
-				m.note = note;
+					m.note = static_cast<ModCommand::NOTE>(Clamp((note & 0x0F) + 12 * (note >> 4) + 12 + NOTE_MIN, NOTE_MIN, NOTE_MAX));
+				else if(note == s3mNoteOff)
+					m.note = NOTE_NOTECUT;
+				else if(note == s3mNoteNone)
+					m.note = NOTE_NONE;
 				m.instr = instr;
 			}
 
@@ -565,20 +617,16 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 				} else
 				{
 					m.volcmd = VOLCMD_VOLUME;
-					m.vol = MIN(volume, 64);
+					m.vol = std::min(volume, uint8(64));
 				}
 			}
 
 			if(info & s3mEffectPresent)
 			{
-				uint8 command = file.ReadUint8(), param = file.ReadUint8();
-
-				if(command != 0)
-				{
-					m.command = command;
-					m.param = param;
-					S3MConvert(m, false);
-				}
+				const auto [command, param] = file.ReadArray<uint8, 2>();
+				m.command = command;
+				m.param = param;
+				S3MConvert(m, false);
 
 				if(m.command == CMD_S3MCMDEX && (m.param & 0xF0) == 0xA0 && fileHeader.cwtv < S3MFileHeader::trkST3_20)
 				{
@@ -598,6 +646,10 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 						else if(m.param > 0x08)
 							zxxCountRight++;
 					}
+				} else if(m.command == CMD_OFFSET && m.param == 0 && fileHeader.cwtv <= S3MFileHeader::trkST3_01)
+				{
+					// Offset command didn't have effect memory in ST3.01; fixed in ST3.03
+					m.command = CMD_DUMMY;
 				}
 			}
 		}
@@ -624,7 +676,7 @@ bool CSoundFile::ReadS3M(FileReader &file, ModLoadingFlags loadFlags)
 
 bool CSoundFile::SaveS3M(std::ostream &f) const
 {
-	static const uint8 filler[16] =
+	static constexpr uint8 filler[16] =
 	{
 		0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
 		0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,
@@ -645,7 +697,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 	S3MFileHeader fileHeader;
 	MemsetZero(fileHeader);
 
-	mpt::String::Write<mpt::String::nullTerminated>(fileHeader.name, m_songName);
+	mpt::String::WriteBuf(mpt::String::nullTerminated, fileHeader.name) = m_songName;
 	fileHeader.dosEof = S3MFileHeader::idEOF;
 	fileHeader.fileType = S3MFileHeader::idS3MType;
 
@@ -672,7 +724,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 	fileHeader.smpNum = static_cast<uint16>(writeSamples);
 
 	// Patterns
-	PATTERNINDEX writePatterns = MIN(Patterns.GetNumPatterns(), 100u);
+	PATTERNINDEX writePatterns = std::min(Patterns.GetNumPatterns(), PATTERNINDEX(100));
 	fileHeader.patNum = static_cast<uint16>(writePatterns);
 
 	// Flags
@@ -692,16 +744,18 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 	// Version info following: ST3.20 = 0x1320
 	// Most significant nibble = Tracker ID, see S3MFileHeader::S3MTrackerVersions
 	// Following: One nibble = Major version, one byte = Minor version (hex)
-	fileHeader.cwtv = S3MFileHeader::trkOpenMPT | static_cast<uint16>((Version::Current().GetRawVersion() >> 16) & S3MFileHeader::versionMask);
+	const uint32 mptVersion = Version::Current().GetRawVersion();
+	fileHeader.cwtv = S3MFileHeader::trkOpenMPT | static_cast<uint16>((mptVersion >> 16) & S3MFileHeader::versionMask);
+	fileHeader.reserved2 = static_cast<uint16>(mptVersion);
 	fileHeader.formatVersion = S3MFileHeader::newVersion;
 	memcpy(fileHeader.magic, "SCRM", 4);
 
 	// Song Variables
-	fileHeader.globalVol = static_cast<uint8>(std::min<unsigned int>(m_nDefaultGlobalVolume / 4u, 64u));
+	fileHeader.globalVol = static_cast<uint8>(std::min(m_nDefaultGlobalVolume / 4u, uint32(64)));
 	fileHeader.speed = static_cast<uint8>(Clamp(m_nDefaultSpeed, 1u, 254u));
 	fileHeader.tempo = static_cast<uint8>(Clamp(m_nDefaultTempo.GetInt(), 33u, 255u));
 	fileHeader.masterVolume = static_cast<uint8>(Clamp(m_nSamplePreAmp, 16u, 127u) | 0x80);
-	fileHeader.ultraClicks = 8;
+	fileHeader.ultraClicks = 16;
 	fileHeader.usePanningTable = S3MFileHeader::idPanning;
 
 	mpt::IO::Write(f, fileHeader);
@@ -723,7 +777,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 	std::vector<uint16le> sampleOffsets(writeSamples);
 	for(SAMPLEINDEX smp = 0; smp < writeSamples; smp++)
 	{
-		STATIC_ASSERT((sizeof(S3MSampleHeader) % 16) == 0);
+		static_assert((sizeof(S3MSampleHeader) % 16) == 0);
 		sampleOffsets[smp] = static_cast<uint16>((sampleHeaderOffset + smp * sizeof(S3MSampleHeader)) / 16);
 	}
 	mpt::IO::Write(f, sampleOffsets);
@@ -772,7 +826,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 		mpt::IO::Offset patOffset = mpt::IO::TellWrite(f);
 		if(patOffset > 0xFFFF0)
 		{
-			AddToLog(LogError, mpt::format(U_("Too much pattern data! Writing patterns failed starting from pattern %1."))(pat));
+			AddToLog(LogError, MPT_UFORMAT("Too much pattern data! Writing patterns failed starting from pattern {}.")(pat));
 			break;
 		}
 		MPT_ASSERT((patOffset % 16) == 0);
@@ -814,7 +868,6 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 
 						if(note == NOTE_NONE)
 						{
-							// No Note, or note is too low
 							note = s3mNoteNone;
 						} else if(ModCommand::IsSpecialNote(note))
 						{
@@ -844,7 +897,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 					{
 						command = CMD_NONE;
 						volcmd = VOLCMD_VOLUME;
-						vol = MIN(param, 64);
+						vol = std::min(param, uint8(64));
 					}
 
 					if(volcmd == VOLCMD_VOLUME)
@@ -859,7 +912,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 					if(command != CMD_NONE)
 					{
 						S3MSaveConvert(command, param, false, true);
-						if(command)
+						if(command || param)
 						{
 							info |= s3mEffectPresent;
 							if(saveMuteStatus && ChnSettings[chn].dwFlags[CHN_MUTE] && m.IsGlobalCommand())
@@ -911,7 +964,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 	}
 	if(globalCmdOnMutedChn)
 	{
-		//AddToLog(LogWarning, U_("Global commands on muted channels are interpreted by only some S3M players."));
+		//AddToLog(LogWarning, U_("Global commands on muted channels are interpreted only by some S3M players."));
 	}
 
 	mpt::IO::Offset sampleDataOffset = mpt::IO::TellWrite(f);
@@ -941,14 +994,14 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 		}
 
 		const SmpLength smpLength = sampleHeader[smp].ConvertToS3M(Samples[realSmp]);
-		mpt::String::Write<mpt::String::nullTerminated>(sampleHeader[smp].name, m_szNames[realSmp]);
+		mpt::String::WriteBuf(mpt::String::nullTerminated, sampleHeader[smp].name) = m_szNames[realSmp];
 
 		if(smpLength != 0)
 		{
 			// Write sample data
 			if(sampleDataOffset > 0xFFFFFF0)
 			{
-				AddToLog(LogError, mpt::format(U_("Too much sample data! Writing samples failed starting from sample %1."))(realSmp));
+				AddToLog(LogError, MPT_UFORMAT("Too much sample data! Writing samples failed starting from sample {}.")(realSmp));
 				break;
 			}
 
@@ -975,7 +1028,7 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 		{
 			if(channelType[chn][S3MChannelType::kPCM] && channelType[chn][S3MChannelType::kAdlib])
 			{
-				AddToLog(LogWarning, mpt::format(U_("Pattern channel %1 constains both samples and OPL instruments, which is not supported by Scream Tracker 3."))(chn + 1));
+				AddToLog(LogWarning, MPT_UFORMAT("Pattern channel {} constains both samples and OPL instruments, which is not supported by Scream Tracker 3.")(chn + 1));
 			}
 			// ST3 only supports 16 PCM channels, so if channels 17-32 are used,
 			// they must be mapped to the same "internal channels" as channels 1-16.
@@ -1003,11 +1056,11 @@ bool CSoundFile::SaveS3M(std::ostream &f) const
 	}
 	if(sampleCh > 16)
 	{
-		AddToLog(LogWarning, mpt::format(U_("This module has more than 16 (%1) sample channels, which is not supported by Scream Tracker 3."))(sampleCh));
+		AddToLog(LogWarning, MPT_UFORMAT("This module has more than 16 ({}) sample channels, which is not supported by Scream Tracker 3.")(sampleCh));
 	}
 	if(adlibCh > 9)
 	{
-		AddToLog(LogWarning, mpt::format(U_("This module has more than 9 (%1) OPL channels, which is not supported by Scream Tracker 3."))(adlibCh));
+		AddToLog(LogWarning, MPT_UFORMAT("This module has more than 9 ({}) OPL channels, which is not supported by Scream Tracker 3.")(adlibCh));
 	}
 
 	mpt::IO::SeekAbsolute(f, 0);
