@@ -256,7 +256,7 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 
 	InitializeGlobals(MOD_TYPE_STP);
 
-	m_modFormat.formatName = mpt::format(U_("Soundtracker Pro II v%1"))(fileHeader.version);
+	m_modFormat.formatName = MPT_UFORMAT("Soundtracker Pro II v{}")(fileHeader.version);
 	m_modFormat.type = U_("stp");
 	m_modFormat.charset = mpt::Charset::ISO8859_1;
 
@@ -374,13 +374,13 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 
 			patternLength = file.ReadUint16BE();
 			channels = file.ReadUint16BE();
+			if(channels > MAX_BASECHANNELS)
+				return false;
 			m_nChannels = std::max(m_nChannels, channels);
 
 			file.Skip(channels * patternLength * 4u);
 		}
 		file.Seek(patOffset);
-		if(m_nChannels > MAX_BASECHANNELS)
-			return false;
 	}
 
 	struct ChannelMemory
@@ -406,6 +406,9 @@ bool CSoundFile::ReadSTP(FileReader &file, ModLoadingFlags loadFlags)
 			channels = file.ReadUint16BE();
 		}
 
+		if(!file.CanRead(channels * patternLength * 4u))
+			break;
+
 		if(!(loadFlags & loadPatternData) || !Patterns.Insert(actualPat, patternLength))
 		{
 			file.Skip(channels * patternLength * 4u);
