@@ -12,8 +12,9 @@
 #include "stdafx.h"
 #include "Loaders.h"
 #include "WAVTools.h"
-#include "../soundbase/SampleFormatConverters.h"
-#include "../soundbase/SampleFormatCopy.h"
+#include "openmpt/soundbase/SampleConvert.hpp"
+#include "openmpt/soundbase/SampleDecode.hpp"
+#include "SampleCopy.h"
 
 
 OPENMPT_NAMESPACE_BEGIN
@@ -36,7 +37,7 @@ static bool CopyWavChannel(ModSample &sample, const FileReader &file, size_t cha
 		return false;
 	}
 
-	const mpt::byte *inBuf = file.GetRawData<mpt::byte>();
+	const std::byte *inBuf = file.GetRawData<std::byte>().data();
 	CopySample<SampleConversion>(reinterpret_cast<typename SampleConversion::output_t*>(sample.samplev()), sample.nLength, 1, inBuf + offset, file.BytesLeft() - offset, numChannels, conv);
 	return true;
 }
@@ -64,11 +65,13 @@ bool CSoundFile::ReadWAV(FileReader &file, ModLoadingFlags loadFlags)
 	WAVReader wavFile(file);
 
 	if(!wavFile.IsValid()
-		|| wavFile.GetNumChannels() == 0
-		|| wavFile.GetNumChannels() > MAX_BASECHANNELS
-		|| wavFile.GetBitsPerSample() == 0
-		|| wavFile.GetBitsPerSample() > 32
-		|| (wavFile.GetSampleFormat() != WAVFormatChunk::fmtPCM && wavFile.GetSampleFormat() != WAVFormatChunk::fmtFloat))
+	   || wavFile.GetNumChannels() == 0
+	   || wavFile.GetNumChannels() > MAX_BASECHANNELS
+	   || wavFile.GetNumChannels() >= MAX_SAMPLES
+	   || wavFile.GetBitsPerSample() == 0
+	   || wavFile.GetBitsPerSample() > 64
+	   || (wavFile.GetBitsPerSample() < 32 && wavFile.GetSampleFormat() == WAVFormatChunk::fmtFloat)
+	   || (wavFile.GetSampleFormat() != WAVFormatChunk::fmtPCM && wavFile.GetSampleFormat() != WAVFormatChunk::fmtFloat))
 	{
 		return false;
 	} else if(loadFlags == onlyVerifyHeader)
@@ -87,7 +90,7 @@ bool CSoundFile::ReadWAV(FileReader &file, ModLoadingFlags loadFlags)
 
 	m_modFormat.formatName = U_("RIFF WAVE");
 	m_modFormat.type = U_("wav");
-	m_modFormat.charset = mpt::CharsetWindows1252;
+	m_modFormat.charset = mpt::Charset::Windows1252;
 	
 	const SmpLength sampleLength = wavFile.GetSampleLength();
 
@@ -95,7 +98,7 @@ bool CSoundFile::ReadWAV(FileReader &file, ModLoadingFlags loadFlags)
 	// Calculate sample length in ticks at tempo 125
 	const uint32 sampleRate = std::max(uint32(1), wavFile.GetSampleRate());
 	const uint32 sampleTicks = mpt::saturate_cast<uint32>(((sampleLength * 50) / sampleRate) + 1);
-	uint32 ticksPerRow = std::max<uint32>((sampleTicks + 63u) / 63u, 1u);
+	uint32 ticksPerRow = std::max((sampleTicks + 63u) / 63u, uint32(1));
 
 	Order().assign(1, 0);
 	ORDERINDEX numOrders = 1;
@@ -136,7 +139,7 @@ bool CSoundFile::ReadWAV(FileReader &file, ModLoadingFlags loadFlags)
 		sample.uFlags = CHN_PANNING;
 		sample.nLength =  sampleLength;
 		sample.nC5Speed = wavFile.GetSampleRate();
-		strcpy(m_szNames[channel + 1], "");
+		m_szNames[channel + 1] = "";
 		wavFile.ApplySampleSettings(sample, GetCharsetInternal(), m_szNames[channel + 1]);
 
 		if(wavFile.GetNumChannels() > 1)
@@ -173,22 +176,22 @@ bool CSoundFile::ReadWAV(FileReader &file, ModLoadingFlags loadFlags)
 
 		if(wavFile.GetSampleFormat() == WAVFormatChunk::fmtFloat)
 		{
-			CopyWavChannel<SC::ConversionChain<SC::Convert<int16, float32>, SC::DecodeFloat32<littleEndian32> > >(sample, sampleChunk, channel, wavFile.GetNumChannels());
+			if(wavFile.GetBitsPerSample() <= 32)
+				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, float32>, SC::DecodeFloat32<littleEndian32>>>(sample, sampleChunk, channel, wavFile.GetNumChannels());
+			else
+				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, float64>, SC::DecodeFloat64<littleEndian64>>>(sample, sampleChunk, channel, wavFile.GetNumChannels());
 		} else
 		{
 			if(wavFile.GetBitsPerSample() <= 8)
-			{
 				CopyWavChannel<SC::DecodeUint8>(sample, sampleChunk, channel, wavFile.GetNumChannels());
-			} else if(wavFile.GetBitsPerSample() <= 16)
-			{
-				CopyWavChannel<SC::DecodeInt16<0, littleEndian16> >(sample, sampleChunk, channel, wavFile.GetNumChannels());
-			} else if(wavFile.GetBitsPerSample() <= 24)
-			{
-				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, int32>, SC::DecodeInt24<0, littleEndian24> > >(sample, sampleChunk, channel, wavFile.GetNumChannels());
-			} else if(wavFile.GetBitsPerSample() <= 32)
-			{
-				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, int32>, SC::DecodeInt32<0, littleEndian32> > >(sample, sampleChunk, channel, wavFile.GetNumChannels());
-			}
+			else if(wavFile.GetBitsPerSample() <= 16)
+				CopyWavChannel<SC::DecodeInt16<0, littleEndian16>>(sample, sampleChunk, channel, wavFile.GetNumChannels());
+			else if(wavFile.GetBitsPerSample() <= 24)
+				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, int32>, SC::DecodeInt24<0, littleEndian24>>>(sample, sampleChunk, channel, wavFile.GetNumChannels());
+			else if(wavFile.GetBitsPerSample() <= 32)
+				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, int32>, SC::DecodeInt32<0, littleEndian32>>>(sample, sampleChunk, channel, wavFile.GetNumChannels());
+			else if(wavFile.GetBitsPerSample() <= 64)
+				CopyWavChannel<SC::ConversionChain<SC::Convert<int16, int64>, SC::DecodeInt64<0, littleEndian64>>>(sample, sampleChunk, channel, wavFile.GetNumChannels());
 		}
 		sample.PrecomputeLoops(*this, false);
 
