@@ -10,7 +10,7 @@
 
 #pragma once
 
-#include "BuildSettings.h"
+#include "openmpt/all/BuildSettings.hpp"
 
 #include "Mixer.h"
 
@@ -23,11 +23,6 @@ void FloatToStereoMix(const float *pIn1, const float *pIn2, int32 *pOut, uint32
 void MonoMixToFloat(const int32 *pSrc, float *pOut, uint32 uint32, const float _i2fc);
 void FloatToMonoMix(const float *pIn, int32 *pOut, uint32 uint32, const float _f2ic);
 
-#ifndef MODPLUG_TRACKER
-void ApplyGain(int32 *soundBuffer, std::size_t channels, std::size_t countChunk, int32 gainFactor16_16);
-void ApplyGain(float *outputBuffer, float * const *outputBuffers, std::size_t offset, std::size_t channels, std::size_t countChunk, float gainFactor);
-#endif // !MODPLUG_TRACKER
-
 void InitMixBuffer(mixsample_t *pBuffer, uint32 nSamples);
 void InterleaveFrontRear(mixsample_t *pFrontBuf, mixsample_t *pRearBuf, uint32 nFrames);
 void MonoFromStereo(mixsample_t *pMixBuf, uint32 nSamples);
