@@ -24,13 +24,13 @@ OPENMPT_NAMESPACE_BEGIN
 /////////////////////////////////////////////////////////////////////////////
 // Note Name Tables
 
-const MPT_UCHAR_TYPE NoteNamesSharp[12][4] =
+const mpt::uchar NoteNamesSharp[12][4] =
 {
 	UL_("C-"), UL_("C#"), UL_("D-"), UL_("D#"), UL_("E-"), UL_("F-"),
 	UL_("F#"), UL_("G-"), UL_("G#"), UL_("A-"), UL_("A#"), UL_("B-")
 };
 
-const MPT_UCHAR_TYPE NoteNamesFlat[12][4] =
+const mpt::uchar NoteNamesFlat[12][4] =
 {
 	UL_("C-"), UL_("Db"), UL_("D-"), UL_("Eb"), UL_("E-"), UL_("F-"),
 	UL_("Gb"), UL_("G-"), UL_("Ab"), UL_("A-"), UL_("Bb"), UL_("B-")
@@ -42,78 +42,84 @@ const MPT_UCHAR_TYPE NoteNamesFlat[12][4] =
 
 struct ModFormatInfo
 {
-	MODTYPE format;              // MOD_TYPE_XXXX
-	const MPT_UCHAR_TYPE *name;  // "ProTracker"
-	const char *extension;       // "mod"
+	MODTYPE format;          // MOD_TYPE_XXXX
+	const mpt::uchar *name;  // "ProTracker"
+	const char *extension;   // "mod"
 };
 
 // Note: Formats with identical extensions must be grouped together.
 static constexpr ModFormatInfo modFormatInfo[] =
 {
-	{ MOD_TYPE_MPT,  UL_("OpenMPT"),                    "mptm" },
-	{ MOD_TYPE_MOD,  UL_("ProTracker"),                 "mod" },
-	{ MOD_TYPE_S3M,  UL_("ScreamTracker 3"),            "s3m" },
-	{ MOD_TYPE_XM,   UL_("FastTracker 2"),              "xm" },
-	{ MOD_TYPE_IT,   UL_("Impulse Tracker"),            "it" },
+	{ MOD_TYPE_MPT,  UL_("OpenMPT"),                            "mptm" },
+	{ MOD_TYPE_MOD,  UL_("ProTracker"),                         "mod" },
+	{ MOD_TYPE_S3M,  UL_("Scream Tracker 3"),                   "s3m" },
+	{ MOD_TYPE_XM,   UL_("FastTracker 2"),                      "xm" },
+	{ MOD_TYPE_IT,   UL_("Impulse Tracker"),                    "it" },
 
-	{ MOD_TYPE_669,  UL_("Composer 669 / UNIS 669"),    "669" },
-	{ MOD_TYPE_AMF0, UL_("ASYLUM Music Format"),        "amf" },
-	{ MOD_TYPE_AMF,  UL_("DSMI Advanced Music Format"), "amf" },
-	{ MOD_TYPE_AMS,  UL_("Extreme's Tracker"),          "ams" },
-	{ MOD_TYPE_AMS,  UL_("Velvet Studio"),              "ams" },
-	{ MOD_TYPE_S3M,  UL_("CDFM / Composer 670"),        "c67" },
-	{ MOD_TYPE_DBM,  UL_("DigiBooster Pro"),            "dbm" },
-	{ MOD_TYPE_DIGI, UL_("DigiBooster"),                "digi" },
-	{ MOD_TYPE_DMF,  UL_("X-Tracker"),                  "dmf" },
-	{ MOD_TYPE_DSM,  UL_("DSIK Format"),                "dsm" },
-	{ MOD_TYPE_DTM,  UL_("Digital Tracker"),            "dtm" },
-	{ MOD_TYPE_FAR,  UL_("Farandole Composer"),         "far" },
-	{ MOD_TYPE_IMF,  UL_("Imago Orpheus"),              "imf" },
-	{ MOD_TYPE_MOD,  UL_("Ice Tracker"),                "ice" },
+	{ MOD_TYPE_669,  UL_("Composer 669 / UNIS 669"),            "669" },
+	{ MOD_TYPE_AMF0, UL_("ASYLUM Music Format"),                "amf" },
+	{ MOD_TYPE_AMF,  UL_("DSMI Advanced Music Format"),         "amf" },
+	{ MOD_TYPE_AMS,  UL_("Extreme's Tracker"),                  "ams" },
+	{ MOD_TYPE_AMS,  UL_("Velvet Studio"),                      "ams" },
+	{ MOD_TYPE_S3M,  UL_("CDFM / Composer 670"),                "c67" },
+	{ MOD_TYPE_DBM,  UL_("DigiBooster Pro"),                    "dbm" },
+	{ MOD_TYPE_DIGI, UL_("DigiBooster"),                        "digi" },
+	{ MOD_TYPE_DMF,  UL_("X-Tracker"),                          "dmf" },
+	{ MOD_TYPE_DSM,  UL_("DSIK Format"),                        "dsm" },
+	{ MOD_TYPE_MOD,  UL_("Digital Symphony"),                   "dsym" },
+	{ MOD_TYPE_DTM,  UL_("Digital Tracker"),                    "dtm" },
+	{ MOD_TYPE_FAR,  UL_("Farandole Composer"),                 "far" },
+	{ MOD_TYPE_S3M,  UL_("FM Tracker"),                         "fmt" },
+	{ MOD_TYPE_IMF,  UL_("Imago Orpheus"),                      "imf" },
+	{ MOD_TYPE_MOD,  UL_("Ice Tracker"),                        "ice" },
 #ifdef MPT_EXTERNAL_SAMPLES
-	{ MOD_TYPE_IT,   UL_("Impulse Tracker Project"),    "itp" },
+	{ MOD_TYPE_IT,   UL_("Impulse Tracker Project"),            "itp" },
 #endif
-	{ MOD_TYPE_J2B,  UL_("Galaxy Sound System"),        "j2b" },
-	{ MOD_TYPE_MOD,  UL_("Soundtracker"),               "m15" },
-	{ MOD_TYPE_MDL,  UL_("Digitrakker"),                "mdl" },
-	{ MOD_TYPE_MED,  UL_("OctaMED"),                    "med" },
-	{ MOD_TYPE_SFX,  UL_("MultiMedia Sound"),           "mms" },
-	{ MOD_TYPE_MT2,  UL_("MadTracker 2"),               "mt2" },
-	{ MOD_TYPE_MTM,  UL_("MultiTracker"),               "mtm" },
-	{ MOD_TYPE_MOD,  UL_("NoiseTracker"),               "nst" },
-	{ MOD_TYPE_OKT,  UL_("Oktalyzer"),                  "okt" },
-	{ MOD_TYPE_PLM,  UL_("Disorder Tracker 2"),         "plm" },
-	{ MOD_TYPE_PSM,  UL_("Epic Megagames MASI"),        "psm" },
-	{ MOD_TYPE_MOD,  UL_("ProTracker"),                 "pt36" },
-	{ MOD_TYPE_PTM,  UL_("PolyTracker"),                "ptm" },
-	{ MOD_TYPE_SFX,  UL_("SoundFX"),                    "sfx" },
-	{ MOD_TYPE_SFX,  UL_("SoundFX"),                    "sfx2" },
-	{ MOD_TYPE_MOD,  UL_("SoundTracker 2.6"),           "st26" },
-	{ MOD_TYPE_MOD,  UL_("Soundtracker"),               "stk" },
-	{ MOD_TYPE_STM,  UL_("ScreamTracker 2"),            "stm" },
-	{ MOD_TYPE_STP,  UL_("Soundtracker Pro II"),        "stp" },
-	{ MOD_TYPE_ULT,  UL_("UltraTracker"),               "ult" },
-	{ MOD_TYPE_MOD,  UL_("Grave Composer"),             "wow" },
+	{ MOD_TYPE_J2B,  UL_("Galaxy Sound System"),                "j2b" },
+	{ MOD_TYPE_MOD,  UL_("Soundtracker"),                       "m15" },
+	{ MOD_TYPE_MDL,  UL_("Digitrakker"),                        "mdl" },
+	{ MOD_TYPE_MED,  UL_("OctaMED"),                            "med" },
+	{ MOD_TYPE_SFX,  UL_("MultiMedia Sound"),                   "mms" },
+	{ MOD_TYPE_MT2,  UL_("MadTracker 2"),                       "mt2" },
+	{ MOD_TYPE_MTM,  UL_("MultiTracker"),                       "mtm" },
+	{ MOD_TYPE_MOD,  UL_("Karl Morton Music Format"),           "mus" },
+	{ MOD_TYPE_MOD,  UL_("NoiseTracker"),                       "nst" },
+	{ MOD_TYPE_OKT,  UL_("Oktalyzer"),                          "okt" },
+	{ MOD_TYPE_PLM,  UL_("Disorder Tracker 2"),                 "plm" },
+	{ MOD_TYPE_PSM,  UL_("Epic Megagames MASI"),                "psm" },
+	{ MOD_TYPE_MOD,  UL_("ProTracker"),                         "pt36" },
+	{ MOD_TYPE_PTM,  UL_("PolyTracker"),                        "ptm" },
+	{ MOD_TYPE_SFX,  UL_("SoundFX"),                            "sfx" },
+	{ MOD_TYPE_SFX,  UL_("SoundFX"),                            "sfx2" },
+	{ MOD_TYPE_MOD,  UL_("SoundTracker 2.6"),                   "st26" },
+	{ MOD_TYPE_MOD,  UL_("Soundtracker"),                       "stk" },
+	{ MOD_TYPE_STM,  UL_("Scream Tracker 2"),                   "stm" },
+	{ MOD_TYPE_STM,  UL_("Scream Tracker Music Interface Kit"), "stx" },
+	{ MOD_TYPE_STP,  UL_("Soundtracker Pro II"),                "stp" },
+	{ MOD_TYPE_MPT,  UL_("Symphonie"),                          "symmod"},
+	{ MOD_TYPE_ULT,  UL_("UltraTracker"),                       "ult" },
+	{ MOD_TYPE_MOD,  UL_("Mod's Grave"),                        "wow" },
 	// converted formats (no MODTYPE)
-	{ MOD_TYPE_NONE, UL_("General Digital Music"),      "gdm" },
-	{ MOD_TYPE_NONE, UL_("Un4seen MO3"),                "mo3" },
+	{ MOD_TYPE_NONE, UL_("General Digital Music"),              "gdm" },
+	{ MOD_TYPE_NONE, UL_("Un4seen MO3"),                        "mo3" },
+	{ MOD_TYPE_NONE, UL_("OggMod FastTracker 2"),               "oxm" },
 #ifndef NO_ARCHIVE_SUPPORT
 	// Compressed modules
-	{ MOD_TYPE_MOD,  UL_("Compressed ProTracker"),      "mdz" },
-	{ MOD_TYPE_MOD,  UL_("Compressed Module"),          "mdr" },
-	{ MOD_TYPE_S3M,  UL_("Compressed ScreamTracker 3"), "s3z" },
-	{ MOD_TYPE_XM,   UL_("Compressed FastTracker 2"),   "xmz" },
-	{ MOD_TYPE_IT,   UL_("Compressed Impulse Tracker"), "itz" },
-	{ MOD_TYPE_MPT,  UL_("Compressed OpenMPT"),         "mptmz" },
+	{ MOD_TYPE_MOD,  UL_("Compressed ProTracker"),              "mdz" },
+	{ MOD_TYPE_MOD,  UL_("Compressed Module"),                  "mdr" },
+	{ MOD_TYPE_S3M,  UL_("Compressed Scream Tracker 3"),        "s3z" },
+	{ MOD_TYPE_XM,   UL_("Compressed FastTracker 2"),           "xmz" },
+	{ MOD_TYPE_IT,   UL_("Compressed Impulse Tracker"),         "itz" },
+	{ MOD_TYPE_MPT,  UL_("Compressed OpenMPT"),                 "mptmz" },
 #endif
 };
 
 
 struct ModContainerInfo
 {
-	MODCONTAINERTYPE format;     // MOD_CONTAINERTYPE_XXXX
-	const MPT_UCHAR_TYPE *name;  // "Unreal Music"
-	const char *extension;       // "umx"
+	MODCONTAINERTYPE format;  // MOD_CONTAINERTYPE_XXXX
+	const mpt::uchar *name;   // "Unreal Music"
+	const char *extension;    // "umx"
 };
 
 static constexpr ModContainerInfo modContainerInfo[] =
@@ -122,11 +128,10 @@ static constexpr ModContainerInfo modContainerInfo[] =
 	{ MOD_CONTAINERTYPE_UMX,   UL_("Unreal Music"),             "umx"   },
 	{ MOD_CONTAINERTYPE_XPK,   UL_("XPK packed"),               "xpk"   },
 	{ MOD_CONTAINERTYPE_PP20,  UL_("PowerPack PP20"),           "ppm"   },
-	{ MOD_CONTAINERTYPE_MMCMP, UL_("Music Module Compressor"),  "mmcmp" }
+	{ MOD_CONTAINERTYPE_MMCMP, UL_("Music Module Compressor"),  "mmcmp" },
 #ifdef MODPLUG_TRACKER
-	,
 	{ MOD_CONTAINERTYPE_WAV,   UL_("Wave"),                     "wav"   },
-	{ MOD_CONTAINERTYPE_UAX,   UL_("Unreal Sounds"),            "uax"   }
+	{ MOD_CONTAINERTYPE_UAX,   UL_("Unreal Sounds"),            "uax"   },
 #endif
 };
 
@@ -175,21 +180,19 @@ std::vector<const char *> CSoundFile::GetSupportedExtensions(bool otherFormats)
 }
 
 
-static bool IsEqualExtension(const char *a, const char *b)
+static bool IsEqualExtension(std::string_view a, std::string_view b)
 {
-	std::size_t lena = std::strlen(a);
-	std::size_t lenb = std::strlen(b);
-	if(lena != lenb)
+	if(a.length() != b.length())
 	{
 		return false;
 	}
-	return mpt::CompareNoCaseAscii(a, b, lena) == 0;
+	return mpt::CompareNoCaseAscii(a, b) == 0;
 }
 
 
-bool CSoundFile::IsExtensionSupported(const char *ext)
+bool CSoundFile::IsExtensionSupported(std::string_view ext)
 {
-	if(ext == nullptr || ext[0] == 0)
+	if(ext.length() == 0)
 	{
 		return false;
 	}
@@ -217,7 +220,7 @@ mpt::ustring CSoundFile::ModContainerTypeToString(MODCONTAINERTYPE containertype
 	{
 		if(containerInfo.format == containertype)
 		{
-			return mpt::ToUnicode(mpt::CharsetUTF8, containerInfo.extension);
+			return mpt::ToUnicode(mpt::Charset::UTF8, containerInfo.extension);
 		}
 	}
 	return mpt::ustring();
@@ -682,46 +685,46 @@ const int16 CResampler::FastSincTable[256*4] =
 
 
 // Compute Bessel function Izero(y) using a series approximation
-static double izero(double y)
+double Izero(double y)
 {
-	double s=1, ds=1, d=0;
+	double s = 1, ds = 1, d = 0;
 	do
 	{
-		d = d + 2; ds = ds * (y*y)/(d*d);
+		d = d + 2;
+		ds = ds * (y * y) / (d * d);
 		s = s + ds;
-	} while (ds > 1E-7 * s);
+	} while(ds > 1E-7 * s);
 	return s;
 }
 
-static void getsinc(SINC_TYPE *psinc, double beta, double lowpass_factor)
+
+static void getsinc(SINC_TYPE *psinc, double beta, double cutoff)
 {
-	if(lowpass_factor >= 0.999)
+	if(cutoff >= 0.999)
 	{
 		// Avoid mixer overflows.
 		// 1.0 itself does not make much sense.
-		lowpass_factor = 0.999;
+		cutoff = 0.999;
 	}
-	const double izero_beta = izero(beta);
-	const double kPi = 4.0*atan(1.0)*lowpass_factor;
-	for (int isrc=0; isrc<8*SINC_PHASES; isrc++)
+	const double izeroBeta = Izero(beta);
+	const double kPi = 4.0 * std::atan(1.0) * cutoff;
+	for(int isrc = 0; isrc < 8 * SINC_PHASES; isrc++)
 	{
 		double fsinc;
 		int ix = 7 - (isrc & 7);
-		ix = (ix*SINC_PHASES)+(isrc>>3);
-		if (ix == (4*SINC_PHASES))
+		ix = (ix * SINC_PHASES) + (isrc >> 3);
+		if(ix == (4 * SINC_PHASES))
 		{
 			fsinc = 1.0;
 		} else
 		{
-			double x = (double)(ix - (4*SINC_PHASES)) * (double)(1.0/SINC_PHASES);
-			fsinc = sin(x*kPi) * izero(beta*sqrt(1-x*x*(1.0/16.0))) / (izero_beta*x*kPi); // Kaiser window
+			const double x = (double)(ix - (4 * SINC_PHASES)) * (double)(1.0 / SINC_PHASES);
+			const double xPi = x * kPi;
+			fsinc = std::sin(xPi) * Izero(beta * std::sqrt(1 - x * x * (1.0 / 16.0))) / (izeroBeta * xPi); // Kaiser window
 		}
-		double coeff = fsinc * lowpass_factor;
+		double coeff = fsinc * cutoff;
 #ifdef MPT_INTMIXER
-		int n = (int)std::floor(coeff * (1<<SINC_QUANTSHIFT) + 0.5);
-		MPT_ASSERT(n <= int16_max);
-		MPT_ASSERT(n > int16_min);
-		*psinc++ = static_cast<SINC_TYPE>(n);
+		*psinc++ = mpt::saturate_round<SINC_TYPE>(coeff * (1 << SINC_QUANTSHIFT));
 #else
 		*psinc++ = static_cast<SINC_TYPE>(coeff);
 #endif
@@ -731,11 +734,12 @@ static void getsinc(SINC_TYPE *psinc, double beta, double lowpass_factor)
 
 #ifdef MODPLUG_TRACKER
 bool CResampler::StaticTablesInitialized = false;
-SINC_TYPE CResampler::gKaiserSinc[SINC_PHASES*8];     // Upsampling
-SINC_TYPE CResampler::gDownsample13x[SINC_PHASES*8];	// Downsample 1.333x
-SINC_TYPE CResampler::gDownsample2x[SINC_PHASES*8];		// Downsample 2x
+SINC_TYPE CResampler::gKaiserSinc[SINC_PHASES * 8];     // Upsampling
+SINC_TYPE CResampler::gDownsample13x[SINC_PHASES * 8];  // Downsample 1.333x
+SINC_TYPE CResampler::gDownsample2x[SINC_PHASES * 8];   // Downsample 2x
+Paula::BlepTables CResampler::blepTables;               // Amiga BLEP resampler
 #ifndef MPT_INTMIXER
-mixsample_t CResampler::FastSincTablef[256 * 4];		// Cubic spline LUT
+mixsample_t CResampler::FastSincTablef[256 * 4];        // Cubic spline LUT
 #endif // !defined(MPT_INTMIXER)
 #endif // MODPLUG_TRACKER
 
@@ -750,7 +754,7 @@ void CResampler::InitFloatmixerTables()
 #endif // MPT_BUILD_FUZZER
 #ifndef MPT_INTMIXER
 	// Prepare fast sinc coefficients for floating point mixer
-	for(size_t i = 0; i < CountOf(FastSincTable); i++)
+	for(std::size_t i = 0; i < std::size(FastSincTable); i++)
 	{
 		FastSincTablef[i] = static_cast<mixsample_t>(FastSincTable[i] * mixsample_t(1.0f / 16384.0f));
 	}
@@ -773,13 +777,15 @@ void CResampler::InitializeTablesFromScratch(bool force)
 	{
 		InitFloatmixerTables();
 
+		blepTables.InitTables();
+
 		getsinc(gKaiserSinc, 9.6377, 0.97);
 		getsinc(gDownsample13x, 8.5, 0.5);
 		getsinc(gDownsample2x, 2.7625, 0.425);
 
-		#ifdef MODPLUG_TRACKER
-			StaticTablesInitialized = true;
-		#endif // MODPLUG_TRACKER
+#ifdef MODPLUG_TRACKER
+		StaticTablesInitialized = true;
+#endif  // MODPLUG_TRACKER
 	}
 
 	if((m_OldSettings == m_Settings) && !force)
@@ -790,7 +796,6 @@ void CResampler::InitializeTablesFromScratch(bool force)
 	m_WindowedFIR.InitTable(m_Settings.gdWFIRCutoff, m_Settings.gbWFIRType);
 
 	m_OldSettings = m_Settings;
-
 }
 
 
@@ -811,6 +816,7 @@ void CResampler::InitializeTablesFromCache()
 	std::copy(s_CachedResampler.gDownsample13x, s_CachedResampler.gDownsample13x + SINC_PHASES*8, gDownsample13x);
 	std::copy(s_CachedResampler.gDownsample2x, s_CachedResampler.gDownsample2x + SINC_PHASES*8, gDownsample2x);
 	std::copy(s_CachedResampler.m_WindowedFIR.lut, s_CachedResampler.m_WindowedFIR.lut + WFIR_LUTLEN*WFIR_WIDTH, m_WindowedFIR.lut);
+	blepTables = s_CachedResampler.blepTables;
 }
 
 #endif // MPT_RESAMPLER_TABLES_CACHED
@@ -825,7 +831,14 @@ struct ResampleCacheInitializer
 		GetCachedResampler();
 	}
 };
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wglobal-constructors"
+#endif // MPT_COMPILER_CLANG
 static ResampleCacheInitializer g_ResamplerCachePrimer;
+#if MPT_COMPILER_CLANG
+#pragma clang diagnostic pop
+#endif // MPT_COMPILER_CLANG
 
 #endif // MPT_RESAMPLER_TABLES_CACHED_ONSTARTUP
 
