@@ -25,6 +25,10 @@
 #include "../../common/mptFileIO.h"
 #include "../mod_specifications.h"
 #endif // MODPLUG_TRACKER
+#include "mpt/base/aligned_array.hpp"
+#include "mpt/io/base.hpp"
+#include "mpt/io/io.hpp"
+#include "mpt/io/io_span.hpp"
 
 #include <cmath>
 
@@ -45,7 +49,7 @@ IMixPlugin::IMixPlugin(VSTPluginLib &factory, CSoundFile &sndFile, SNDMIXPLUGIN
 	, m_pMixStruct(mixStruct)
 {
 	m_SndFile.m_loadedPlugins++;
-	m_MixState.pMixBuffer = (mixsample_t *)((((intptr_t)m_MixBuffer) + 7) & ~7);
+	m_MixState.pMixBuffer = mpt::align_bytes<8, MIXBUFFERSIZE * 2>(m_MixBuffer);
 	while(m_pMixStruct != &(m_SndFile.m_MixPlugins[m_nSlot]) && m_nSlot < MAX_MIXPLUGINS - 1)
 	{
 		m_nSlot++;
@@ -98,16 +102,31 @@ void IMixPlugin::SetSlot(PLUGINDEX slot)
 }
 
 
+PlugParamValue IMixPlugin::GetScaledUIParam(PlugParamIndex param)
+{
+	const auto [paramMin, paramMax] = GetParamUIRange(param);
+	return (std::clamp(GetParameter(param), paramMin, paramMax) - paramMin) / (paramMax - paramMin);
+}
+
+
+void IMixPlugin::SetScaledUIParam(PlugParamIndex param, PlugParamValue value)
+{
+	const auto [paramMin, paramMax] = GetParamUIRange(param);
+	const auto scaledVal = paramMin + std::clamp(value, 0.0f, 1.0f) * (paramMax - paramMin);
+	SetParameter(param, scaledVal);
+}
+
+
 CString IMixPlugin::GetFormattedParamName(PlugParamIndex param)
 {
 	CString paramName = GetParamName(param);
 	CString name;
 	if(paramName.IsEmpty())
 	{
-		name.Format(_T("%02u: Parameter %02u"), param, param);
+		name = MPT_CFORMAT("{}: Parameter {}")(mpt::cfmt::dec0<2>(param), mpt::cfmt::dec0<2>(param));
 	} else
 	{
-		name.Format(_T("%02u: %s"), param, paramName.GetString());
+		name = MPT_CFORMAT("{}: {}")(mpt::cfmt::dec0<2>(param), paramName);
 	}
 	return name;
 }
@@ -136,9 +155,9 @@ CString IMixPlugin::GetFormattedProgramName(int32 index)
 
 	CString formattedName;
 	if(rawname[0] >= 0 && rawname[0] < _T(' '))
-		formattedName = mpt::cformat(_T("%1 - Program %2"))(mpt::cfmt::dec0<2>(index), index);
+		formattedName = MPT_CFORMAT("{} - Program {}")(mpt::cfmt::dec0<2>(index), index);
 	else
-		formattedName = mpt::cformat(_T("%1 - %2"))(mpt::cfmt::dec0<2>(index), rawname);
+		formattedName = MPT_CFORMAT("{} - {}")(mpt::cfmt::dec0<2>(index), rawname);
 
 	return formattedName;
 }
@@ -360,7 +379,7 @@ float IMixPlugin::RenderSilence(uint32 numFrames)
 	while(numFrames > 0)
 	{
 		uint32 renderSamples = numFrames;
-		LimitMax(renderSamples, mpt::saturate_cast<uint32>(MPT_ARRAY_COUNT(out[0])));
+		LimitMax(renderSamples, mpt::saturate_cast<uint32>(std::size(out[0])));
 		MemsetZero(out);
 
 		Process(out[0], out[1], renderSamples);
@@ -478,7 +497,7 @@ void IMixPlugin::SaveAllParameters()
 	m_pMixStruct->defaultProgram = -1;
 	
 	// Default implementation: Save all parameter values
-	PlugParamIndex numParams = std::min<uint32>(GetNumParameters(), (std::numeric_limits<uint32>::max() - sizeof(uint32)) / sizeof(IEEE754binary32LE));
+	PlugParamIndex numParams = std::min(GetNumParameters(), static_cast<int32>((std::numeric_limits<uint32>::max() - sizeof(uint32)) / sizeof(IEEE754binary32LE)));
 	uint32 nLen = numParams * sizeof(IEEE754binary32LE);
 	if (!nLen) return;
 	nLen += sizeof(uint32);
@@ -488,14 +507,16 @@ void IMixPlugin::SaveAllParameters()
 		m_pMixStruct->pluginData.resize(nLen);
 		auto memFile = std::make_pair(mpt::as_span(m_pMixStruct->pluginData), mpt::IO::Offset(0));
 		mpt::IO::WriteIntLE<uint32>(memFile, 0);	// Plugin data type
+		BeginGetProgram();
 		for(PlugParamIndex i = 0; i < numParams; i++)
 		{
 			mpt::IO::Write(memFile, IEEE754binary32LE(GetParameter(i)));
 		}
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+		EndGetProgram();
+	} catch(mpt::out_of_memory e)
 	{
 		m_pMixStruct->pluginData.clear();
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 	}
 }
 
@@ -511,10 +532,11 @@ void IMixPlugin::RestoreAllParameters(int32 /*program*/)
 			const uint32 numParams = GetNumParameters();
 			if((m_pMixStruct->pluginData.size() - sizeof(uint32)) >= (numParams * sizeof(IEEE754binary32LE)))
 			{
-				BeginSetProgram(-1);
+				BeginSetProgram();
 				for(uint32 i = 0; i < numParams; i++)
 				{
-					SetParameter(i, memFile.ReadFloatLE());
+					const auto value = memFile.ReadFloatLE();
+					SetParameter(i, std::isfinite(value) ? value : 0.0f);
 				}
 				EndSetProgram();
 			}
@@ -554,9 +576,9 @@ CAbstractVstEditor *IMixPlugin::OpenEditor()
 	try
 	{
 		return new CDefaultVstEditor(*this);
-	} MPT_EXCEPTION_CATCH_OUT_OF_MEMORY(e)
+	} catch(mpt::out_of_memory e)
 	{
-		MPT_EXCEPTION_DELETE_OUT_OF_MEMORY(e);
+		mpt::delete_out_of_memory(e);
 		return nullptr;
 	}
 }
@@ -590,28 +612,26 @@ void IMixPlugin::AutomateParameter(PlugParamIndex param)
 		modDoc->RecordParamChange(GetSlot(), param);
 	}
 
-	modDoc->PostMessageToAllViews(WM_MOD_PLUGPARAMAUTOMATE, m_nSlot, param);
-	// TODO: This should rather be posted to the GUI thread!
-	CAbstractVstEditor *pVstEditor = GetEditor();
+	modDoc->SendNotifyMessageToAllViews(WM_MOD_PLUGPARAMAUTOMATE, m_nSlot, param);
 
-	if(pVstEditor && pVstEditor->m_hWnd)
+	if(auto *vstEditor = GetEditor(); vstEditor && vstEditor->m_hWnd)
 	{
 		// Mark track modified if GUI is open and format supports plugins
 		SetModified();
 
-		if (CMainFrame::GetInputHandler()->ShiftPressed() && TrackerSettings::Instance().midiMappingInPluginEditor)
+		// Do not use InputHandler in case we are coming from a bridged plugin editor
+		if((GetAsyncKeyState(VK_SHIFT) & 0x8000) && TrackerSettings::Instance().midiMappingInPluginEditor)
 		{
 			// Shift pressed -> Open MIDI mapping dialog
-			CMainFrame::GetInputHandler()->SetModifierMask(ModNone); // Make sure that the dialog will open only once.
 			CMainFrame::GetMainFrame()->PostMessage(WM_MOD_MIDIMAPPING, m_nSlot, param);
 		}
 
 		// Learn macro
-		int macroToLearn = pVstEditor->GetLearnMacro();
+		int macroToLearn = vstEditor->GetLearnMacro();
 		if (macroToLearn > -1)
 		{
 			modDoc->LearnMacro(macroToLearn, param);
-			pVstEditor->SetLearnMacro(-1);
+			vstEditor->SetLearnMacro(-1);
 		}
 	}
 }
@@ -630,7 +650,7 @@ void IMixPlugin::SetModified()
 bool IMixPlugin::SaveProgram()
 {
 	mpt::PathString defaultDir = TrackerSettings::Instance().PathPluginPresets.GetWorkingDir();
-	bool useDefaultDir = !defaultDir.empty();
+	const bool useDefaultDir = !defaultDir.empty();
 	if(!useDefaultDir && m_Factory.dllPath.IsFile())
 	{
 		defaultDir = m_Factory.dllPath.GetPath();
@@ -652,19 +672,21 @@ bool IMixPlugin::SaveProgram()
 		TrackerSettings::Instance().PathPluginPresets.SetWorkingDir(dlg.GetWorkingDirectory());
 	}
 
-	bool bank = (dlg.GetExtension() == P_("fxb"));
+	const bool isBank = (dlg.GetExtension() == P_("fxb"));
 
-	mpt::SafeOutputFile sf(dlg.GetFirstFile(), std::ios::binary, mpt::FlushModeFromBool(TrackerSettings::Instance().MiscFlushFileBuffersOnSave));
-	mpt::ofstream& f = sf;
-	if(f.good() && VSTPresets::SaveFile(f, *this, bank))
+	try
 	{
-		return true;
-	} else
+		mpt::SafeOutputFile sf(dlg.GetFirstFile(), std::ios::binary, mpt::FlushModeFromBool(TrackerSettings::Instance().MiscFlushFileBuffersOnSave));
+		mpt::ofstream &f = sf;
+		f.exceptions(f.exceptions() | std::ios::badbit | std::ios::failbit);
+		if(f.good() && VSTPresets::SaveFile(f, *this, isBank))
+			return true;
+	} catch(const std::exception &)
 	{
-		Reporting::Error("Error saving preset.", m_pEditor);
-		return false;
+		
 	}
-
+	Reporting::Error("Error saving preset.", m_pEditor);
+	return false;
 }
 
 
@@ -696,7 +718,7 @@ bool IMixPlugin::LoadProgram(mpt::PathString fileName)
 	}
 
 	const char *errorStr = nullptr;
-	InputFile f(fileName);
+	InputFile f(fileName, SettingCacheCompleteFileBeforeLoading());
 	if(f.IsValid())
 	{
 		FileReader file = GetFileReader(f);
@@ -753,9 +775,21 @@ void IMidiPlugin::ApplyPitchWheelDepth(int32 &value, int8 pwd)
 
 
 // Get the MIDI channel currently associated with a given tracker channel
+uint8 IMidiPlugin::GetMidiChannel(const ModChannel &chn, CHANNELINDEX trackChannel) const
+{
+	if(auto ins = chn.pModInstrument; ins != nullptr)
+		return ins->GetMIDIChannel(chn, trackChannel);
+	else
+		return 0;
+}
+
+
 uint8 IMidiPlugin::GetMidiChannel(CHANNELINDEX trackChannel) const
 {
-	return m_SndFile.GetBestMidiChannel(trackChannel);
+	if(trackChannel < std::size(m_SndFile.m_PlayState.Chn))
+		return GetMidiChannel(m_SndFile.m_PlayState.Chn[trackChannel], trackChannel);
+	else
+		return 0;
 }
 
 
@@ -773,6 +807,13 @@ void IMidiPlugin::MidiCC(MIDIEvents::MidiCC nController, uint8 nParam, CHANNELIN
 }
 
 
+// Set MIDI pitch for given MIDI channel to the specified raw 14-bit position
+void IMidiPlugin::MidiPitchBendRaw(int32 pitchbend, CHANNELINDEX trackerChn)
+{
+	SendMidiPitchBend(GetMidiChannel(trackerChn), EncodePitchBendParam(Clamp(pitchbend, MIDIEvents::pitchBendMin, MIDIEvents::pitchBendMax)));
+}
+
+
 // Bend MIDI pitch for given MIDI channel using fine tracker param (one unit = 1/64th of a note step)
 void IMidiPlugin::MidiPitchBend(int32 increment, int8 pwd, CHANNELINDEX trackerChn)
 {
@@ -788,15 +829,15 @@ void IMidiPlugin::MidiPitchBend(int32 increment, int8 pwd, CHANNELINDEX trackerC
 		ApplyPitchWheelDepth(increment, pwd);
 	}
 
-	int32 newPitchBendPos = (increment + m_MidiCh[midiCh].midiPitchBendPos) & vstPitchBendMask;
+	int32 newPitchBendPos = (increment + m_MidiCh[midiCh].midiPitchBendPos) & kPitchBendMask;
 	Limit(newPitchBendPos, EncodePitchBendParam(MIDIEvents::pitchBendMin), EncodePitchBendParam(MIDIEvents::pitchBendMax));
 
-	MidiPitchBend(midiCh, newPitchBendPos);
+	SendMidiPitchBend(midiCh, newPitchBendPos);
 }
 
 
 // Set MIDI pitch for given MIDI channel using fixed point pitch bend value (converted back to 0-16383 MIDI range)
-void IMidiPlugin::MidiPitchBend(uint8 midiCh, int32 newPitchBendPos)
+void IMidiPlugin::SendMidiPitchBend(uint8 midiCh, int32 newPitchBendPos)
 {
 	MPT_ASSERT(EncodePitchBendParam(MIDIEvents::pitchBendMin) <= newPitchBendPos && newPitchBendPos <= EncodePitchBendParam(MIDIEvents::pitchBendMax));
 	m_MidiCh[midiCh].midiPitchBendPos = newPitchBendPos;
@@ -809,12 +850,12 @@ void IMidiPlugin::MidiVibrato(int32 depth, int8 pwd, CHANNELINDEX trackerChn)
 {
 	auto midiCh = GetMidiChannel(trackerChn);
 	depth = EncodePitchBendParam(depth);
-	if(depth != 0 || (m_MidiCh[midiCh].midiPitchBendPos & vstVibratoFlag))
+	if(depth != 0 || (m_MidiCh[midiCh].midiPitchBendPos & kVibratoFlag))
 	{
 		ApplyPitchWheelDepth(depth, pwd);
 
 		// Temporarily add vibrato offset to current pitch
-		int32 newPitchBendPos = (depth + m_MidiCh[midiCh].midiPitchBendPos) & vstPitchBendMask;
+		int32 newPitchBendPos = (depth + m_MidiCh[midiCh].midiPitchBendPos) & kPitchBendMask;
 		Limit(newPitchBendPos, EncodePitchBendParam(MIDIEvents::pitchBendMin), EncodePitchBendParam(MIDIEvents::pitchBendMax));
 
 		MidiSend(MIDIEvents::PitchBend(midiCh, DecodePitchBendParam(newPitchBendPos)));
@@ -822,14 +863,17 @@ void IMidiPlugin::MidiVibrato(int32 depth, int8 pwd, CHANNELINDEX trackerChn)
 
 	// Update vibrato status
 	if(depth != 0)
-		m_MidiCh[midiCh].midiPitchBendPos |= vstVibratoFlag;
+		m_MidiCh[midiCh].midiPitchBendPos |= kVibratoFlag;
 	else
-		m_MidiCh[midiCh].midiPitchBendPos &= ~vstVibratoFlag;
+		m_MidiCh[midiCh].midiPitchBendPos &= ~kVibratoFlag;
 }
 
 
 void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vol, CHANNELINDEX trackChannel)
 {
+	if(trackChannel >= MAX_CHANNELS)
+		return;
+
 	auto midiCh = GetMidiChannel(trackChannel);
 	PlugInstrChannel &channel = m_MidiCh[midiCh];
 
@@ -846,7 +890,7 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 		uint8 high = static_cast<uint8>(midiBank >> 7);
 		uint8 low = static_cast<uint8>(midiBank & 0x7F);
 
-		//m_SndFile.ProcessMIDIMacro(trackChannel, false, m_SndFile.m_MidiCfg.szMidiGlb[MIDIOUT_BANKSEL], 0, m_nSlot + 1);
+		//m_SndFile.ProcessMIDIMacro(trackChannel, false, m_SndFile.m_MidiCfg.Global[MIDIOUT_BANKSEL], 0, m_nSlot + 1);
 		MidiSend(MIDIEvents::CC(MIDIEvents::MIDICC_BankSelect_Coarse, midiCh, high));
 		MidiSend(MIDIEvents::CC(MIDIEvents::MIDICC_BankSelect_Fine, midiCh, low));
 
@@ -859,7 +903,7 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 	if(progChanged || (midiProg < 0x80 && bankChanged))
 	{
 		channel.currentProgram = midiProg;
-		//m_SndFile.ProcessMIDIMacro(trackChannel, false, m_SndFile.m_MidiCfg.szMidiGlb[MIDIOUT_PROGRAM], 0, m_nSlot + 1);
+		//m_SndFile.ProcessMIDIMacro(trackChannel, false, m_SndFile.m_MidiCfg.Global[MIDIOUT_PROGRAM], 0, m_nSlot + 1);
 		MidiSend(MIDIEvents::ProgramChange(midiCh, midiProg));
 	}
 
@@ -884,7 +928,7 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 		MidiSend(MIDIEvents::CC(MIDIEvents::MIDICC_AllSoundOff, midiCh, 0));
 
 		// Turn off all notes
-		for(uint8 i = 0; i < CountOf(channel.noteOnMap); i++)
+		for(uint8 i = 0; i < std::size(channel.noteOnMap); i++)
 		{
 			channel.noteOnMap[i][trackChannel] = 0;
 			MidiSend(MIDIEvents::NoteOff(midiCh, i, volume));
@@ -896,7 +940,7 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 	// using note mask.
 	else if(note == NOTE_KEYOFF || note == NOTE_FADE) // ==, ~~
 	{
-		for(uint8 i = 0; i < CountOf(channel.noteOnMap); i++)
+		for(uint8 i = 0; i < std::size(channel.noteOnMap); i++)
 		{
 			// Some VSTis need a note off for each instance of a note on, e.g. fabfilter.
 			while(channel.noteOnMap[i][trackChannel])
@@ -908,24 +952,27 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 	}
 
 	// Note On
-	else if(ModCommand::IsNote(static_cast<ModCommand::NOTE>(note)))
+	else if(note >= NOTE_MIN && note < NOTE_MIN + mpt::array_size<decltype(channel.noteOnMap)>::size)
 	{
 		note -= NOTE_MIN;
 
 		// Reset pitch bend on each new note, tracker style.
 		// This is done if the pitch wheel has been moved or there was a vibrato on the previous row (in which case the "vstVibratoFlag" bit of the pitch bend memory is set)
-		if(m_MidiCh[midiCh].midiPitchBendPos != EncodePitchBendParam(MIDIEvents::pitchBendCentre))
+		auto newPitchBendPos = EncodePitchBendParam(Clamp(m_SndFile.m_PlayState.Chn[trackChannel].GetMIDIPitchBend(), MIDIEvents::pitchBendMin, MIDIEvents::pitchBendMax));
+		if(m_MidiCh[midiCh].midiPitchBendPos != newPitchBendPos)
 		{
-			MidiPitchBend(midiCh, EncodePitchBendParam(MIDIEvents::pitchBendCentre));
+			SendMidiPitchBend(midiCh, newPitchBendPos);
 		}
 
 		// count instances of active notes.
 		// This is to send a note off for each instance of a note, for plugs like Fabfilter.
 		// Problem: if a note dies out naturally and we never send a note off, this counter
 		// will block at max until note off. Is this a problem?
-		// Safe to assume we won't need more than 16 note offs max on a given note?
+		// Safe to assume we won't need more than 255 note offs max on a given note?
 		if(channel.noteOnMap[note][trackChannel] < uint8_max)
+		{
 			channel.noteOnMap[note][trackChannel]++;
+		}
 
 		MidiSend(MIDIEvents::NoteOn(midiCh, static_cast<uint8>(note), volume));
 	}
@@ -934,7 +981,7 @@ void IMidiPlugin::MidiCommand(const ModInstrument &instr, uint16 note, uint16 vo
 
 bool IMidiPlugin::IsNotePlaying(uint8 note, CHANNELINDEX trackerChn)
 {
-	if(!ModCommand::IsNote(note) || trackerChn >= CountOf(m_MidiCh[GetMidiChannel(trackerChn)].noteOnMap[note]))
+	if(!ModCommand::IsNote(note) || trackerChn >= std::size(m_MidiCh[GetMidiChannel(trackerChn)].noteOnMap[note]))
 		return false;
 
 	note -= NOTE_MIN;
